package fiber;



import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public class DeadlockDetectFeature extends IFeature {



    private static final int ACQUIRE_COUNT = 3;
    private static final int LOCK_COUNT = 10;
    private static final long SLEEP_TIME = 200L;

    private static Object lockA;
    private static Object lockB;
    private static Object lockC;
    private static Object lockD;
    private static Object lockE;
    private static Object lockF;
    private static Object lockG;
    private static Object lockH;
    private static Object lockI;
    private static Object lockJ;
    private static Object finishedCounterLock;
    private static int finishedCounter;

    private int[] acquireIndexArray;



    public DeadlockDetectFeature() {
        super("Finding monitor deadlocks when threads randomly acquires multiple monitors, "
              + "to solve deadlock, some processes must be terminated");
    }



    private Object getLock(int lockIndex) {
        switch (lockIndex) {
            case 0:
                return lockA;
            case 1:
                return lockB;
            case 2:
                return lockC;
            case 3:
                return lockD;
            case 4:
                return lockE;
            case 5:
                return lockF;
            case 6:
                return lockG;
            case 7:
                return lockH;
            case 8:
                return lockI;
            case 9:
            default:
                return lockJ;
        }
    }



    private void acquireLock(int acquireIndex) throws Exception {
        Object lock = getLock(acquireIndexArray[acquireIndex]);
        Thread.sleep(SLEEP_TIME);
        println("Acquiring " + lock + " ...");
        synchronized (lock) {
            println("Acquired " + lock);
            Thread.sleep(SLEEP_TIME);
            acquireIndex++;
            if (acquireIndex < ACQUIRE_COUNT) {
                acquireLock(acquireIndex);
            } else {
                println("Reached synchronization summit");
                synchronized (finishedCounterLock) {
                    finishedCounter++;
                }
                Thread.sleep(SLEEP_TIME);
            }
        }
        println("Released " + lock);
        Thread.sleep(SLEEP_TIME);
    }



    @Override
    protected void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        String lockOrder = "";
        acquireIndexArray = new int[ACQUIRE_COUNT];
        for (int i = 0; i < ACQUIRE_COUNT; i++) {
            acquireIndexArray[i] = (int) (Math.random() * LOCK_COUNT);
            if (i != 0) {
                lockOrder += ", ";
            }
            lockOrder += getLock(acquireIndexArray[i]).toString();
        }
        println("Lock order: " + lockOrder);
        acquireLock(0);
    }



    @Override
    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        lockA = new LockWithName("lock_A");
        lockB = new LockWithName("lock_B");
        lockC = new LockWithName("lock_C");
        lockD = new LockWithName("lock_D");
        lockE = new LockWithName("lock_E");
        lockF = new LockWithName("lock_F");
        lockG = new LockWithName("lock_G");
        lockH = new LockWithName("lock_H");
        lockI = new LockWithName("lock_I");
        lockJ = new LockWithName("lock_J");
        finishedCounterLock = new Object();
        finishedCounter = 0;
        println("Finished counter value is " + finishedCounter);
    }



    @Override
    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Finished counter value is " + finishedCounter + " (expected " + getWorkerCount() + ")");
    }
}
