package fiber;



import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public class PiMultiFeature extends IFeature {



    private static final int ITERATION_COUNT = 1000000;
    private static final int WORK_QUANTUM = 10000;

    private static Object lock;
    private static double sharedPi;
    private static int nextIndex;
    private static int lastProgress;



    public PiMultiFeature() {
        super("Calculating PI in separate threads with paralel cooperation");
    }



    private double piSegment(int index) {
        double segment = 4.0 / (2 * index + 1);
        if (index % 2 != 0) {
            segment = -segment;
        }
        return segment;
    }



    @Override
    protected void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Calculating PI...");
        int printScale = (ITERATION_COUNT / 100);
        int index;
        int progress;
        int iterationCount = 0;
        double pi = 0.0;
        while (true) {
            synchronized (lock) {
                sharedPi += pi;
                index = nextIndex;
                if (index >= ITERATION_COUNT) {
                    println("Calculating finished with " + iterationCount + " iterations");
                    return;
                }
                nextIndex += WORK_QUANTUM;
                progress = index / printScale;
                if (lastProgress < progress) {
                    lastProgress = progress;
                } else {
                    progress = -1;
                }
            }
            if (progress != -1) {
                println("Progress " + progress + "%");
            }
            pi = 0.0;
            for (int i = index; (i < index + WORK_QUANTUM) && (i < ITERATION_COUNT); i++) {
                pi += piSegment(i);
                iterationCount++;
            }
        }
    }



    @Override
    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Calculating PI with " + ITERATION_COUNT + " iterations");
        lock = new Object();
        sharedPi = 0.0;
        nextIndex = 0;
        lastProgress = 0;
    }



    @Override
    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        formatln("PI = %.20f", sharedPi);
    }
}
