package fiberping;



import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import san.application.IApplication;
import san.core.DestinationNotAvailableException;
import san.core.IApplicationAPI;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.Guid;
import san.ident.NetIdentifier;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberPing implements IApplication, ReceiveDataListener {



    private static int conditionId;
    private static TransmitData transmitData;

    private PrintWriter out;



    @Override
    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        out = applicationAPI.getStdOut();
        out.println("+------------+");
        out.println("| Fiber ping |");
        out.println("+------------+");
        out.println();
        if (args.length != 2) {
            out.println("Usage:");
            out.println("    > run fiberping <targetAddress> <targetNetwork>");
            out.println();
            return;
        }
        Guid targetAddress;
        Guid targetNetwork;
        try {
            targetAddress = Guid.parseGuid(args[0]);
            targetNetwork = Guid.parseGuid(args[1]);
        } catch (Exception e) {
            out.println("Target address can not be resolved");
            out.println();
            return;
        }
        long sendTime = System.currentTimeMillis();
        long targetTime;
        long receiveTime;
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream;
        try {
            objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
            objectOutputStream.writeBoolean(false);
            objectOutputStream.writeLong(sendTime);
            objectOutputStream.close();
        } catch (IOException e) {
            return;
        }
        conditionId = guid.hashCode();
        transmitData = null;
        applicationAPI.addOnSendDataListener(guid);
        applicationAPI.conditionInitialize(conditionId);
        try {
            applicationAPI.injectCapsule(applicationAPI.getExecutedCodeIdent(),
                                         new NetIdentifier(targetAddress, targetNetwork),
                                         new TransmitData(byteArrayOutputStream.toByteArray()));
        } catch (DestinationNotAvailableException e) {
            out.println("Target address can not be resolved");
            out.println();
            return;
        }
        if (!applicationAPI.conditionWait(conditionId, 10000)) {
            out.println("Response timeout exceeded");
            out.println();
            return;
        }
        if (transmitData == null) {
            out.println("No data received from capsule");
            out.println();
            return;
        }
        ByteArrayInputStream arrayInputStream = new ByteArrayInputStream(transmitData.getData());
        ObjectInputStream objectInputStream;
        try {
            objectInputStream = new ObjectInputStream(arrayInputStream);
            objectInputStream.readBoolean();
            objectInputStream.readLong();
            targetTime = objectInputStream.readLong();
        } catch (IOException e) {
            return;
        }
        receiveTime = System.currentTimeMillis();
        out.println("From local node to target node: " + (targetTime - sendTime) + " ms");
        out.println("From target node to local node: " + (receiveTime - targetTime) + " ms");
        out.println("Overall time:                   " + (receiveTime - sendTime) + " ms");
        out.println();
    }



    @Override
    public void receiveData(Guid sender, TransmitData data, IApplicationAPI applicationAPI) {
        transmitData = data;
        applicationAPI.conditionSignalOne(conditionId);
    }
}
