package netAnt;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.SortedSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import san.application.IApplication;
import san.core.IApplicationAPI;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.routing.RoutingRecord;
import san.routing.RoutingTable;
import san.networkInterfaces.NetInterface;
import java.util.Arrays;
import java.io.PrintWriter;

public class NetAnt implements IApplication, ReceiveDataListener {
    
    // pocet vyslanych kapsuli
    private final int CAPSULES_COUNT = 1;
    private final int ADDRESS_RECORD_LENGTH = Guid.LENGHT;
    private final int NETWORK_RECORD_LENGTH = Guid.LENGHT;
    private Random generator;

    private TransmitData receivedData = null;
    private CodePkgIdent capsulePkgIdent;
    private TransmitData data;
    // input settings
    private ByteArrayInputStream bais;
    private ObjectInputStream ois;
    // output settings
    private ByteArrayOutputStream bos;
    private ObjectOutputStream oos;

    //vystupni stream
    private PrintWriter out;

    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        out = applicationAPI.getStdOut();

        //prvotni kontrolni vypis routovaci tabulky
        out.println("...vychozi stav smerovaci tabulky...");
        vypisRoutovaciTabulky(applicationAPI.getServerControl().getRoutingTable());
        out.println("\n");



        for(int i = 0; i < CAPSULES_COUNT; i++){
            applicationAPI.addOnSendDataListener(guid);

            boolean owb;    // onWayBack
            int fwrc;       // ForwardRecordCount
            int bwrc;       // BackwardRecordCount
            byte[] destinationNode = new byte[ADDRESS_RECORD_LENGTH + NETWORK_RECORD_LENGTH];

            // vyslani jedny kapsule do site
            if (!sendCapsule(applicationAPI)) {
                System.out.println("chyba pri odesilani kapsule");
                return;
            }

            waitUntilArrivalOfCapsule();
            
            // naplnena kapsule se vratila
            if (receivedData != null) {
                try {
                    bais = new ByteArrayInputStream(receivedData.getData());
                    ois = new ObjectInputStream(bais);

                    //precteni hodnot ze zacatku kapsule
                    owb = ois.readBoolean();
                    fwrc = ois.readInt();
                    bwrc = ois.readInt();
                    ois.read(destinationNode);

//                    System.out.println("Na ceste zpet: " + owb);
//                    System.out.println("Pocet doprednych zaznamu(FWRecordCount): " + fwrc +
//                    "  Pocet nasbiranych smerovacich zaznamu(BWRecordCount): " + bwrc);

                    parsingReceivedData(applicationAPI, bwrc);
                    ois.close();
                    bais.close();

                } catch (IOException ex) {
                    System.out.println("!!! VYJIMKA PRI ZPRACOVAVANI PRIJATE KAPSULE !!!");
                }
            }
//            System.out.println("KONEC VYSILANI " + (i+1) + ".KAPSULE");
        }
        // konec aplikace
        return;
    }

    // vyslani kapsule do site
    private boolean sendCapsule(IApplicationAPI applicationAPI) {

        NetIdentifier one = selectOneHopAddress(applicationAPI);
//        System.out.println("SKOK NA " + one);
        
//        NetIdentifier destination = getRandomNeighbor(applicationAPI);
        NetIdentifier destination = getRandomDestination(applicationAPI);
//        System.out.println("Cilovy uzel: " + destination);

        if ((destination == null)||(one == null)) {
            System.out.println("!!! CHYBA ODESILANI, NENI KE ME NIKDO PRIPOJEN !!!");
            return false;
        }

        try {
            bos = new ByteArrayOutputStream();
            oos = new ObjectOutputStream(bos);

            // boolean jestli je kapsle na zpatecni ceste
            oos.writeBoolean(false);
            // pocet zaznamu s adresami navstivenych uzlu
            oos.writeInt(1);
            // pocet skoku pri ceste zpet
            oos.writeInt(0);

            //zapis cilove adresy do kapsule
            writeNetIdentifierToCapsule(destination);

            // zapis moji adresy do kapsule, kvuli navratu
            NetIdentifier ni = getMyAddress(applicationAPI, one);
            writeNetIdentifierToCapsule(ni);

            oos.close();

            // obaleni posilanych dat datovou strukturou
            data = new TransmitData(bos.toByteArray());

            // pripraveni indentifikatoru zdrojove aplikace
            capsulePkgIdent = applicationAPI.getExecutedCodeIdent();

            // vkladani kapsule do site, predavam indentifikator zdrojove aplikace, cilovou adresu a data
            applicationAPI.injectCapsule(capsulePkgIdent, one, data);
//            System.out.println("KAPSLE ODESLANA");

        } catch (Exception e) {
            System.out.println("!!! VYJIMKA PRI ODESILANI KAPSULE !!!");
            return false;
        }
        return true;
    }

    // zvoleni nahodneho ciloveho uzlu, na ktery bude kapsule naadresovana
    private NetIdentifier getRandomDestination(IApplicationAPI applicationAPI) {
        RoutingTable routingTable = applicationAPI.getServerControl().getRoutingTable();
        
        //vypisRoutovaciTabulky(routingTable);
        Collection<SortedSet<RoutingRecord>> col = routingTable.getRoutingRecords().values();

        if (col != null) {
//            System.out.println("Velikost routovaci tabulky pri vyberu nahodneho cile: " + col.size());
            generator = new Random();

            int index = generator.nextInt(col.size());
//            System.out.println("Nahodne vybrany index: " + index);

            int i = 0;
            for (Iterator<SortedSet<RoutingRecord>> it = col.iterator(); it.hasNext();) {
                SortedSet<RoutingRecord> sortedSet = it.next();

                if (i == index) {
                    for (Iterator<RoutingRecord> it1 = sortedSet.iterator(); it1.hasNext();) {
                        RoutingRecord rr = it1.next();
//                        System.out.println("RANDOM DESTINATION: " + rr.getDestination());
                        return rr.getDestination();
                    }
                }
                i++;
            }
        }
        return null;
    }

    // zvoleni nahodneho primeho souseda, kteremu bude predana kapsule
    private NetIdentifier selectOneHopAddress(IApplicationAPI applicationAPI) {
        RoutingTable routingTable = applicationAPI.getServerControl().getRoutingTable();
        Collection<RoutingRecord> directNeighbors = routingTable.getDirectNeighbours();

        if (directNeighbors != null) {
            generator = new Random();

            int index = generator.nextInt(directNeighbors.size());
//            System.out.println("Nahodne vybrany index: " + index);
            int i = 0;

            for (Iterator<RoutingRecord> it = directNeighbors.iterator(); it.hasNext();) {
                RoutingRecord rr = it.next();
            
                if (i == index) {
//                    System.out.println("DALSI SKOK NA: " + rr.getDestination());
                    return rr.getDestination();
                }
                i++;
            }
        }
        return null;
    }


    // ziskani sitoveho identifikatoru meho rozhranni podle identifikatoru ciloveho rozhranni
    private NetIdentifier getMyAddress(IApplicationAPI applicationAPI, NetIdentifier destination) {
//        System.out.println("CIL KAPSLE: " + destination);
        NetIdentifier s = applicationAPI.getServerControl().getRoutingTable().getRoute(destination).getNetInterface();
//        System.out.println("ZISKANA ADRESA MEHO ROZHRANNI: " + s);
        return s;
    }

    // metoda pro zapis sitoveho identifikatoru uzlu do kapsule
    private void writeNetIdentifierToCapsule(NetIdentifier ni) throws IOException {
        if (ni != null) {
            oos.write(ni.getAddress().toByte());
            oos.write(ni.getNetwork().toByte());
//            System.out.println("Moje adresa byla uspesne zapsana do kapsule");
        } else {
            System.out.println("!!! CHYBA, PRI ZAPISOVANI ADRESY DO KAPSULE, " +
                    "PROTOZE SITOVY IDENTIFIKATOR BYL NULL !!!");
        }
    }

    // cekani na data z prichozi kapsule
    private void waitUntilArrivalOfCapsule() {
        while (this.receivedData == null) {
            try {
                Thread.sleep(1L);
            } catch (InterruptedException ex) {
                Logger.getLogger(NetAnt.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    // metoda pro naparsovani casti smerovacich zaznamu z prijate kapsule
    private void parsingReceivedData(IApplicationAPI applicationAPI, int BWRecordCount) throws IOException {
        // pole pro polozky routovaciho zaznamu
        byte[] sourceAddress;
        byte[] sourceNetwork;
        byte[] neighboorAddress;
        byte[] neighboorNetwork;
        byte distance;

        // smycka pres vsechny routovaci zaznamy v prijate kapsuli
        for (int j = 0; j < BWRecordCount; j++) {

            sourceAddress = new byte[ADDRESS_RECORD_LENGTH];
            sourceNetwork = new byte[NETWORK_RECORD_LENGTH];
            neighboorAddress = new byte[ADDRESS_RECORD_LENGTH];
            neighboorNetwork = new byte[NETWORK_RECORD_LENGTH];

            ois.read(sourceAddress);
            ois.read(sourceNetwork);
            ois.read(neighboorAddress);
            ois.read(neighboorNetwork);
            distance = ois.readByte();

//            System.out.println("zpracovavany zaznam: sa:" + Arrays.toString(sourceAddress)
//                    + "  sn: " + Arrays.toString(sourceNetwork)
//                    + "  da: " + Arrays.toString(neighboorAddress)
//                    + "  dn: " + Arrays.toString(neighboorNetwork)
//                    + " dist: " + distance);

            processingReceivedRoutingData(applicationAPI, sourceAddress, sourceNetwork, neighboorAddress,
                    neighboorNetwork, distance);
        }
        
        //kontrolni vypis routovaci tabulky po aktualizaci vsemi daty z jedne prichozi kapsule
        out.println("...aktualizovany stav smerovaci tabulky...");
        vypisRoutovaciTabulky(applicationAPI.getServerControl().getRoutingTable());
    }

    // zpracovani naparsovanych casti routovaciho zaznamu
    private void processingReceivedRoutingData(IApplicationAPI applicationAPI, byte[] sourceAddress, byte[] sourceNetwork,
            byte[] neighboorAddress, byte[] neighboorNetwork, byte distance) {

        RoutingRecord tmprr;    // temporary routing record
        NetIdentifier sni;      // source network identifier
        NetIdentifier nni;      // neighboor network identifier

        sni = new NetIdentifier(new Guid(sourceAddress), new Guid(sourceNetwork));
        nni = new NetIdentifier(new Guid(neighboorAddress), new Guid(neighboorNetwork));


        if (!isThisServer(applicationAPI, sni)){
            tmprr = new RoutingRecord();
            tmprr.setNetInterface(getMyAddress(applicationAPI, sni));
            tmprr.setDestination(nni);
            tmprr.setGateway(sni);
            tmprr.setMetric(distance + 1);

//            System.out.println("Vypis zpracovaneho routovaciho zaznamu: sni=" + tmprr.getNetInterface() + "  nni=" + tmprr.getDestination() + "  gni=" + tmprr.getGateway() + "  distance=" + tmprr.getMetric());

            // separovani pouze spravnych zaznamu
            if (!isThisServer(applicationAPI, tmprr.getDestination())) {
    //            System.out.println("DOBRY ZAZNAM, ukladat");
                updateRoutingTable(applicationAPI, tmprr);
                //kontrolni vypis routovaci tabulky kazde aktualizaci
                //vypisRoutovaciTabulky(applicationAPI.getServerControl().getRoutingTable());
            }
        }
    }

    // testovani zda dany sitovy identifikator patri tomuto uzlu ci nikoliv
    private boolean isThisServer(IApplicationAPI applicationAPI, NetIdentifier ni) {
        Map<NetIdentifier, NetInterface> myInterfaces = applicationAPI.getServerControl().getNetInterfaces().getNetInterfaces();
//        System.out.println("POCET MYCH ROZHRANNI == " + myInterfaces.size());

        Set<NetIdentifier> col = myInterfaces.keySet();
        return col.contains(ni);
    }

    // metoda pro aktualizaci smerovaci tabulky konkretnim zaznamem
    private void updateRoutingTable(IApplicationAPI applicationAPI, RoutingRecord newRecord) {
        RoutingTable routingTable = applicationAPI.getServerControl().getRoutingTable();
        Map<NetIdentifier, SortedSet<RoutingRecord>> map = routingTable.getRoutingRecords();
        Collection<SortedSet<RoutingRecord>> routingRecordsCollection = map.values();

        boolean replaced = false;

        if (!map.isEmpty()) {
            // update
            for (Iterator<SortedSet<RoutingRecord>> it = routingRecordsCollection.iterator(); it.hasNext();) {
                SortedSet<RoutingRecord> sortedSet = it.next();

                // cyklus pres vsechny routovaci zaznamy
                for (Iterator<RoutingRecord> it1 = sortedSet.iterator(); it1.hasNext();) {
                    RoutingRecord rr = it1.next();

                    // hledam zaznam se shodnou zdrojovou a cilovou adresou, u kteryho bych pripadne upravoval branu
                    if ((rr.getNetInterface().equals(newRecord.getNetInterface())) && (rr.getDestination().equals(newRecord.getDestination()))) {
//                        System.out.print("NALEZEN SHODNY ZAZNAM.....");

                        if (rr.getMetric() > newRecord.getMetric()) {
//                            System.out.println("NOVA DISTANCE (" + newRecord.getMetric() + ") JE MENSI NEZ BYLA TA PUVODNI (" + rr.getMetric() + ")");
//                            System.out.println("UPDATE routovaciho zaznamu");
                            rr.setGateway(newRecord.getGateway());
                            rr.setMetric(newRecord.getMetric());
                        } else {
//                            System.out.println("ALE HORSI nebo STEJNA DISTANCE, TAKZE NEUKLADAM");
                        }
                        replaced = true;
                    }
                }
            }
        }
        if (!replaced) {
            // insert
//            System.out.println("TABULKA JE UPLNE PRAZDNA nebo V NI NEBYL ZAZNAM NALEZEN => INSERT");
            RoutingRecord rrr = new RoutingRecord();
            rrr.setMetric(newRecord.getMetric());
            rrr.setDestination(newRecord.getDestination());
            rrr.setGateway(newRecord.getGateway());
            rrr.setNetInterface(newRecord.getNetInterface());
            routingTable.add(rrr);
        }
    }

    // kontrolni vypis obsahu routovaci tabulky
    private void vypisRoutovaciTabulky(RoutingTable rt) {
        Map<NetIdentifier, SortedSet<RoutingRecord>> map = rt.getRoutingRecords();
        Collection<SortedSet<RoutingRecord>> col = map.values();

        int cislovac = 0;
        for (Iterator<SortedSet<RoutingRecord>> it = col.iterator(); it.hasNext();) {
            SortedSet<RoutingRecord> sortedSet = it.next();

            for (Iterator<RoutingRecord> it1 = sortedSet.iterator(); it1.hasNext();) {
                RoutingRecord rr = it1.next();

                out.println(++cislovac + ". NIC: " + rr.getNetInterface() + "   DST:" + rr.getDestination() + "   GW:" + rr.getGateway() + "   DIS:" + rr.getMetric());
            }
        }
        out.println("DELKA ROUTOVACI TABULKY: " + cislovac);
    }

    // metoda pro prijem dat
    public void receiveData(Guid sender, TransmitData data, IApplicationAPI applicationAPI) {
        this.receivedData = data;
//        System.out.println("-----------------------------------------");
//        System.out.println("--------- D A T A   P R I S L Y ---------");
//        System.out.println("-----------------------------------------");
    }
}
