package netAnt;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.transform.Source;
import san.application.ICapsule;
import san.core.ICapsuleAPI;
import san.data.TransmitData;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.logging.ILogger;
import san.logging.LoggerFactory;
import san.ident.NetIdentifier;
import san.routing.RoutingRecord;
import san.networkInterfaces.NetInterface;

public class NetAntCapsule implements ICapsule {

    private final int ADDRESS_LENGTH = Guid.LENGHT;
    private final int NETWORK_LENGTH = Guid.LENGHT;
    private TransmitData data;
    private boolean onTheWayBack;
    private int FwRecordCount, BwRecordCount;
    // pole sitovych identifikatoru navstivenych uzlu
    private NetIdentifier visitedNodes[];
    // pole bajtu, tvorici jeden konkretni adresni zaznam
    private byte visitedNodeAddressAsBytes[];
    // pole bajtu, tvorici jeden konkretni zaznam o siti
    private byte visitedNodeNetworkAsBytes[];
    private NetIdentifier destinationNode;
    private Random generator;
    // input settings
    private ByteArrayInputStream bais;
    private ObjectInputStream ois;
    // output settings
    private ByteArrayOutputStream bos;
    private ObjectOutputStream oos;

    public void main(ICapsuleAPI capsuleAPI) {
        try {
            data = capsuleAPI.getDataLoad();
            bais = new ByteArrayInputStream(data.getData());
            ois = new ObjectInputStream(bais);

            onTheWayBack = ois.readBoolean();
            FwRecordCount = ois.readInt();
            BwRecordCount = ois.readInt();
//            System.out.println("FW: " + FwRecordCount + "BW: " + BwRecordCount);

            destinationNode = readDestionationNode();
//            System.out.println("NACTENY CIL KAPSULE: " + destinationNode);


            // test zda je prijata kapsule pro me
            if (capsuleAPI.isThisServer(destinationNode)) {
//                System.out.println("ANO, KAPSULE JE PRO MNE");

                // kapsule je doma
                if (onTheWayBack) {
//                  System.out.println("KAPSULE JDE ZPATKY a JE DOMA");

                    ois.close();
                    bais.close();

                    // prectu data z kapsule a predam je zdrojove aplikaci
                    Guid sourceApp = capsuleAPI.getSourceApplication();
                    capsuleAPI.sendData(sourceApp, data);
                    capsuleAPI.finished();
                } 
                // kapsule je v cily
                else {
//                    System.out.println("KAPSULE JDE TAM a JE V CILY");

                    // cteni doprednych zaznamu
                    readForwardRecords();

                    // otoceni hlavicky kapsule, ze ji nyni odesilam ja
                    destinationNode = capsuleAPI.getSource();
                    capsuleAPI.setSource(capsuleAPI.getDestination());
                    capsuleAPI.setDestination(getNextBackwardAddress());
//                    System.out.println("Novy zdroj kapsule: " + capsuleAPI.getSource());
//                    System.out.println("Novy cil kapsule: " + capsuleAPI.getDestination());

                    // metoda pro shromazdovani routovacich zaznamu a odeslani kapsle
                    backwardAntProcessing(capsuleAPI);
                }
            } else {
                // mezilehly uzel na zpatecni ceste
                if (onTheWayBack){
//                    System.out.println("KAPSULE JDE ZPATKY, ale JESTE NENI DOMA");

                    // cteni doprednych zaznamu
                    readForwardRecords();

                    // nastaveni nove cilove adresy pro kapsli
                    capsuleAPI.setDestination(getNextBackwardAddress());
//                    System.out.println("Novy cil kapsule: " + capsuleAPI.getDestination());

                    // metoda pro shromazdovani routovacich zaznamu a odeslani kapsle
                    backwardAntProcessing(capsuleAPI);
                }
                // mezilehly uzel na ceste k cily
                else {
//                    System.out.println("NE, KAPSULE NENI PRO MNE - JDE K CILY a JEN PROCHAZI");

                    // cteni doprednych zaznamu
                    readForwardRecords();

                    // nastaveni cilove adresy na uzel o skok dale
                    capsuleAPI.setDestination(selectOneHopAddress(capsuleAPI));

                    // metoda pro shromazdovani routovacich zaznamu a odeslani kapsle
                    forwardAntProcessing(capsuleAPI);
                }
            }
        } catch (Exception e) {
            LoggerFactory.createLogger().Log(e, ILogger.LogType.ERROR);
        }
    }

    // precteni sitoveho identifikatoru ciloveho uzlu, ze zacatku kapsule
    private NetIdentifier readDestionationNode() throws IOException {
        byte[] address = new byte[ADDRESS_LENGTH];
        byte[] network = new byte[NETWORK_LENGTH];
        ois.read(address);
        ois.read(network);

        return new NetIdentifier(new Guid(address), new Guid(network));
    }

    // metoda pro nacteni zaznamu o jiz navstivenych uzlech
    private void readForwardRecords() throws IOException {
        visitedNodes = new NetIdentifier[FwRecordCount];

        // cyklus pro nacitani vsech doprednych adresnich zaznamu z kapsle
        for (int i = 0; i < FwRecordCount; i++) {
            visitedNodeAddressAsBytes = new byte[ADDRESS_LENGTH];
            visitedNodeNetworkAsBytes = new byte[NETWORK_LENGTH];

            // precteni adresy konkretniho navstiveneho uzlu
            ois.read(visitedNodeAddressAsBytes);
            // precteni site konkretniho navstiveho uzlu
            ois.read(visitedNodeNetworkAsBytes);

            // ulozeni zaznamu
            visitedNodes[i] = new NetIdentifier(new Guid(visitedNodeAddressAsBytes), new Guid(visitedNodeNetworkAsBytes));
//            System.out.println("Nacteny dopredny zaznam: " + visitedNodes[i]);
        }
    }

    // vybrani nove cilove adresy z kapsule, vybrani zaznamu o poslednim navstivenem uzlu
    private NetIdentifier getNextBackwardAddress() {
        int newLength = visitedNodes.length - 1;
        NetIdentifier[] newVisitedNodes = new NetIdentifier[newLength];
        for (int i = 0; i < newLength; i++) {
            newVisitedNodes[i] = visitedNodes[i];
        }

        NetIdentifier nextBackwardAddress = visitedNodes[visitedNodes.length - 1];
        visitedNodes = newVisitedNodes;
        FwRecordCount = newLength;
        return nextBackwardAddress;
    }

    // metoda pro zpracovani vracejici se kapsule
    private void backwardAntProcessing(ICapsuleAPI capsuleAPI) throws Exception {
        bos = new ByteArrayOutputStream();
        oos = new ObjectOutputStream(bos);

        ByteArrayOutputStream tmpBaos = new ByteArrayOutputStream();

        // zpracovani smerovacich zaznamu
        List<MyRoutingRecord> records = mergeNeighboors(capsuleAPI, tmpBaos);


        // kapsule je od ted na ceste zpet
        oos.writeBoolean(true);

        // zapsani poctu navstivenych uzlu
        oos.writeInt(FwRecordCount);
        oos.writeInt(BwRecordCount);

        //zapsani sitoveho identifikatoru ciloveho uzlu
        writeNetIdentifierToCapsule(destinationNode);

        // zapis doprednych zaznamu, pro cestu zpet
        for (int i = 0; i < FwRecordCount; i++) {
            writeNetIdentifierToCapsule(visitedNodes[i]);
//            System.out.println(visitedNodes[i]);
        }

        // zapisovani routovacich zaznamu
        for (Iterator<MyRoutingRecord> it = records.iterator(); it.hasNext();) {
            MyRoutingRecord mrr = it.next();
//            System.out.println("ZAPISOVANY RR= "
//                    + "SA: "+ Arrays.toString(mrr.getSrcAddress())
//                    + "SN: "+ Arrays.toString(mrr.getSrcNetwork())
//                    + "DA: "+ Arrays.toString(mrr.getDstAddress())
//                    + "DN: "+ Arrays.toString(mrr.getDstNetwork())
//                    + "Dist: "+ mrr.getDistance());
            writeMyRoutingRecordToCapsule(mrr);
        }

        oos.close();
        data = new TransmitData(bos.toByteArray());
        
//        System.out.println("ODESLANI KAPSULE");
        capsuleAPI.setDataLoad(data);
    }

    // metoda na spojeni smerovacich zaznamu z kapsule s temi z uzlu
    private List<MyRoutingRecord> mergeNeighboors(ICapsuleAPI capsuleAPI, ByteArrayOutputStream tmpBaos) throws IOException {

        // zpracovani smerovacich zaznamu z kapsule a jejich ulozeni do seznamu
        List<MyRoutingRecord> records = processingReceivedRoutingRecord(capsuleAPI, tmpBaos);
        MyRoutingRecord mrr;
        Byte distanceToNeighboor = new Byte("1");

        // ziskani smerovacich zaznamu z lokalni smerovaci tabulky
        Collection<RoutingRecord> directNeighbors = capsuleAPI.getRoutingModification().getDirectNeighbours();

	// ulozeni lokalnich smerovacich zaznamu na konec seznamu zpracovanych smerovacich zaznamu z kapsule
        for (Iterator<RoutingRecord> it = directNeighbors.iterator(); it.hasNext();) {
            RoutingRecord rr = it.next();
//            System.out.println("Sousedi:  SA" + rr.getNetInterface() + "  NA" + rr.getDestination() + "  D = 1");

            NetIdentifier source = getMyAddress(capsuleAPI);
            mrr = new MyRoutingRecord(source.getAddress().toByte(),
                    source.getNetwork().toByte(),
                    rr.getDestination().getAddress().toByte(),
                    rr.getDestination().getNetwork().toByte(),
                    distanceToNeighboor.byteValue());

            records.add(mrr);

            // zvyseni citace poctu routovaci zaznamu
            BwRecordCount++;
        }
        tmpBaos.close();

        
	// testovani spojenych smerovacich zaznamu a odstraneni tech zbytecnych, pred zapisem do kapsule
        // nutno dospsat, zatim jsou ukladany i zbytecne zaznamy
        for (Iterator<MyRoutingRecord> it = records.iterator(); it.hasNext();) {
            MyRoutingRecord mrrr = it.next();

//            System.out.println(Arrays.toString(mrrr.getSrcAddress()));
//            System.out.println(Arrays.toString(mrrr.getSrcNetwork()));
//            System.out.println();
//            System.out.println(Arrays.toString(mrrr.getDstAddress()));
//            System.out.println(Arrays.toString(mrrr.getDstNetwork()));
//            System.out.println();
//            System.out.println();

//            if (Arrays.equals(mrrr.getSrcAddress(), mrrr.getDstAddress()) &&
//               (Arrays.equals(mrrr.getSrcNetwork(), mrrr.getDstNetwork()))){
//
//                System.out.println("NALEZEN SPATNY NEBO ZBYTECNY ZAZNAM");
//                boolean remove = records.remove(mrrr);
//                System.out.println("Odebrani zaznamu = " + remove);
//
//                BwRecordCount--;
//            }
        }
        return records;
    }

    // zpracovani smerovacich zaznamu z prijate kapsule
    private List<MyRoutingRecord> processingReceivedRoutingRecord(ICapsuleAPI capsuleAPI, ByteArrayOutputStream tmpOos) throws IOException {
//        System.out.println("Upravovani routovacich zaznamu, ktere jiz byli v kapsli");
        List<MyRoutingRecord> records = readReceivedRoutingRecord();
        NetIdentifier ni;
        for (Iterator<MyRoutingRecord> it = records.iterator(); it.hasNext();) {
            MyRoutingRecord mrr = it.next();

            ni = getMyAddress(capsuleAPI);
//            mrr.setDstAddress(ni.getAddress().toByte());
//            mrr.setDstNetwork(ni.getNetwork().toByte());
            mrr.setSrcAddress(ni.getAddress().toByte());
            mrr.setSrcNetwork(ni.getNetwork().toByte());

//            System.out.println("PUVODNI DISTANCE: " + mrr.getDistance());
            int a = (int) mrr.getDistance() + 1;
//            System.out.println("UPRAVENA DISTANCE: " + a);
            mrr.setDistance(new Integer(a).byteValue());

//            System.out.println("UPRAVENY PREJATY RR = SA:" + Arrays.toString(mrr.getSrcAddress()) + " SN:" + Arrays.toString(mrr.getSrcNetwork()) + " DA:" + Arrays.toString(mrr.getDstAddress()) + " DN:" + Arrays.toString(mrr.getDstNetwork()) + " DIST:" + mrr.getDistance());
        }
        return records;
    }

    // nacteni smerovacich zaznamu z prijate kapsule
    private List<MyRoutingRecord> readReceivedRoutingRecord() throws IOException {
        List<MyRoutingRecord> records = new ArrayList<MyRoutingRecord>();

        byte[] srcAddress;
        byte[] srcNetwork;
        byte[] dstAddress;
        byte[] dstNetwork;
        Byte distanceLocal;

        MyRoutingRecord tmp;
//        System.out.println("MUSIM JICH PRECIST: " + BwRecordCount);
        int konec = BwRecordCount;
        for (int i = 0; i < konec; i++) {
            srcAddress = new byte[ADDRESS_LENGTH];
            srcNetwork = new byte[NETWORK_LENGTH];
            dstAddress = new byte[ADDRESS_LENGTH];
            dstNetwork = new byte[NETWORK_LENGTH];

            // nacteni jednoho routovaci zaznamu
            ois.read(srcAddress);
//            System.out.println(Arrays.toString(srcAddress));
            ois.read(srcNetwork);
//            System.out.println(Arrays.toString(srcNetwork));
            ois.read(dstAddress);
//            System.out.println(Arrays.toString(dstAddress));
            ois.read(dstNetwork);
//            System.out.println(Arrays.toString(dstNetwork));
            distanceLocal = new Byte(ois.readByte());
//            System.out.println("DISTANCE_LOCAL: " + distanceLocal.byteValue());

            tmp = new MyRoutingRecord(srcAddress, srcNetwork, dstAddress, dstNetwork, distanceLocal.byteValue());
//            System.out.println("NACITANY PREJATY RR = SA:" + Arrays.toString(tmp.getSrcAddress()) + " SN:" + Arrays.toString(tmp.getSrcNetwork()) + " DA:" + Arrays.toString(tmp.getDstAddress()) + " DN:" + Arrays.toString(tmp.getDstNetwork()) + " DIST:" + tmp.getDistance());
            records.add(tmp);
        }
        return records;
    }

    // ziskani sitoveho identifikatoru meho rozhranni podle rozhranni ciloveho uzlu kapsule
    private NetIdentifier getMyAddress(ICapsuleAPI capsuleAPI) {
        NetIdentifier s = capsuleAPI.getServerControl().getRoutingTable().getRoute(capsuleAPI.getDestination()).getNetInterface();
//        System.out.println("MOJE ZDROJOVA ADRESA: " + s);
        return s;
    }

    // zapis sitoveho identifikatoru do kapsule
    private void writeNetIdentifierToCapsule(NetIdentifier ni) throws IOException {
        if (ni != null) {
            oos.write(ni.getAddress().toByte());
            oos.write(ni.getNetwork().toByte());
//            System.out.println("MOJE ADRESA USPESNE ZAPSANA");
        } else {
            System.out.println("!!! CHYBA, PRI ZAPISOVANI ADRESY DO KAPSULE, " +
                    "PROTOZE SITOVY IDENTIFIKATOR BYL NULL !!!");
        }
    }

    // zapis smerovaciho zaznamu do kapsule
    private void writeMyRoutingRecordToCapsule(MyRoutingRecord mrr) throws IOException {
        if (mrr != null) {
//            System.out.println("ZAPISOVANY RR = SA:" + Arrays.toString(mrr.getSrcAddress()) + " SN:" + Arrays.toString(mrr.getSrcNetwork()) + " DA:" + Arrays.toString(mrr.getDstAddress()) + " DN:" + Arrays.toString(mrr.getDstNetwork()) + " DIST:" + mrr.getDistance());
            oos.write(mrr.getSrcAddress());
            oos.write(mrr.getSrcNetwork());
            oos.write(mrr.getDstAddress());
            oos.write(mrr.getDstNetwork());
            oos.writeByte(mrr.getDistance());
//            System.out.println("SMEROVACI ZAZNAM USPESNE ZAPSAN");
        } else {
            System.out.println("!!! CHYBA, PRI ZAPISOVANI SMEROVACIHO ZAZNAMU" +
                    "DO KAPSULE, PROTOZE BYL NULL !!!");
        }
    }

    // zpracovani kapsule jdouci k cilovemu uzlu
    private void forwardAntProcessing(ICapsuleAPI capsuleAPI) {
        try {
            bos = new ByteArrayOutputStream();
            oos = new ObjectOutputStream(bos);

            // test zda kapsle nesla po siti v nejake smycce
            checkCycle(capsuleAPI);

            // kapsule je teprve na ceste k cily
            oos.writeBoolean(onTheWayBack);

            // zapsani poctu navstivenych uzlu, +1 je navic moje adresa
            oos.writeInt(FwRecordCount + 1);
            // zapsani poctu routovacich zaznamu, v tuhle chvili porad nula
            oos.writeInt(BwRecordCount);

            //zapisani cilove adresy
            writeNetIdentifierToCapsule(destinationNode);

            // zapisovani doprednych zaznamu
            for (int i = 0; i < FwRecordCount; i++) {
                writeNetIdentifierToCapsule(visitedNodes[i]);
//                System.out.println(visitedNodes[i]);
            }

            // zapsani moji adresy na konec doprednych zaznamu
//            System.out.println("MOJE ADRESA" + getMyAddress(capsuleAPI));
            writeNetIdentifierToCapsule(getMyAddress(capsuleAPI));
            oos.close();

            //odeslani kapsule
            data = new TransmitData(bos.toByteArray());
            capsuleAPI.setDataLoad(data);
        } catch (Exception ex) {
            Logger.getLogger(NetAntCapsule.class.getName()).log(Level.SEVERE, null, ex);
        }
    }


    // kontrolovani zda kapsule nesla po siti v nejake smycce
    // a pripadne posunuti citace z konce pole na moji adresu
    // tzn. ze se pak budou kopirovat zaznamy z visitedNodes pouze
    // do tohoto zaznamu
    private void checkCycle(ICapsuleAPI capsuleAPI) {
        for (int i = 0; i < visitedNodes.length; i++) {
            NetIdentifier ni = visitedNodes[i];
//            System.out.print("Prozkoumavana adresa: " + ni);

            if (capsuleAPI.isThisServer(ni)) {
//                System.out.println(" ==>  !!! NALEZENA SHODA u dopredneho zaznamu " + ni + " , JSEM V KRUHU !!!");
                FwRecordCount = i;
                break;
            }
        }
    }

    // zvoleni nahodneho primeho souseda pro dalsi skok kapsule
    private NetIdentifier selectOneHopAddress(ICapsuleAPI capsuleAPI) {
        Collection<RoutingRecord> directNeighbors = capsuleAPI.getRoutingModification().getDirectNeighbours();

        if (directNeighbors != null) {
            generator = new Random();
            int index = generator.nextInt(directNeighbors.size());
            int i = 0;

            for (Iterator<RoutingRecord> it = directNeighbors.iterator(); it.hasNext();) {
                RoutingRecord rr = it.next();

                if (i == index) {
//                    System.out.println("DALSI SKOK NA: " + rr.getDestination());
                    return rr.getDestination();
                }
                i++;
            }
        }
        return null;
    }



    // privatni trida, pro usnadneni prace s bajtovymi poli predstavujici
    // jeden prenaseny routovaci zaznam
    private class MyRoutingRecord {

        private byte[] srcAddress;
        private byte[] srcNetwork;
        private byte[] dstAddress;
        private byte[] dstNetwork;
        private byte distance;

        public MyRoutingRecord(byte[] srcAddress, byte[] srcNetwork, byte[] dstAddress, byte[] dstNetwork, byte distance) {
            this.srcAddress = srcAddress;
            this.srcNetwork = srcNetwork;
            this.dstAddress = dstAddress;
            this.dstNetwork = dstNetwork;
            this.distance = distance;
        }

        public byte getDistance() {
            return distance;
        }

        public void setDistance(byte distance) {
            this.distance = distance;
        }

        public byte[] getDstAddress() {
            return dstAddress;
        }

        public void setDstAddress(byte[] dstAddress) {
            this.dstAddress = dstAddress;
        }

        public byte[] getDstNetwork() {
            return dstNetwork;
        }

        public void setDstNetwork(byte[] dstNetwork) {
            this.dstNetwork = dstNetwork;
        }

        public byte[] getSrcAddress() {
            return srcAddress;
        }

        public void setSrcAddress(byte[] srcAddress) {
            this.srcAddress = srcAddress;
        }

        public byte[] getSrcNetwork() {
            return srcNetwork;
        }

        public void setSrcNetwork(byte[] srcNetwork) {
            this.srcNetwork = srcNetwork;
        }
    }
}
