/*************************************************************************
 *                           Smart Active Node                           * 
 *                      copyright (C) 2007 KIV-FAV-ZCU                   *
 *************************************************************************
 * The licence for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package remoteShell;

import java.io.PrintWriter;

import java.security.PublicKey;
import san.security.Crypto.KeyReader;
import san.core.ExecuteException;
import san.core.IApplicationAPI;
import san.core.IServerControl;
import san.application.IApplication;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import java.util.Collection;
import san.networkInterfaces.NetInterfaces;
import san.routing.*;
import san.routing.IRoutingTableControl;
import san.data.ReceiveDataListener;

class RemoteCommand implements Util.ICommand {

    private String command = "";
    private String arg = "";
    private String arg2 = "";
    private String sourceAddress = "";
    private String sourceNet = "";
    private String destinationAddress = "";
    private String destinationNet = "";
    private boolean pathToSource = false;

    public RemoteCommand(String command) {
        this.command = command;
    }

    public RemoteCommand(String command, String arg) {
        this.command = command;
        this.arg = arg;
    }

    public RemoteCommand(String command, String arg, String arg2) {
        this.command = command;
        this.arg = arg;
        this.arg2 = arg2;
    }

    public void setSourceAddress(String sourceAddress)
    {
        this.sourceAddress = sourceAddress;
    }
    
    public void setSourceNet(String sourceNet)
    {
        this.sourceNet = sourceNet;
    }
    
    public void setDestinationAddress(String destinationAddress)
    {
        this.destinationAddress = destinationAddress;
    }
    
    public void setDestinationNet(String destinationNet)
    {
        this.destinationNet = destinationNet;
    }

    public void setCommand(String command) {
        this.command = command;
    }

    public void setArgument(String arg) {
        this.arg = arg;
    }

    public void setArgument2(String arg) {
        this.arg2 = arg;
    }

    public String getCommand() {
        return this.command;
    }

    public String getArgument() {
        return this.arg;
    }

    public String getArgument2() {
        return this.arg2;
    }

    public void setIsPathToSource()
    {
        this.pathToSource = true;
    }

    public boolean isPathToSource()
    {
        return this.pathToSource;
    }

    public String getSourceAddress()
    {
        return this.sourceAddress;
    }
    public String getSourceNet()
    {
        return this.sourceNet;
    }
    public String getDestinationAddress()
    {
        return this.destinationAddress;
    }

    public String getDestinationNet()
    {
        return this.destinationNet;
    }

    public NetIdentifier getDestinationIdentifier()
    {
        NetIdentifier netIdent = Util.getDestination(this.destinationAddress, this.destinationNet);

        return netIdent;
    }

    public NetIdentifier getSourceIdentifier()
    {
        NetIdentifier netIdent = Util.getDestination(this.sourceAddress, this.sourceNet);

        return netIdent;
    }
}

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> RemoteShell.java<br>
 * @author Vladimír Aubrecht
 * @since 15.12.2007
 * @version 1.0
 */
public class RemoteShell implements IApplication, ReceiveDataListener {

    IApplicationAPI applicationAPI;
    PrintWriter out;

    private void Init(IApplicationAPI applicationAPI) {
        this.applicationAPI = applicationAPI;
        this.out = applicationAPI.getStdOut();
    }
    private TransmitData received = null;

    private void sendCommand(NetIdentifier netIdent, RemoteCommand cmd) throws Exception {
        Util.sendCapsule(applicationAPI, netIdent, cmd);
    }

    private void readCommand(RemoteCommand cmd) throws Exception {
        Util.readCapsuleData(this.received, cmd);
    }

    class RemoteServerControl implements IServerControl {

        NetIdentifier netIdent;

        public RemoteServerControl(NetIdentifier netIdent) {
            this.netIdent = netIdent;
        }

        public String storeApplication(String localPath) {
            return null;
        }

        public Guid runApplication(String name, String[] attr) {
            return null;
        }

        public Guid runApplication(CodePkgIdent appIdent, String[] attr) {
            return null;
        }

        public Guid runApplication(String name, String[] attr, Guid parentAppl) {
            return null;
        }

        public Guid runApplication(CodePkgIdent appIdent, String[] attr, Guid parentAppl) {
            return null;
        }

        public void waitForApplication(Guid app, Collection<Guid> applId) {
        }

        public void createConsole(Guid id) {
        }

        public NetInterfaces getNetInterfaces() {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public Router getRouter() {
            throw new UnsupportedOperationException("Not supported yet.");
        }

        public IRoutingTableControl getRoutingModification() {
            return null;
        }

        public void shutDownServer() {
            try {
                RemoteCommand cmd = new RemoteCommand("exit");
                sendCommand(netIdent, cmd);
                readCommand(cmd);
            } catch (Exception ex) {
            }
        }

        public Guid getInitGuid() {
            return null;
        }

        public RoutingTable getRoutingTable() {
            return null;
        }
    }

    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        Init(applicationAPI);

        applicationAPI.addOnSendDataListener(guid);

        try {

            String login = args[2];
            String pass = args[3];

            RemoteCommand cmd = new RemoteCommand("getPublicKey");
            cmd.setDestinationAddress(args[0]);
            cmd.setDestinationNet(args[1]);
            cmd.setSourceAddress("0001");
            cmd.setSourceNet("0001");
            sendCommand(cmd.getDestinationIdentifier(), cmd);
            System.out.println("Poslana kapsule s zadosti o verejny klic");
            
            while (received == null)
            {
                Thread.sleep(1L);
            }

            readCommand(cmd);

            System.out.println(cmd.getArgument());

            KeyReader keyReader = new KeyReader();
            PublicKey publicKey = keyReader.getPublicKey(cmd.getArgument().getBytes());

            System.out.println(publicKey.toString());

            login = applicationAPI.getEncryptedString(login, publicKey);
            pass = applicationAPI.getEncryptedString(pass, publicKey);

            System.out.println(pass);

            cmd.setCommand("login");
            cmd.setArgument(login);
            cmd.setArgument2(pass);

            System.out.println(pass);

            sendCommand(cmd.getDestinationIdentifier(), cmd);
            readCommand(cmd);

            System.out.println(cmd.getArgument());

           if (cmd.getArgument().equals("OK")) {
                RemoteServerControl remoteServer = new RemoteServerControl(cmd.getDestinationIdentifier());
                remoteShell.Shell shell = new remoteShell.Shell();
                shell.setServerControlAPI(remoteServer);
                shell.main(null, applicationAPI, guid);
            }

        } catch (Exception ex) {
            out.println(ex.getMessage());
        }
    }

    public void receiveData(Guid sender, TransmitData data, IApplicationAPI applicationAPI) {
        this.received = data;
        System.out.println("prijmuto!");

       RemoteCommand cmd = new RemoteCommand("");
        System.out.println("haaaaaaaaaaaa!");
        try
        {
            Util.readCapsuleData(data, cmd);

            if (!cmd.isPathToSource())
            {
                
                /*cmd.setIsPathToSource();

                String command = cmd.getCommand();
                if (command.equals("getPublicKey"))
                {
                    cmd.setArgument("TSSSS");
                }
                else if (command.equals("login"))
                {
                    cmd.setArgument("OK");
                }
                else if (command.equals("exit"))
                {

                }

                sendCommand(cmd.getSourceIdentifier(), cmd);*/
            }
        }
        catch (Exception ex) {}
    }
}
