/*************************************************************************
 *                           Smart Active Node                           *
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package rendezvous;

import java.io.PrintWriter;
import san.RendezVous.IRendezvousServer;
import san.application.IApplication;
import san.core.IApplicationAPI;
import san.ident.Guid;

/**
 * This application testing rendezvous on one node. Main application launch
 * a lot of this application in server and client mode and write results of test.
 * @author Miroslav Hendrych
 */
public class RendezVous implements IApplication, IRendezvousServer {
    public static final int DEF_POCET_SRV = 5;
    public static final int DEF_POCET_CLNT = 13;
    PrintWriter out;
    IApplicationAPI applicationAPI;
    public static String ENTRY_CALL = "readyForAccept";

    private void ridiciAplikace(int pocetServeru, int pocetKlientu) throws Exception {
        out.println("pred Guid: ");
        Guid guid_main = Guid.randomGuid();
        out.println("Main Entry Guid: "+guid_main);
        Guid[] guidS = new Guid[pocetServeru];
        Guid[] guidC = new Guid[pocetKlientu];
        if (applicationAPI.registerServer(this, guid_main)) {
            applicationAPI.registerEntryCall(guid_main, ENTRY_CALL);
        } else {
            out.println("Neporadilo se zaregistrovat main entry");
            return;
        }

        for (int i=0; i<pocetServeru; i++) {
            guidS[i] = applicationAPI.runApplication("rendezvous", new String[]{"-s", String.valueOf(i), guid_main.toString()} );
        }
        for (int i=0; i<pocetServeru; i++)
            applicationAPI.makeAccept(guid_main, new String[] {ENTRY_CALL});

        for (int i=0; i<pocetKlientu; i++) {
            guidC[i] = applicationAPI.runApplication("rendezvous", new String[]{"-c", String.valueOf(i)} );
        }
        for (int i=0; i<pocetKlientu; i++) {
            applicationAPI.waitForApplication(guidC[i]);
        }
        
        Guid[] guidStops = applicationAPI.getServerGuidsByName("stop");
        for (int i=0; i<guidStops.length; i++) {
            applicationAPI.callEntryCallByName(guidStops[i], "stop", new Integer(-1));
        }
        for (int i=0; i<pocetServeru; i++) {
            applicationAPI.waitForApplication(guidS[i]);
        }
        applicationAPI.unregisterServer(guid_main);
        out.println("Odregistrovano main entry");
    }

    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        this.applicationAPI = applicationAPI;
		out = applicationAPI.getStdOut();

        if (args.length == 0 || args.length==2 && !args[0].equals("-c")) {
            int pocet_srv = DEF_POCET_SRV;
            int pocet_clnt = DEF_POCET_CLNT;
            try {
                if (args.length == 2) {
                    pocet_srv = Integer.parseInt(args[0]);
                    pocet_clnt = Integer.parseInt(args[1]);
                }
                if (pocet_srv <= 0 || pocet_srv <= 0)
                    out.println("Chybne parametry prikazove radky");
                else
                    ridiciAplikace(pocet_srv, pocet_clnt);
            } catch (NumberFormatException ex) {
                out.println("Jeden ze zadanych parametru neni cislo!");
            } catch (Exception ex) {
                ex.printStackTrace(out);
            }
        } else {
            if (args.length==3 && args[0].equals("-s")) {
                out.println("Server "+args[1]+" Guid: " + applicationAPI.getId().toString());
                RendezVousSrv srv = new RendezVousSrv();
                Guid guid_main = Guid.parseGuid(args[2]);
                srv.run(applicationAPI, out, guid_main);
                out.println("Server "+args[1]+":  plusu: "+srv.getPlus()+", minusu: "+srv.getMinus());
            }
            if (args.length==2 && args[0].equals("-c")) {
                out.println("Klient "+args[1]+" Guid: " + applicationAPI.getId());
                RendezVousClnt clnt = new RendezVousClnt(applicationAPI);
                out.println("Klient "+args[1]+":  "+(clnt.isCorrect() ? "OK" : "CHYBA"));
            }
        }
    }

    public Object entry(String name, Object param) {
        return null;
    }
}
