/*************************************************************************
 *                           Smart Active Node                           *
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.core;

import san.ident.Guid;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This interface is basic for Capsule and Applications.<br />
 * @author Miroslav Hendrych
 * @since 9.3.2010
 */
public interface ICommonApi {
    /**
     * Call registered entry call by entry call ID
     * @param entryID Entry call ID
     * @param param Parameter for entry call
     * @return Returned entry call value
     * @throws java.lang.InterruptedException
     */
    public Object callEntryCallByID(Integer entryID, Object param) throws InterruptedException;

    /**
     * Call registered entry call by name and server Guid
     * @param server Registered server guid
     * @param entry Registered entry call name
     * @param param Parameter for entry call
     * @return Returned entry call value
     * @throws java.lang.InterruptedException
     */
    public Object callEntryCallByName(Guid server, String entry, Object param) throws InterruptedException;

    /**
     * Return server guids which have registered entry call name
     * @param entryName Registered entry call name
     * @return Array of registered server guids
     */
    public Guid[] getServerGuidsByName(String entryName);

    /**
     * Return entry call names registered by server guid
     * @param server Server guid
     * @return Array of entry call names
     */
    public String[] getEntryCallsNamesByGuid(Guid server);

    /**
     * Return entry call id by server guid and entry call name
     * @param server Server guid
     * @param entryName Entry call name
     * @return Entry call ID or null
     */
    public Integer getEntryCallIdByGuidName(Guid server, String entryName);
}
