/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.io.InputStream;
import java.io.PrintWriter;
import java.security.InvalidKeyException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.LinkedList;
import java.util.Collection;
import java.util.Vector;

import san.applicationRepository.SanCodePkg;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.ii.Packet;
import san.networkInterfaces.NetInterface;
import san.softstate.ISoftState;
import java.util.logging.*;
import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import san.RendezVous.RendezvousServerRef;
import san.RendezVous.IRendezvousServer;
import san.interpreter.sanint.fiber.call.ReceiveDataCall;
import san.security.Crypto.CryptUtil;
import san.security.Crypto.KeyGenerator;
import san.security.SecurityMonitor;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class is an application process.<br>
 * @author Michal Rejda
 * @since 21.10.2007
 * @version 1.0
 */
public class ProcessApplication extends Process implements IApplicationAPI {

	private static final transient Logger log = Logger.getLogger(ProcessApplication.class.getName());
	
        /**
	 * @uml.property name="onSendDataListeners"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="processApplication:san.data.OnSendDataListener"
	 */
	private Collection<Guid> onSendDataListeners;
	/**
	 * @uml.property name="args" multiplicity="(0 -1)" dimension="1"
	 */
	private String[] args;

	public ProcessApplication(Kernel kernel) {
		super(kernel,
              Type.APPLICATION,
              "main",
              "([Ljava/lang/String;Lsan/core/IApplicationAPI;Lsan/ident/Guid;)V");
		this.onSendDataListeners = new LinkedList<Guid>();
	}

	public void addOnSendDataListener(Guid listener) {
		synchronized (onSendDataListeners) {
			onSendDataListeners.add(listener);
		}
	}

	public boolean removeOnSendDataListener(ReceiveDataListener listener) {
		synchronized (onSendDataListeners) {
			return onSendDataListeners.remove(listener);
		}
	}

	protected void fireOnSendDataListeners(Guid sender, TransmitData data) {
		synchronized (onSendDataListeners) {
			for (Guid listener : onSendDataListeners) {
//				try
//				{
//					Reference thisRef = kernel.getInterpretManager().getRunningApplication(listener);
//					ClassManager cman = new ClassManager((SanCodePkg)this.codePackage, this);
//					//receiveData(Guid sender, TransmitData data);
//					cman.runMethod(thisRef.getClassInfo().getName(), "receiveData",
//							"(Lsan/ident/Guid;Lsan/data/TransmitData;)V",
//							new Object[] {
//								sender,
//								data,
//							}, thisRef, false);
//				}
//				catch (Exception e)
//				{
//					e.printStackTrace();
//				}
//				///listener.receiveData(sender, data);
                ProcessApplication process = new ProcessApplication(kernel);
                process.setMainMethodName("receiveData");
                process.setMainMethodDescriptor("(Lsan/ident/Guid;Lsan/data/TransmitData;Lsan/core/IApplicationAPI;)V");
                process.setParent(id);
                process.setSpecialCall(new ReceiveDataCall(listener, sender, data));
                kernel.getScheduler().runSpecialProcess(process, codePackage);
			}
		}
	}


	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload)
			throws DestinationNotAvailableException {
		getKernel().injectCapsule(executedCode, getId(), destination, payload);
	}
	
	@Override
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload,
			boolean executeContinuously)
			throws DestinationNotAvailableException {

		getKernel().injectCapsule(executedCode, getId(), destination, payload, executeContinuously);
		
	}

	public IServerControl getServerControl() {
		return getKernel();
	}

	@Override
	public Guid getId() {
		return super.getId();
	}

	public CodePkgIdent getExecutedCodeIdent() {
		return codePackage.getPkgIdentification();
	}

	public InputStream getStdIn() {
		if (console != null)
			return console.getIn();
		return null;
	}

	public PrintWriter getStdOut() {
		if (console != null)
			return console.getOut();
		return null;
	}

	public Guid runApplication(CodePkgIdent ident, String[] args)
			throws ExecuteException {
		return kernel.runApplication(ident, args, id);
	}

	public Guid runApplication(String name, String[] args)
			throws ExecuteException {

        return kernel.runApplication(name, args, id);
	}

	public void waitForApplication(Collection<Guid> applId)
			throws InterruptedException {
		kernel.waitForApplication(id, applId);
	}

	public void waitForApplication(Guid applId) throws InterruptedException {
		Collection<Guid> waitFor = new Vector<Guid>();
		waitFor.add(applId);
		waitForApplication(waitFor);
	}

	/**
	 * Getter of the property <tt>args</tt>
	 * @return Returns the args.
	 * @uml.property name="args"
	 */
	public String[] getArgs() {
		return args;
	}

	/**
	 * Setter of the property <tt>args</tt>
	 * @param args The args to set.
	 * @uml.property name="args"
	 */
	public void setArgs(String[] args) {
		this.args = args;
	}
	
	public ISoftState getSoftState() {
		return kernel.getSoftState();
	}

        public boolean logUser(String username, String password) {
            SecurityMonitor monitor = san.security.SecurityMonitor.getInstance();

            boolean userLogged = monitor.log(username, password);
            
            if (userLogged)
            {
                this.setOwner(username);
            }

            return userLogged;
        }

        public String whoAmI() {
            return this.getOwner();
        }

        public String getParentOwner() {
                String currentUser = "anonymous";

                if (id != null)
                    currentUser = this.kernel.getScheduler().getProcess(id).getOwner();

                return currentUser;
        }

        public KeyGenerator createKeyGenerator()
        {
            KeyGenerator gen = new KeyGenerator();
            return gen;
        }

        public String getEncryptedString(String message, PublicKey publicKey) throws InvalidKeyException, IllegalBlockSizeException, BadPaddingException
        {
            return new String(CryptUtil.getEncryptedBytes(message.getBytes(), publicKey));
        }

        public String getDecryptedString(String encryptedMessage, PrivateKey privateKey) throws InvalidKeyException, IllegalBlockSizeException, BadPaddingException
        {
            return new String (CryptUtil.getDecryptedBytes(encryptedMessage.getBytes(), privateKey));
        }

	@Override
	public String getNodeInfo() {	
		String info = "";
		
		for(NetIdentifier n : kernel.getNetInterfaces().getNetInterfaces().keySet()){			
			info += "Local interface: \n";
			info += "address: " + n.getAddress().toString() + "\n";
			info += "network: " + n.getNetwork().toString() + "\n";
			NetInterface ni = kernel.getNetInterfaces().getNetInterfaces().get(n);
			for(NetIdentifier r : ni.getRemoteInterfaces().keySet()){
				info += "\t\tRemote interface: \n";
				info += "\t\taddress: " + r.getAddress().toString() + "\n";
				info += "\t\tnetwork: " + r.getNetwork().toString() + "\n";
			}						
		}	
		
		info += "Applications: \n";
		for(String s : kernel.getRepository().getCpBaseOnPkgNamesMapKeys()){
			info += "app: " + s + "\n";
		}
		
		return info;
	}

  public void sendPayload(Packet p) {
    try {
      InterceptorInjector.getInterceptorInjector().sendPayload(p);
    } catch (Exception ex) {
      // Just do nothing - no data will be delivered
    }
  }

    public Guid registerServer(IRendezvousServer entryClass) {
        if (entryClass instanceof RendezvousServerRef) {
            ((RendezvousServerRef)entryClass).setAppGuid(id);
            ((RendezvousServerRef)entryClass).setCodePkg((SanCodePkg)this.codePackage);
            return kernel.registerServer(entryClass);
        }
        return null;
    }

    public boolean registerServer(IRendezvousServer entryClass, Guid serverId) {
        if (entryClass instanceof RendezvousServerRef) {
            ((RendezvousServerRef)entryClass).setAppGuid(id);
            ((RendezvousServerRef)entryClass).setCodePkg((SanCodePkg)this.codePackage);
            return kernel.registerServer(entryClass, serverId);
        }
        return false;
    }

    public Integer registerEntryCall(Guid server, String entryName) {
        return kernel.registerEntryCall(server, entryName);
    }

    public void makeAccept(Guid server) throws InterruptedException {
        kernel.makeAccept(server, this);
    }

    public void makeAccept(Guid server, String[] entryNames) throws InterruptedException {
        kernel.makeAccept(server, entryNames, this);
    }

    public void unregisterServer(Guid server) {
        kernel.unregisterServer(server);
    }

    public String[] getEntryCallsNamesByGuid(Guid server) {
        return kernel.getEntryCallsNamesByGuid(server);
    }

    public Object callEntryCallByName(Guid server, String entry, Object param) throws InterruptedException {
        Object obj = kernel.callEntryCallByName(server, entry, param);
        return obj;
    }

    public Object callEntryCallByID(Integer entryID, Object param) throws InterruptedException {
        return kernel.callEntryCallByID(entryID, param);
    }

    public Guid[] getServerGuidsByName(String entryName) {
        return kernel.getServerGuidsByName(entryName);
    }

    public Integer getEntryCallIdByGuidName(Guid server, String entryName) {
        return kernel.getEntryCallIdByGuidName(server, entryName);
    }
}
