package san.core.fiber.info;



import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberInfoHttpHandler implements HttpHandler {



    private final FiberInfo fiberInfo;

    private byte[] cachedPageHeader;



    public FiberInfoHttpHandler(FiberInfo fiberInfo) {
        this.fiberInfo = fiberInfo;
        cachedPageHeader = null;
    }



    private byte[] buildPageHeader() {
        if (cachedPageHeader == null) {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            PrintStream printStream;
            try {
                printStream = new PrintStream(byteArrayOutputStream, true, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                printStream = new PrintStream(byteArrayOutputStream, true);
            }
            printStream.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
            printStream.println("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
            printStream.println("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">");
            printStream.println("<head>");
            printStream.println("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\" />");
            printStream.println("<meta http-equiv=\"refresh\" content=\"10\" />");
            printStream.println("<title>Fiber info HTTP monitor</title>");
            printStream.println("</head>");
            printStream.println("<body>");
            printStream.flush();
            cachedPageHeader = byteArrayOutputStream.toByteArray();
        }
        return cachedPageHeader;
    }



    private byte[] buildPageBody() {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        PrintStream printStream;
        try {
            printStream = new PrintStream(byteArrayOutputStream, true, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            printStream = new PrintStream(byteArrayOutputStream, true);
        }
        printStream.println("<h1>Fiber info HTTP monitor</h1>");
        printStream.println("<table border=\"1\">");

        printStream.println("<tr>");
        printStream.println("<td>Applications started count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getApplicationsStartedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Applications finished count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getApplicationsFinishedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Applications failed count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getApplicationsFailedCount());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Capsules started count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getCapsulesStartedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Capsules finished count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getCapsulesFinishedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Capsules failed count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getCapsulesFailedCount());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Special call started count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getSpecialCallStartedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Special call finished count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getSpecialCallFinishedCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Special call failed count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getSpecialCallFailedCount());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Startup time</td>");
        printStream.format("<td>%s</td>%n", fiberInfo.getStartupTime());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Run time</td>");
        printStream.format("<td>%s</td>%n", fiberInfo.getRunTime());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Physical CPU count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getPhysicalCPUCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Virtual CPU count</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getVirtualCPUCount());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Max capsules running</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getMaxCapsulesRunning());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Scheduler sleep interval</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getSchedulerSleepInterval());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Scheduler strategy</td>");
        printStream.format("<td>%s</td>%n", fiberInfo.getSchedulerStrategy());
        printStream.println("</tr>");

        printStream.println("<tr>");
        printStream.println("<td>Deadlock detector enabled</td>");
        printStream.format("<td>%b</td>%n", fiberInfo.isDeadlockDetectorEnabled());
        printStream.println("</tr>");
        printStream.println("<tr>");
        printStream.println("<td>Deadlock detector sleep interval</td>");
        printStream.format("<td>%d</td>%n", fiberInfo.getDeadlockDetectorSleepInterval());
        printStream.println("</tr>");

        printStream.println("</table>");
        printStream.println("</body>");
        printStream.println("</html>");
        printStream.flush();
        return byteArrayOutputStream.toByteArray();
    }



    @Override
    public void handle(HttpExchange he) throws IOException {
        byte[] pageHeader = buildPageHeader();
        byte[] pageBody = buildPageBody();
        he.sendResponseHeaders(200, pageHeader.length + pageBody.length);
        OutputStream outputStream = he.getResponseBody();
        outputStream.write(pageHeader);
        outputStream.write(pageBody);
        outputStream.close();
    }
}
