package san.core.fiber.info;



import com.sun.net.httpserver.HttpServer;
import java.net.BindException;
import java.net.InetSocketAddress;
import org.apache.log4j.Logger;
import san.core.fiber.log.FiberLoggerFactory;
import san.core.fiber.settings.FiberSettings;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberInfoHttpServer {



    private static final Logger logger = FiberLoggerFactory.getStartupLogger();

    private static final int SOCKET_BIND_ATTEMPT_COUNT = 10;

    private final FiberInfo fiberInfo;

    private HttpServer httpServer;
    private int port;



    public FiberInfoHttpServer(FiberInfo fiberInfo) {
        this.fiberInfo = fiberInfo;
        this.port = FiberSettings.getInstance().getFiberInfoHttpPort();
    }



    public void startServer() {
        Thread startServerThread = new Thread() {
            @Override
            public void run() {
                for (int i = 0; i < SOCKET_BIND_ATTEMPT_COUNT; i++) {
                    try {
                        httpServer = HttpServer.create(new InetSocketAddress(port), 0);
                        httpServer.createContext("/", new FiberInfoHttpHandler(fiberInfo));
                        httpServer.start();
                        System.out.format("Fiber info HTTP monitor starting on port %d%n", port);
                        return;
                    } catch (BindException e) {
                        port++;
                        continue;
                    } catch (Exception e) {
                        logger.error("CFG - can not start Fiber info HTTP monitor", e);
                        return;
                    }
                }
                logger.warn(String.format("CFG - can not bind Fiber info HTTP monitor on given port"));
            }
        };
        startServerThread.start();
    }



    public void stopServer() {
        if (httpServer != null) {
            httpServer.stop(0);
        }
    }
}
