package san.core.fiber.log;



import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberLoggerFactory {



    private static final Level SCHEDULER_LOGGER_LEVEL = Level.WARN;
    private static final Level INTERPRETER_LOGGER_LEVEL = Level.WARN;
    private static final Level STARTUP_LOGGER_LEVEL = Level.WARN;
    private static final Level CONSOLE_APPENDER_TRESHOLD = Level.ALL;
    private static final Level FILE_APPENDER_TRESHOLD = Level.WARN;
    private static final String CONSOLE_APPENDER_FORMAT = "%d{HH:mm:ss,SSS} %p %m%n";
    private static final String FILE_APPENDER_FORMAT = "%d{dd.MM.yyyy HH:mm:ss,SSS} %p %m%n";
    private static final String CONSOLE_APPENDER_TARGET = "System.out";
    private static final String FILE_APPENDER_FILE_NAME_FORMAT = "log/fiberLog%s.txt";

    private static ConsoleAppender consoleAppender;
    private static FileAppender fileAppender;
    private static String fileName;



    private static ConsoleAppender getConsoleAppender() {
        if (consoleAppender == null) {
            consoleAppender = new ConsoleAppender();
            consoleAppender.setLayout(new PatternLayout(CONSOLE_APPENDER_FORMAT));
            consoleAppender.setName("Fiber console appender");
            consoleAppender.setThreshold(CONSOLE_APPENDER_TRESHOLD);
            consoleAppender.setTarget(CONSOLE_APPENDER_TARGET);
            consoleAppender.activateOptions();
        }
        return consoleAppender;
    }



    private static FileAppender getFileAppender() {
        if (fileAppender == null) {
            fileAppender = new FileAppender();
            fileAppender.setLayout(new PatternLayout(FILE_APPENDER_FORMAT));
            fileAppender.setName("Fiber file appender");
            fileAppender.setThreshold(FILE_APPENDER_TRESHOLD);
            fileAppender.setAppend(true);
            fileAppender.setBufferedIO(false);
            fileAppender.setBufferSize(0);
            if (fileName == null) {
                fileName = String.format(FILE_APPENDER_FILE_NAME_FORMAT, "");
            }
            fileAppender.setFile(fileName);
            fileAppender.activateOptions();
        }
        return fileAppender;
    }



    private static Logger getLogger() {
        String invokingClassName = Thread.currentThread().getStackTrace()[3].getClassName();
        Logger logger = Logger.getLogger(invokingClassName);
        logger.setAdditivity(false);
        logger.addAppender(getConsoleAppender());
        logger.addAppender(getFileAppender());
        return logger;
    }



    public static Logger getSchedulerLogger() {
        Logger logger = getLogger();
        logger.setLevel(SCHEDULER_LOGGER_LEVEL);
        return logger;
    }



    public static Logger getInterpreterLogger() {
        Logger logger = getLogger();
        logger.setLevel(INTERPRETER_LOGGER_LEVEL);
        return logger;
    }



    public static Logger getStartupLogger() {
        Logger logger = getLogger();
        logger.setLevel(STARTUP_LOGGER_LEVEL);
        return logger;
    }



    public static void setLogFileIdentifier(String identifier) {
        fileName = String.format(FILE_APPENDER_FILE_NAME_FORMAT, identifier);
        if (fileAppender != null) {
            fileAppender.setFile(fileName);
            fileAppender.activateOptions();
        }
    }
}
