package san.core.fiber.monitor;



import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
import san.core.Process;



/**
 *
 * @author Jiri Nohavec
 */
public class MonitorWrap {



    private final Object monitor;
    private final Queue<Process> processQueue;

    private int lockCount;



    public MonitorWrap(Object monitor) {
        this.monitor = monitor;
        processQueue = new LinkedList<Process>();
        lockCount = 0;
    }



    public Object getMonitor() {
        return monitor;
    }



    public Iterable<Process> getProcessIterable() {
        return processQueue;
    }



    public Iterator<Process> getProcessIterator() {
        return processQueue.iterator();
    }



    public Process getHeadProcess() {
        return processQueue.element();
    }



    public Process removeHeadProcess() {
        return processQueue.remove();
    }



    public int getProcessQueueSize() {
        return processQueue.size();
    }



    public boolean isProcessQueueEmpty() {
        return processQueue.isEmpty();
    }



    public boolean containsProcess(Process process) {
        return processQueue.contains(process);
    }



    public boolean isHeadProcess(Process process) {
        return (!processQueue.isEmpty() && processQueue.element().equals(process));
    }



    public void addProcess(Process process) {
        processQueue.add(process);
    }



    public int getLockCount() {
        return lockCount;
    }



    public void incrementLockCount() {
        lockCount++;
    }



    public void decrementLockCount() {
        lockCount--;
    }



    public void resetLockCount() {
        lockCount = 0;
    }
}
