package san.core.fiber.settings;



import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import san.core.fiber.log.FiberLoggerFactory;
import san.core.fiber.strategy.StrategyManager;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberConfigFileParser {



    private static final Logger logger = FiberLoggerFactory.getStartupLogger();



    private static String getAttributeValue(Document document, String tagName, String attributeName) {
        NodeList nodeList;
        NamedNodeMap namedNodeMap;
        Node node;
        Node attributeNode;
        nodeList = document.getElementsByTagName(tagName);
        node = nodeList.item(0);
        if (node == null) {
            return null;
        }
        namedNodeMap = node.getAttributes();
        attributeNode = namedNodeMap.getNamedItem(attributeName);
        if (attributeNode == null) {
            return null;
        }
        return attributeNode.getNodeValue().trim();
    }



    public static void parse(Document document) {
        FiberSettings fiberSettings = FiberSettings.getInstance();
        String value;
        int intValue;
        long longValue;
        try {
            value = getAttributeValue(document, "scheduler", "virtualCPUCount");
            if (value != null) {
                intValue = Integer.parseInt(value);
                if (intValue < 1) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setVirtualCPUCount(intValue);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> virtualCPUCount");
        }
        try {
            value = getAttributeValue(document, "scheduler", "maxCapsulesRunning");
            if (value != null) {
                intValue = Integer.parseInt(value);
                if (intValue < 1) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setMaxCapsulesRunning(intValue);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> maxCapsulesRunning");
        }
        try {
            value = getAttributeValue(document, "scheduler", "schedulerSleepInterval");
            if (value != null) {
                longValue = Long.parseLong(value);
                if (longValue < 0L) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setSchedulerSleepInterval(longValue);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> schedulerSleepInterval");
        }
        try {
            value = getAttributeValue(document, "scheduler", "schedulerStrategy");
            if (value != null) {
                if (!StrategyManager.isStrategyNameValid(value)) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setSchedulerStrategy(value);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> schedulerStrategy");
        }
        try {
            value = getAttributeValue(document, "scheduler", "deadlockDetectorEnabled");
            if (value != null) {
                fiberSettings.setDeadlockDetectorEnabled(Boolean.parseBoolean(value));
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> deadlockDetectorEnabled");
        }
        try {
            value = getAttributeValue(document, "scheduler", "deadlockDetectorSleepInterval");
            if (value != null) {
                longValue = Long.parseLong(value);
                if (longValue < 0L) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setDeadlockDetectorSleepInterval(longValue);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> deadlockDetectorSleepInterval");
        }
        try {
            value = getAttributeValue(document, "scheduler", "fiberInfoHttpPort");
            if (value != null) {
                intValue = Integer.parseInt(value);
                if (intValue < 1) {
                    throw new IllegalArgumentException();
                }
                fiberSettings.setFiberInfoHttpPort(intValue);
            }
        } catch (Exception e) {
            logger.warn("CFG - configuration file parse error at tag scheduler -> fiberInfoHttpPort");
        }
    }
}
