package san.core.fiber.settings;



import san.core.fiber.strategy.StrategyManager;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberSettings {



    private static FiberSettings fiberSettingsSingleton;



    public static FiberSettings getInstance() {
        if (fiberSettingsSingleton == null) {
            fiberSettingsSingleton = new FiberSettings();
        }
        return fiberSettingsSingleton;
    }



    private static final int DEFAULT_VIRTUAL_CPU_COUNT = Runtime.getRuntime().availableProcessors();
    private static final int DEFAULT_MAX_CAPSULES_RUNNING = 4;
    private static final long DEFAULT_SCHEDULER_SLEEP_INTERVAL = 100L;
    private static final String DEFAULT_SCHEDULER_STRATEGY = StrategyManager.getDefaultStrategyName();
    private static final boolean DEFAULT_DEADLOCK_DETECTOR_ENABLED = true;
    private static final long DEFAULT_DEADLOCK_DETECTOR_SLEEP_INTERVAL = 1000L;
    private static final int DEFAULT_FIBER_INFO_HTTP_PORT = 47470;

    private int virtualCPUCount;
    private int maxCapsulesRunning;
    private long schedulerSleepInterval;
    private String schedulerStrategy;
    private boolean deadlockDetectorEnabled;
    private long deadlockDetectorSleepInterval;
    private int fiberInfoHttpPort;



    private FiberSettings() {
        virtualCPUCount = DEFAULT_VIRTUAL_CPU_COUNT;
        maxCapsulesRunning = DEFAULT_MAX_CAPSULES_RUNNING;
        schedulerSleepInterval = DEFAULT_SCHEDULER_SLEEP_INTERVAL;
        schedulerStrategy = DEFAULT_SCHEDULER_STRATEGY;
        deadlockDetectorEnabled = DEFAULT_DEADLOCK_DETECTOR_ENABLED;
        deadlockDetectorSleepInterval = DEFAULT_DEADLOCK_DETECTOR_SLEEP_INTERVAL;
        fiberInfoHttpPort = DEFAULT_FIBER_INFO_HTTP_PORT;
    }



    public int getVirtualCPUCount() {
        return virtualCPUCount;
    }



    public void setVirtualCPUCount(int virtualCPUCount) {
        this.virtualCPUCount = virtualCPUCount;
    }



    public int getMaxCapsulesRunning() {
        return maxCapsulesRunning;
    }



    public void setMaxCapsulesRunning(int maxCapsulesRunning) {
        this.maxCapsulesRunning = maxCapsulesRunning;
    }



    public long getSchedulerSleepInterval() {
        return schedulerSleepInterval;
    }



    public void setSchedulerSleepInterval(long schedulerSleepInterval) {
        this.schedulerSleepInterval = schedulerSleepInterval;
    }



    public String getSchedulerStrategy() {
        return schedulerStrategy;
    }



    public void setSchedulerStrategy(String schedulerStrategy) {
        this.schedulerStrategy = schedulerStrategy;
    }



    public boolean isDeadlockDetectorEnabled() {
        return deadlockDetectorEnabled;
    }



    public void setDeadlockDetectorEnabled(boolean deadlockDetectorEnabled) {
        this.deadlockDetectorEnabled = deadlockDetectorEnabled;
    }



    public long getDeadlockDetectorSleepInterval() {
        return deadlockDetectorSleepInterval;
    }



    public void setDeadlockDetectorSleepInterval(long deadlockDetectorSleepInterval) {
        this.deadlockDetectorSleepInterval = deadlockDetectorSleepInterval;
    }



    public int getFiberInfoHttpPort() {
        return fiberInfoHttpPort;
    }



    public void setFiberInfoHttpPort(int fiberInfoHttpPort) {
        this.fiberInfoHttpPort = fiberInfoHttpPort;
    }
}
