package san.core.fiber.strategy;



import java.util.Queue;
import san.core.Process;



/**
 *
 * @author Jiri Nohavec
 */
public class PriorityStrategy extends ISchedulerStrategy {



    public PriorityStrategy(int cpuCount, Queue<Process> allProcessQueue) {
        super(cpuCount, allProcessQueue);
    }



    @Override
    protected void reschedule() {
        Process bestProcess;
        long bestPriorityNumber;
        selectedProcessQueue.clear();
        for (int i = 0; i < cpuCount; i++) {
            bestProcess = null;
            bestPriorityNumber = Long.MAX_VALUE;
            for (Process process: allProcessQueue) {
                switch (process.getState()) {
                    case RUNNING:
                    case READY_TO_RUN:
                        if ((bestProcess == null || process.getPriorityNumber() < bestPriorityNumber)
                                && !selectedProcessQueue.contains(process)) {
                            bestProcess = process;
                            bestPriorityNumber = process.getPriorityNumber();
                        }
                        break;
                }
            }
            if (bestProcess != null) {
                selectedProcessQueue.add(bestProcess);
            } else {
                break;
            }
        }
    }
}
