/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.ident;

import java.io.Serializable;


/**
 * <b>Title:</b> Smart Active Node
 * <b>Description:</b> Identifier of the node. Identifications is in address and network.
 * @author Jan Boháč, Michal Rejda
 * @since 26.11.2006
 * @version 1.1
 */
public class NetIdentifier implements Comparable<NetIdentifier>, Serializable{

	/**
	 * @uml.property  name="address"
	 * @uml.associationEnd  multiplicity="(1 1)" inverse="netIdentifier:san.ident.Guid"
	 */
	private Guid address = null;
	
	/** 
	 * @uml.property name="network"
	 * @uml.associationEnd multiplicity="(1 1)" inverse="netIdentifier:san.ident.Guid"
	 */
	private Guid network = null;
	
	/**
	 * Construct identifier from identifier.
	 */
	public NetIdentifier(NetIdentifier ident) {
		this.address = new Guid(ident.address);
		this.network = new Guid(ident.network);
	}
	
	/**
	 * Construct nodes identifier. 
	 * @param address address of the node
	 * @param network network identifier
	 */
	public NetIdentifier(Guid address, Guid network) {
		this.address = address;
		this.network = network;
	}
	
	public int compareTo(NetIdentifier ident) {
		return Math.abs(ident.getAddress().compareTo(address)) + 
			   Math.abs(ident.getNetwork().compareTo(network));
	}
	
	/**
	 * Return hash code as hesh code of address.
	 */
	public int hashCode() {
		return address.hashCode();	//using hash code of the address
	}
	
	public boolean equals(Object arg0) {
		NetIdentifier i = (NetIdentifier) arg0;
		return (i.address.equals(address) && i.network.equals(network));
	}
	
	public String toString() {
		return address.toString().substring(33) + ", " + network.toString().substring(33);
	}

	/**
	 * Getter of the property <tt>address</tt>
	 * @return   Returns the address.
	 * @uml.property   name="address"
	 */
	public Guid getAddress() 
	{
		return address;
	}

	/**
	 * Setter of the property <tt>address</tt>
	 * @param address   The address to set.
	 * @uml.property   name="address"
	 */
	public void setAddress(Guid address) 
	{
		this.address = address;
	}

	/**
	 * Getter of the property <tt>network</tt>
	 * @return  Returns the network.
	 * @uml.property  name="network"
	 */
	public Guid getNetwork() {
		return network;
	}

	/**
	 * Setter of the property <tt>network</tt>
	 * @param network  The network to set.
	 * @uml.property  name="network"
	 */
	public void setNetwork(Guid network) {
		this.network = network;
	}
}
