/*************************************************************************
 *                           Smart Active Node                           * 
 *                      copyright (C) 2007 KIV-FAV-ZCU                   *
 *************************************************************************
 * The licence for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.interpreter;

import san.core.Process;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Interpreter interface. It is used by scheduler for code
 * executing. This interface must implement each interpret.<br>
 * @author Michal Rejda
 * @since 20.10.2007
 * @version 1.0
 */
public interface IInterpreter {

	/**
	 * Give code for interpreting with its context.
	 */
	public void runProcess(Process executable);

	/**
	 * Terminate code interpreting and return it with its context. Return null,
	 * if nothing is interpreted.
	 */
	public Process terminate();

	/**
	 * Return program code with its context, but interpreting is not stopped.
	 */
	public Process getProcess();

	/**
	 * Return true if interpreter is interpreting some code. Return false if
	 * interpreter has nothing to do.
	 */
	public boolean isInterpreting();
}
