package san.interpreter.sanint.classLoading;

import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.HashMap;

import org.apache.log4j.Logger;

import san.applicationRepository.SanCodePkg;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.References;
import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.fiber.override.IMethodCall;
import san.interpreter.sanint.fiber.override.MethodCallOverride;
import san.interpreter.sanint.fiber.stack.CallStackLevel;
import san.interpreter.sanint.fiber.staticm.StaticManager;

import com.sun.org.apache.bcel.internal.Constants;
import com.sun.org.apache.bcel.internal.classfile.ClassParser;
import com.sun.org.apache.bcel.internal.classfile.Constant;
import com.sun.org.apache.bcel.internal.classfile.ConstantClass;
import com.sun.org.apache.bcel.internal.classfile.ConstantDouble;
import com.sun.org.apache.bcel.internal.classfile.ConstantFieldref;
import com.sun.org.apache.bcel.internal.classfile.ConstantFloat;
import com.sun.org.apache.bcel.internal.classfile.ConstantInteger;
import com.sun.org.apache.bcel.internal.classfile.ConstantInterfaceMethodref;
import com.sun.org.apache.bcel.internal.classfile.ConstantLong;
import com.sun.org.apache.bcel.internal.classfile.ConstantMethodref;
import com.sun.org.apache.bcel.internal.classfile.ConstantNameAndType;
import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import com.sun.org.apache.bcel.internal.classfile.ConstantString;
import com.sun.org.apache.bcel.internal.classfile.ConstantUtf8;
import com.sun.org.apache.bcel.internal.classfile.Field;
import com.sun.org.apache.bcel.internal.classfile.JavaClass;
import com.sun.org.apache.bcel.internal.classfile.Method;

import san.security.SecurityMonitor;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;


public class HardClass implements Class {
	
	private static final transient Logger log = Logger.getLogger(HardClass.class);
	
	ClassManager classManager;
    private final ReferenceConverter referenceConverter;
    private IMethodCall methodCall;
	
	Class superClass = null;
	JavaClass cl = null;
	
	HashMap<String, MethodInfo> methods = new HashMap<String, MethodInfo>();
	HashMap<Integer, VariableInfo> variableOffsets = new HashMap<Integer, VariableInfo>();
	HashMap<String, VariableInfo> stringVariableOffsets = new HashMap<String, VariableInfo>();
	
	
	HashMap<String, StaticVariableInfo> staticVariableOffsets = new HashMap<String, StaticVariableInfo>();
	ByteBuffer staticData = null;
	Reference[] staticReferences = null;
	int staticDataLength = 0;
	int numStaticReferences = 0;
	
	
	int dataLength = 0;
	int numReferences = 0;

	protected HardClass(String className, SanCodePkg pkg, ClassManager manager)
	throws Exception
	{
		classManager = manager;
        referenceConverter = new ReferenceConverter(manager);
        methodCall = null;

		cl = new ClassParser(pkg.getClass(className), className).parse();
		
		if (log.isTraceEnabled()) {
			//System.out.println("Nacitam tridu " + cl.getClassName());
			log.trace("Nacitam tridu " + cl.getClassName());
		}
		
		for(Method m : cl.getMethods())
		{
			//if (m.isNative())
			//{
			//	System.out.println("\tMetoda " + m.getName() + " je nativni a tudiz ignorovana");
			//}
			//else
			//{
				methods.put(getMethodKey(m.getName(), m.getSignature()),
							new MethodInfo(m, this));
			//}
		}
		
		for(int i = 1; i < cl.getConstantPool().getLength(); i++)
		{
			Constant constant = cl.getConstantPool().getConstant(i);
			if (constant != null && constant.getTag() == 9)
			{
				ConstantFieldref fc = (ConstantFieldref)constant;
				ConstantNameAndType nameConstant = 
						(ConstantNameAndType)cl.getConstantPool().getConstant(fc.getNameAndTypeIndex());
				ConstantUtf8 signature = 
						(ConstantUtf8)cl.getConstantPool().getConstant(nameConstant.getSignatureIndex());
				
				ConstantUtf8 name = 
					(ConstantUtf8)cl.getConstantPool().getConstant(nameConstant.getNameIndex());
				boolean staticVar = false;
				for(Field f : cl.getFields())
				{
					if (f.getNameIndex() == nameConstant.getNameIndex())
					{
						char c = signature.getBytes().charAt(0);
						if (f.isStatic())
						{
							if (c == 'L' || c == '[')
							{
								staticVariableOffsets.put(name.getBytes(), new StaticVariableInfo(c, numStaticReferences));
								numStaticReferences++;
							}
							else
							{
								int length = Math.abs(HardClass.getVariableLength(c));
								staticVariableOffsets.put(name.getBytes(), new StaticVariableInfo(c, staticDataLength));
								staticDataLength += length;
							}
						}
						else
						{
							int length = Math.abs(HardClass.getVariableLength(c));
							if (c == 'L' || c == '[')
							{
								VariableInfo info = new VariableInfo(length, numReferences, c);
								variableOffsets.put(i, info);
								stringVariableOffsets.put(name.getBytes(), info);
								numReferences++;
							}
							else
							{
								VariableInfo info = new VariableInfo(length, dataLength, c);
								variableOffsets.put(i, info);
								stringVariableOffsets.put(name.getBytes(), info);
								dataLength += length;
							}
						}
						break;
					}
				}
				

			}
		}
		staticData = ByteBuffer.allocate(Math.max(1, staticDataLength));
		staticReferences = new Reference[numStaticReferences];
		if (cl.getSuperclassName() != null && !cl.getClassName().equals(cl.getSuperclassName()))
			superClass = manager.getClass(cl.getSuperclassName());
	}
	
	
	
	protected HardClass(String filename, ClassManager manager)
	throws Exception
	{
		classManager = manager;
        referenceConverter = new ReferenceConverter(manager);
        methodCall = null;

		int ind = filename.indexOf('!');
		if (ind >= 0)
		{
			String jarFileName = filename.substring(0, ind);
			String fileName = filename.substring(ind + 2).replace('\\', '/'); /* Windows VM likes to reverse this in URI - don't know why... */
            
            try {
                cl = new ClassParser(jarFileName, fileName).parse();
            } catch (Exception e) {
                throw new Exception("Could not load " + jarFileName + " " + fileName);
            }
		}
		else
		{
			//filename = filename.substring(0);
			cl = new ClassParser(filename).parse();
		}
		
		
		if (log.isTraceEnabled()) {
			//System.out.println("Nacitam tridu " + cl.getClassName());
			log.trace("Nacitam tridu " + cl.getClassName());
		}
		
		
		for(Method m : cl.getMethods())
		{
			//if (m.isNative())
			//{
			//	System.out.println("\tMetoda " + m.getName() + " je nativni a tudiz ignorovana");
			//}
			//else
			//{
				methods.put(getMethodKey(m.getName(), m.getSignature()),
							new MethodInfo(m, this));
			//}
		}
		
		for(int i = 1; i < cl.getConstantPool().getLength(); i++)
		{
			Constant constant = cl.getConstantPool().getConstant(i);
			if (constant != null && constant.getTag() == 9)
			{
				ConstantFieldref fc = (ConstantFieldref)constant;
				ConstantNameAndType nameConstant = 
						(ConstantNameAndType)cl.getConstantPool().getConstant(fc.getNameAndTypeIndex());
				ConstantUtf8 signature = 
						(ConstantUtf8)cl.getConstantPool().getConstant(nameConstant.getSignatureIndex());
				
				ConstantUtf8 name = 
					(ConstantUtf8)cl.getConstantPool().getConstant(nameConstant.getNameIndex());
				boolean staticVar = false;
				for(Field f : cl.getFields())
				{
					if (f.getNameIndex() == nameConstant.getNameIndex())
					{
						char c = signature.getBytes().charAt(0);
						if (f.isStatic())
						{
							if (c == 'L' || c == '[')
							{
								staticVariableOffsets.put(name.getBytes(), new StaticVariableInfo(c, numStaticReferences));
								numStaticReferences++;
							}
							else
							{
								int length = Math.abs(HardClass.getVariableLength(c));
								staticVariableOffsets.put(name.getBytes(), new StaticVariableInfo(c, staticDataLength));
								staticDataLength += length;
							}
						}
						else
						{
							int length = Math.abs(HardClass.getVariableLength(c));
							if (c == 'L' || c == '[')
							{
								VariableInfo info = new VariableInfo(length, numReferences, c);
								variableOffsets.put(i, info);
								stringVariableOffsets.put(name.getBytes(), info);
								numReferences++;
							}
							else
							{
								VariableInfo info = new VariableInfo(length, dataLength, c);
								variableOffsets.put(i, info);
								stringVariableOffsets.put(name.getBytes(), info);
								dataLength += length;
							}
						}
						break;
					}
				}
				

			}
		}
		staticData = ByteBuffer.allocate(Math.max(1, staticDataLength));
		staticReferences = new Reference[numStaticReferences];
		if (cl.getSuperclassName() != null && !cl.getClassName().equals(cl.getSuperclassName()))
			superClass = manager.getClass(cl.getSuperclassName());
	}

	protected int getVariableInfos(ConstantPool constants, int thisClassIndex)
	throws Exception
	{
		throw new NotImplementedException();
	}
	public static int getVariableLength(char type)
	throws Exception
	{
		switch(type)
		{
			case 'I': //integer
			case 'F': return 4; //float
			case 'D': //double
			case 'J': return 8; //long
			case 'Z': //boolean			
			case 'B': return 1; //byte
			case 'C': //char
			case 'S': return 2; //short
			case '[': 			//array
			case 'L': return -4; //reference
		}
		throw new Exception("Unknown variable type - " + type);
	}
	public void getConstant(int index, InterpreterStack stack, References references)
	throws Exception
	{
		ConstantUtf8 utfConst = null;
		String str = null;
		Constant constant = cl.getConstantPool().getConstant(index);
		if (constant == null)
			System.out.println();
		switch(constant.getTag())
		{
		case Constants.CONSTANT_Integer:
			stack.write(((ConstantInteger)constant).getBytes());
			break;
		case Constants.CONSTANT_Double:
			stack.write(((ConstantDouble)constant).getBytes());
			break;
		case Constants.CONSTANT_Float:
			stack.write(((ConstantFloat)constant).getBytes());
			break;			
		case Constants.CONSTANT_Long:
			stack.write(((ConstantLong)constant).getBytes());
			break;
		case Constants.CONSTANT_String:
			utfConst = (ConstantUtf8)cl.getConstantPool().getConstant(((ConstantString)constant).getStringIndex());
			str = utfConst.getBytes();
			stack.write(
					references.putReference(new SoftReference(str, this.classManager)));
			break;
		case Constants.CONSTANT_Class:
			utfConst = (ConstantUtf8)cl.getConstantPool().getConstant(((ConstantClass)constant).getNameIndex());
			java.lang.Class constClass = java.lang.Class.forName(utfConst.getBytes().replace('/', '.'));
			stack.write(
					references.putReference(new SoftReference(constClass, classManager)));	
			break;
		default:
			throw new NotImplementedException();
		}

	}
	
	
	
	public String getClassType(int index)
	{
		ConstantClass constant = (ConstantClass)cl.getConstantPool().getConstant(index);
		ConstantUtf8 ret = (ConstantUtf8)cl.getConstantPool().getConstant(constant.getNameIndex());
		return ret.getBytes();
	}
	public String getName()
	{
		return cl.getClassName().replace('.', '/');
	}
	protected Class getSuperClass()
	{
		return superClass;
	}
	protected int getNumReferences()
	{
		return numReferences;
	}
	public boolean canBeCasted(String type)
	{
		if (cl.getClassName().equals(type))
			return true;
		String []ifs = cl.getInterfaceNames();
		for(String s : ifs)
			if (type.equals(s))
				return true;
	
		if (superClass != null)
			return superClass.canBeCasted(type);
		
		return false;
		 
	}
	public HashMap<String, VariableInfo> getVariableInfos()
	{
		return stringVariableOffsets;
	}
	
	protected ConstantPool getConstants()
	{
		return cl.getConstantPool();
	}

	public MethodInfo getMethodOnlyInThisClass(String name, String descriptor)
	{
		return methods.get(getMethodKey(name, descriptor));
	}
	public MethodInfo getMethod(String name, String descriptor)
	{
		MethodInfo ret =  methods.get(getMethodKey(name, descriptor));
		if (ret == null && superClass != null)
			return superClass.getMethod(name, descriptor);
		return ret;
			
	}
	protected MethodInfo getMethod(int index)
	throws Exception
	{
		Constant constant = cl.getConstantPool().getConstant(index);
		ConstantNameAndType name = null;
		ConstantClass methodClass = null;
		if (constant instanceof ConstantMethodref)
		{
			ConstantMethodref meth = (ConstantMethodref)constant;
			name = (ConstantNameAndType)cl.getConstantPool().getConstant(meth.getNameAndTypeIndex());
			methodClass = (ConstantClass)cl.getConstantPool().getConstant(meth.getClassIndex());
		}
		else
		{
			ConstantInterfaceMethodref meth = (ConstantInterfaceMethodref)constant;
			name = (ConstantNameAndType)cl.getConstantPool().getConstant(meth.getNameAndTypeIndex());
			methodClass = (ConstantClass)cl.getConstantPool().getConstant(meth.getClassIndex());
		}
			
		ConstantUtf8 classConst = (ConstantUtf8)cl.getConstantPool().getConstant(methodClass.getNameIndex());
		ConstantUtf8 nameConst = (ConstantUtf8)cl.getConstantPool().getConstant(name.getNameIndex());
		ConstantUtf8 signConst = (ConstantUtf8)cl.getConstantPool().getConstant(name.getSignatureIndex());
        methodCall = MethodCallOverride.tryOverride(String.valueOf(classConst.getBytes()),
                                                    String.valueOf(nameConst.getBytes()),
                                                    String.valueOf(signConst.getBytes()));
        if (methodCall != null && methodCall.isReturnAfterExecution()) {
            return null;
        }
		Class c = classManager.getClass(classConst.getBytes());
		MethodInfo ret =  c.getMethod(nameConst.getBytes(), signConst.getBytes());
        // pokud metoda neni nalezena, musime jeste prozkoumat dalsi rozhrani
        if (ret == null && c instanceof HardClass) {
            HardClass hcl = (HardClass) c;
            for (String interfaceName : hcl.cl.getInterfaceNames()) {
                c = classManager.getClass(interfaceName);
                ret = c.getMethod(nameConst.getBytes(), signConst.getBytes());
                if (ret != null)
                    return ret;
            }
        }
		
		if (ret == null && superClass != null)
			return superClass.getMethod(nameConst.getBytes(), signConst.getBytes());

                return ret;
	}
	protected int getVariableOffset(int index)
	{
		VariableInfo ret = variableOffsets.get(index);
		if (ret != null)
			return ret.getOffset();
		else return -1;
	}
	protected int getVariableOffset(String name)
	{
		VariableInfo ret = stringVariableOffsets.get(name);
		if (ret != null)
			return ret.getOffset();
		else return -1;
	}
	public int getVariableLength(int index)
		throws Exception
	{
	
		ConstantFieldref fieldRef = (ConstantFieldref)cl.getConstantPool().getConstant(index);
		ConstantNameAndType nameAndType = (ConstantNameAndType)
				cl.getConstantPool().getConstant(fieldRef.getNameAndTypeIndex());
		return HardClass.getVariableLength(nameAndType.getSignature(cl.getConstantPool()).charAt(0));
	}
	public String getVariableName(int index)
	{
		ConstantFieldref fieldRef =  (ConstantFieldref)cl.getConstantPool().getConstant(index);
		ConstantNameAndType nameAndType = (ConstantNameAndType)
						cl.getConstantPool().getConstant(fieldRef.getNameAndTypeIndex());
		return nameAndType.getName(cl.getConstantPool());
	}
	protected int getDataLength()
	{

		return dataLength;
	}
	
	public void runMethod(int index, 
			References interpreterReferences, 
			InterpreterStack stack, 
			LocalVars localVars,
			boolean staticMethod,
			boolean virtualMethod, san.core.Process process)
	throws Exception
	{
        CallStackLevel callStackLevel = process.getInterpretingContext().getNewCall();
		MethodInfo method = getMethod(index);
        if (methodCall != null) {
            methodCall.execute(interpreterReferences, stack, process);
            if (methodCall.isReturnAfterExecution()) {
                return;
            }
        }

                //test if we can run this method (consulting with security monitor)
                if (method != null)
                {
                    String methodToRun = method.currentClass.getName().replace("/", ".") + "." + method.getName();

                    String owner = process.getOwner();
                    SecurityMonitor securityMonitor = SecurityMonitor.getInstance();
                    
                    if (!securityMonitor.canUserUse(owner, methodToRun, SecurityMonitor.UseType.Method))
                    {
                        throw new Exception(SecurityMonitor.NoPermissionError);
                    }
                }

		if (method == null)
		{
                    Constant c = cl.getConstantPool().getConstant(index);
                    ConstantNameAndType nameAndType = null;
                    ConstantClass classInfo = null;
                    if (c instanceof ConstantMethodref)
                    {
                            ConstantMethodref methC = (ConstantMethodref)c;
                            nameAndType = (ConstantNameAndType)cl.getConstantPool().getConstant(methC.getNameAndTypeIndex());
                            classInfo = (ConstantClass)cl.getConstantPool().getConstant(methC.getClassIndex());
                    }
                    else
                    {
                            ConstantInterfaceMethodref methC = (ConstantInterfaceMethodref)c;
                            nameAndType = (ConstantNameAndType)cl.getConstantPool().getConstant(methC.getNameAndTypeIndex());
                            classInfo = (ConstantClass)cl.getConstantPool().getConstant(methC.getClassIndex());
                    }
                    String name  = nameAndType.getName(cl.getConstantPool());
                    String descriptor = nameAndType.getSignature(cl.getConstantPool());
                    ConstantUtf8 className = (ConstantUtf8)cl.getConstantPool().getConstant(classInfo.getNameIndex());

                    if (log.isDebugEnabled()) {
				/*System.out.println("Metoda nenalezena: " + className.getBytes() + "::" +
						name + " - " + descriptor); */
				log.debug("Metoda nenalezena: " + className.getBytes() + "::" +
						name + " - " + descriptor);
			}
                        //tato vyjimka brani v normalnim spousteni aplikace netAnt a mozna i dalsich, bude ji nutno dale predelat
			//throw new ClassNotFoundException("Metoda nenalezena: " + className.getBytes() + "::" +name);
                        return;
		}
		int argsSize = method.getArgumentsLength();
		int specialFrom = stack.stackPointer() - argsSize;
		if (!staticMethod)
			specialFrom -= 4;

		
		if (method.isNative())
		{
			ByteBuffer specialLocalVarsBuffer = 
				ByteBuffer.wrap(Arrays.copyOfRange(stack.getData(), specialFrom, 
								specialFrom + NativeMethodSupport.MAX_LOCALS * 4));
			LocalVars specialLocalVars = new LocalVars(specialLocalVarsBuffer);
			NativeMethodSupport sup = new NativeMethodSupport();
			sup.runNativeMethod(method, stack, interpreterReferences, specialLocalVars, specialFrom, classManager);
			return;
		}	
		if(method.isAbstract()) //metoda je abstraktni -> prechazime na nadtridu
		{
			int refIndex = ByteBuffer.wrap(Arrays.copyOfRange(stack.getData(), specialFrom, 
						specialFrom + 4)).getInt();
			Reference ref = interpreterReferences.getReference(refIndex);
			while (true)
			{
				method = ref.getClassInfo().getMethod(method.getName(), method.getDescriptor());
				if (ref instanceof SoftReference)
				{
					((SoftReference) ref).runMethod(method, interpreterReferences, stack, classManager);
					return;
				}
				if (method != null && method.method.getCode() != null)
					break;
				ref = ref.getSuperReference();
			}
		}
//		if(method.getName().equals("parseGuid"))// &&
				//method.getClassInfo().getName().equals("java/lang/StringBuilder"))// &&
				//method.getDescriptor().equals("()Ljava/lang/String"))
//			System.out.println();
		byte bytecode[] = null;
		int maxLocals = 0;		
		try 
		{
			bytecode 	= method.getByteCode();
			maxLocals 	= method.getMaxLocals();
		} 
		catch (Exception e) 
		{
				e.printStackTrace();
		}
			
        ByteBuffer specialLocalVarsBuffer =
			ByteBuffer.wrap(Arrays.copyOfRange(stack.getData(), specialFrom, 
							specialFrom + maxLocals * 4));
		LocalVars specialLocalVars = new LocalVars(specialLocalVarsBuffer);
	
		if (log.isTraceEnabled()) {
			log.trace("Spousti se: " + method.getClassInfo().getName() + "::" + method.getName() + " - " + method.getDescriptor());
			System.out.println("Spousti se: " + method.getClassInfo().getName() + "::" + method.getName() + " - " + method.getDescriptor());
		}
		
		
	
		if (!staticMethod)
		{
			Reference r = interpreterReferences.getReference( specialLocalVars.GetLocalReference(0));
			if (r == null) // zavolana metoda na null pointru
				throw new NullPointerException(method.getClassInfo().getName() + "::" + method.getName() + " - " + method.getDescriptor());

			if (virtualMethod)
			{
				method = r.getClassInfo().getMethod(method.cachedName, method.cachedDescriptor);
				bytecode = method.getByteCode();
                // kvuli prekrytym metodam musime znovu nacist pocet lokalnich promennych (napr. toString)
                maxLocals = method.getMaxLocals();
                specialLocalVarsBuffer =
                        ByteBuffer.wrap(Arrays.copyOfRange(stack.getData(), specialFrom,
                        specialFrom + maxLocals * 4));
                specialLocalVars = new LocalVars(specialLocalVarsBuffer);
			}
			
			if (r instanceof HardReference) {
//				interpreter.Run(bytecode, method.getClassInfo(), interpreterReferences, stack, specialLocalVars, specialFrom, process);
                callStackLevel.set(bytecode,
                                   method.getClassInfo(),
                                   interpreterReferences,
                                   stack,
                                   specialLocalVars,
                                   specialFrom);
            } else {
                ((SoftReference) r).runMethod(method, interpreterReferences, stack, classManager);
            }
			if (log.isTraceEnabled()) {
				//System.out.println("Ukoncuje se: " + r.getClassInfo().getName() + "::" + method.getName());
				log.trace("Ukoncuje se: " + r.getClassInfo().getName() + "::" + method.getName());
			}
		}
		else
		{
			ConstantMethodref methodInfo = (ConstantMethodref)cl.getConstantPool().getConstant(index);
			ConstantClass classInfo = (ConstantClass)cl.getConstantPool().getConstant(methodInfo.getClassIndex());
			String type = ((ConstantUtf8)cl.getConstantPool().getConstant(classInfo.getNameIndex())).getBytes();
			Class cl = this.classManager.getClass(type);
			if (cl instanceof HardClass) {
//				interpreter.Run(bytecode, method.getClassInfo(), interpreterReferences, stack, specialLocalVars, specialFrom, process);
                callStackLevel.set(bytecode,
                                   method.getClassInfo(),
                                   interpreterReferences,
                                   stack,
                                   specialLocalVars,
                                   specialFrom);
            } else {
				SoftReference.runStaticMethod((SoftClass)cl , method, interpreterReferences, stack, classManager);
            }
			if (log.isTraceEnabled()) {
				//System.out.println("Ukoncuje se: " + method.getClassInfo().getName() + "::" + method.getName());
				log.trace("Ukoncuje se: " + method.getClassInfo().getName() + "::" + method.getName());
			}
			
		}
		
	}
	public void getStaticVariable(String name, InterpreterStack stack,
			References references) throws Exception{
		StaticVariableInfo varInfo = staticVariableOffsets.get(name);
		if (varInfo == null)
		{
			superClass.getStaticVariable(name, stack, references);
			return;
		}
        if (StaticManager.getInstance().getStatic(getName(),
                                                  name,
                                                  varInfo,
                                                  stack,
                                                  references,
                                                  staticDataLength,
                                                  numStaticReferences,
                                                  referenceConverter)) {
            return;
        }
		switch(varInfo.getType())
		{
		case 'I': //integer
		case 'F': //float
			int intData = staticData.getInt(varInfo.getOffset());
			stack.write(intData);
			break;
		case 'C': //char
		case 'S': //short
			short shortData = staticData.getShort(varInfo.getOffset());
			stack.write((int)shortData);
			break;
		case 'Z': //boolean			
		case 'B': //byte
			byte byteData = staticData.get(varInfo.getOffset());
			stack.write((int)byteData);
			break;
		case 'D': //double
		case 'J': //long
			long longData = staticData.getLong(varInfo.getOffset());
			stack.write(longData);
			break;
		case '[': //array
		case 'L': //reference
			Reference ref = staticReferences[varInfo.getOffset()];
			stack.write(references.putReference(ref));
			break;
		}
	}
	public void setStaticVariable(String name, InterpreterStack stack, References references) throws Exception 
	{
		StaticVariableInfo varInfo = staticVariableOffsets.get(name);
        if (StaticManager.getInstance().setStatic(getName(),
                                                  name,
                                                  varInfo,
                                                  stack,
                                                  references,
                                                  staticDataLength,
                                                  numStaticReferences,
                                                  referenceConverter)) {
            return;
        }
		switch(varInfo.getType())
		{
		case 'I': //integer
		case 'F': //float
			int intData = stack.readInt();
			staticData.putInt(varInfo.getOffset(), intData);
			break;
		case 'C': //char
		case 'S': //short
			short shortData = (short)stack.readInt();
			staticData.putShort(varInfo.getOffset(), shortData);
			break;
		case 'Z': //boolean			
		case 'B': //byte
			byte byteData = (byte)stack.readInt();
			staticData.put(varInfo.getOffset(), byteData);
			break;
		case 'D': //double
		case 'J': //long
			long longData = stack.readLong();
			staticData.putLong(varInfo.getOffset(), longData);
			break;
		case '[': //array
		case 'L': //reference
			int refIndex = stack.readRef();
			Reference ref = references.getReference(refIndex);
			staticReferences[varInfo.getOffset()] = ref;
			break;
		}		
		
	}	
	private String getMethodKey(String name, String descriptor)
	{
		return name + "*" + descriptor;
	}

}
