package san.interpreter.sanint.classLoading;


import java.io.DataInputStream;
import java.util.ArrayList;

import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import com.sun.org.apache.bcel.internal.classfile.Method;

public class MethodInfo {
	Method method;
	String cachedName = null;
	String cachedDescriptor = null;
	
	HardClass currentClass;
    
    public static final String nameInit = "<init>";
    public static final String nameClinit = "<clinit>";


    MethodInfo(String name, String descriptor)
    {
    	cachedName = name;
    	cachedDescriptor = descriptor;
    	method = null;
    }
    MethodInfo(Method method, HardClass currentClass)
    {
    	this.method = method;
    	this.cachedName = method.getName();
    	this.cachedDescriptor = method.getSignature();
    	this.currentClass = currentClass;
    }
    
    
    public byte[] getByteCode()
    {
    	if (method == null)
    		return null;
    	return method.getCode().getCode();
    }
    public int getMaxLocals()
    {
    	if (method == null)
    		return 1;
    	return method.getCode().getMaxLocals();
    }
    public int getNameIndex()
    {
    	return method.getNameIndex();
    }
    public String getName()
    {
    	return cachedName;
    }
    public int getDescriptorIndex()
    {
    	return method.getSignatureIndex();
    }
    public String getDescriptor()
    {
    	return cachedDescriptor;
    }
    public HardClass getClassInfo()
    {
    	return currentClass;
    }

    public boolean isNative()
    {
    	return method.isNative();
    }
    public boolean isStatic()
    {
    	return method.isStatic();
    }
    public boolean isAbstract()
    {
    	return method.isAbstract();
    }
    public ConstantPool getConstants()			
    {
    	return method.getConstantPool();
    }
    
    
    public int getArgumentsLength()
    throws Exception
    {
    	String descStr = cachedDescriptor;
    	int ret = 0;
    	String desc = descStr.substring(1, descStr.lastIndexOf(')'));
    	for(int i = 0; i < desc.length(); i++)
    	{
    		ret += Math.max(4, Math.abs(HardClass.getVariableLength( desc.charAt(i))));
    		if (desc.charAt(i) == '[') {
    			while(desc.charAt(i) == '[')
    				i++;
    		}    		
    		if (desc.charAt(i) == 'L') {
    			while(desc.charAt(i) != ';')
    				i++;
    		}

    	}
    	return ret;
    }
    
    
    @Override
	public String toString()
    {
    	String ret = "Name: " + cachedName + "\n";
    	ret += "Descriptor: " + cachedDescriptor + "\n";
    	return ret;
    	
    }
}
