package san.interpreter.sanint.classLoading;

import java.util.HashMap;

import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.References;


public abstract class Reference {
	boolean locked = false;
	long instanceId = 0;
	
	public void lock()	{
		synchronized(this) {
			locked = true;
		}
	}
	public void unlock() {
		synchronized(this) {
			locked = false;
		}
	}
	public long getInstanceId() {
		return instanceId;
	}
	public void setInstanceId(long instanceId) {
		this.instanceId = instanceId;
	}	
	
	/**Sets variable value
	 * @param currentClass Class of method, from this variable is to set
	 * @param index index of variable in constant pool
	 * @param stack current stack
	 * @param refs list of references
	 * @param value value to be set
	 * @throws Exception
	 */
	public abstract void setVariable(HardClass currentClass, int index, InterpreterStack stack, References refs, long value)
		throws Exception;
	/**Gets variable value
	 * @param name name of variable
	 * @param length length of variable in bytes
	 * @param stack current stack
	 * @param refs list of references
	 * @throws Exception
	 */
	public abstract void getVariable(String name, int length, InterpreterStack stack, References refs)
		throws Exception;
	
	
	/**Gets variable value
	 * @param current Class Class of method, from this variable is to set
	 * @param index index of variable in constant pool
	 * @param stack current stack
	 * @param refs list of references
	 * @throws Exception
	 */
	public abstract void getVariable(HardClass currentClass, int index, InterpreterStack stack, References refs)
		throws Exception;
	/**Sets variable value
	 * @param name name of variable
	 * @param length length of variable in bytes
	 * @param stack current stack
	 * @param refs list of references
	 * @param value value to be set
	 * @throws Exception
	 */
	public abstract void setVariable(String name, int length, InterpreterStack stack, References refs, long value)
		throws Exception;
	public abstract boolean checkCast(Class cl, int index);
	
	public abstract Reference getSuperReference() throws Exception;

	/**
	 * @return
	 */
	public abstract boolean isNull();
	public abstract Class getClassInfo();
	
	public abstract void fromObject(HashMap<Long, Object> convertedObjects, Object newData)
	throws Exception;
	
	public abstract boolean isInstanceOf(HardClass currentClass, int classIndex)
		throws Exception;
}
