package san.interpreter.sanint.fiber;



import java.util.Deque;
import java.util.LinkedList;
import san.interpreter.IInterpretingContext;
import san.interpreter.sanint.classLoading.ClassManager;
import san.interpreter.sanint.fiber.stack.CallStackLevel;
import san.interpreter.sanint.fiber.stack.StaticVariableCallStackLevel;
import san.interpreter.sanint.fiber.stack.SystemInitCallStackLevel;



/**
 *
 * @author Jiri Nohavec
 */
public class FiberInterpretingContext implements IInterpretingContext {



    private final ClassManager classManager;
    private final Deque<CallStackLevel> callStack;



    public FiberInterpretingContext(ClassManager classManager) {
        this.classManager = classManager;
        callStack = new LinkedList<CallStackLevel>();
    }



    @Override
    public ClassManager getClassManager() {
        return classManager;
    }



    @Override
    public CallStackLevel getNewCall() {
        CallStackLevel callStackLevel = new CallStackLevel();
        callStack.push(callStackLevel);
        return callStackLevel;
    }



    @Override
    public SystemInitCallStackLevel getNewSystemInitCall() {
        SystemInitCallStackLevel systemInitCallStackLevel = new SystemInitCallStackLevel();
        callStack.push(systemInitCallStackLevel);
        return systemInitCallStackLevel;
    }



    @Override
    public StaticVariableCallStackLevel getNewStaticVariableCall() {
        StaticVariableCallStackLevel staticVariableCallStackLevel = new StaticVariableCallStackLevel();
        callStack.push(staticVariableCallStackLevel);
        return staticVariableCallStackLevel;
    }



    @Override
    public CallStackLevel popCall() {
        while (!callStack.peek().isUsed()) {
            callStack.pop();
        }
        return callStack.pop();
    }



    @Override
    public CallStackLevel peekCall() {
        while (!callStack.peek().isUsed()) {
            callStack.pop();
        }
        return callStack.peek();
    }



    @Override
    public int getCallStackSize() {
        return callStack.size();
    }



    @Override
    public boolean isCallStackEmpty() {
        return callStack.isEmpty();
    }



    @Override
    public void replaceReturnValue(boolean newValue) {
        peekCall().getStack().readInt();
        if (newValue) {
            peekCall().getStack().write(1);
        } else {
            peekCall().getStack().write(0);
        }
    }



    @Override
    public void replaceReturnValue(int newValue) {
        peekCall().getStack().readInt();
        peekCall().getStack().write(newValue);
    }
}
