package san.interpreter.sanint.interpreter;
import java.nio.ByteBuffer;
import java.util.Arrays;

public class InterpreterStack { 
	public static final int BYTE_BYTES 		= 1;
	public static final int SHORT_BYTES 	= 2;
	public static final int INT_BYTES	 	= 4;
	public static final int FLOAT_BYTES 	= 4;
	public static final int LONG_BYTES 		= 8;
	public static final int DOUBLE_BYTES 	= 8;
	
	//public static final int NULL_REFERENCE = -1;

	
	private byte data[];
	private ByteBuffer stack = null;
	
	public InterpreterStack(int stackCapacity) {
		this.data = new byte[stackCapacity];
		this.stack = ByteBuffer.wrap(data);
	}

	
	public byte[] read(int numBytes)
	{
		byte[] data = new byte[numBytes];
				
		this.stack.position(stack.position() - numBytes);
		this.stack.get(data, 0, numBytes);
		this.stack.position(stack.position() - numBytes);
			
		return data;
	}
	
	public byte readByte() {
		byte ret = (byte)stack.getInt(stack.position() - INT_BYTES);
		setPosition(INT_BYTES);

		return ret;
	}
	
	
	public double readDouble() {
		double ret = stack.getDouble(stack.position() - DOUBLE_BYTES);
		setPosition(DOUBLE_BYTES);

		return ret;
	}

	public float readFloat() {
		float ret = stack.getFloat(stack.position() - FLOAT_BYTES);
		//float ret = stack.getFloat();
		setPosition(FLOAT_BYTES);

		return ret;
	}

	public int readInt() {
		int ret = stack.getInt(stack.position() - INT_BYTES);
		setPosition(INT_BYTES);

		return ret;
	}
	
	public long readLong() {
		long ret = stack.getLong(stack.position() - LONG_BYTES);
		setPosition(LONG_BYTES);

		return ret;
	}

	public int readRef()
	{
		return this.readInt();
	}

	public short readShort() {
		short ret = (short)stack.getInt(stack.position() - INT_BYTES);
		setPosition(INT_BYTES);

		return ret;
	}
	
	
	private void setPosition(int numBytes) {
		stack.position(stack.position() - numBytes);
	}


	public void write(byte value)
	{
		stack.putInt(value);
	}

	public int write(byte[] bytecode, int index, int count)
	{
		byte[] value = Arrays.copyOfRange(bytecode, index, index + count);
		stack.put(value);

		return index + count - 1;
	}

	public void write(double value) {
		stack.putDouble(value);
	}

	public void write(float value) {
		stack.putFloat(value);
	}

	public void write(int value) {
		stack.putInt(value);
	}

	public void write(long value) {
		stack.putLong(value);
	}

	public void write(short value) {
		stack.putInt(value);
	}
	
	public void writeRef(int ref) {
		this.write(ref);
	}
	
	public void clearStack()
	{
		this.stack.clear();
	}
	
	public void setStackPointer(int index)
	{
		this.stack.position(index);
	}
	
	public byte[] getData()
	{
		return this.data;
	}
	
	public int stackPointer()
	{
		return this.stack.position();
	}
	
	public static void printByteArray(byte[] array)
	{
		for(int a = 0; a < array.length; a++)
		{
			System.out.print(array[a]);
			if(a != array.length - 1)
				System.out.print(", ");
		}
		
		System.out.println();
	}
}
