/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.logging;

import san.logging.ILogger.LogType;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This is a class with factory method to make Logger.<br>
 * @author Michal Rejda
 * @since 27.9.2007
 * @version 1.1
 * @uml.dependency supplier="san.logging.ILogger" stereotypes="Standard::Create"
 */
public class LoggerFactory {

	public enum LogOutput {
		CONSOLE, FILE
	}

	/**
	 * Define witch output is used for logging. Default is set FILE for logging.
	 */
	public static LogOutput logOutput = LogOutput.FILE;
	/**
	 * Define logging level. Default is set ERROR level.
	 */
	public static LogType logLevel = LogType.ERROR;
	/**
	 * Name of file into is log written.
	 */
	public static final String LOG_FILE_NAME = "log.txt";
	
	private static ILogger fileLogger = null;

	/**
	 * Get the logger class.
	 */
	public static ILogger createLogger() {
		switch (logOutput) {
			case CONSOLE :
				return new ConsoleLogger(logLevel);
			default :
				if(fileLogger == null)
					fileLogger = new FileLogger(LOG_FILE_NAME, logLevel);
				return fileLogger;
		}
	}

	/*
	 * Main method used by testing.
	 */
	public static void main(String[] args) throws Exception {
		LoggerFactory.logOutput = LogOutput.FILE;
		LoggerFactory.logLevel = LogType.DEBUG;
		ILogger logger = LoggerFactory.createLogger();
		logger.Log("Debug message", LogType.DEBUG);
		Thread.sleep(100);
		logger.Log("Information message", LogType.INFORMATION);
		Thread.sleep(100);
		logger.Log("Warning message", LogType.WARNING);
		Thread.sleep(100);
		logger.Log("Error message", LogType.ERROR);
		Thread.sleep(100);
	}
}
