/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.networkInterfaces;

/**
 * <b>Title:</b> Smart Active Node
 * <b>Description:</b> Accessiable method to access to the network interface.
 * @author Jan Boháč, Michal Rejda
 * @since 28.11.2006
 * @version 1.0
 */
public class AccessibleMethod implements Comparable<AccessibleMethod>{
	
	/**
	 * Method to sending and recieving is TCP.
	 */
	public static final int TCP = 0;
	
	/**
	 * Method to sending and recieving is UDP.
	 */
	public static final int UDP = 1;
	
	/**
	 * Unknown method to sending.
	 */
	public static final int UNKNOWN = 255;
	
	/**
	 * Priority of using this number.
	 */
	public int priority;
	
	/**
	 * Number of the method.
	 */
	public int number;
	
	/**
	 * Create a accessible methot with UNKOWN number of method
	 * and minimal priority for using.
	 */
	public AccessibleMethod() {
		this.number = UNKNOWN;
		this.priority = Integer.MIN_VALUE;
	}
	
	/**
	 * Create a accessible method.
	 * @param priority priority of using this metod
	 * @param number	number of this method
	 */
	public AccessibleMethod(int priority, int number) {
		this.priority = priority;
		this.number = number;
	}
	
	/**
	 * To sort with priority.
	 */
	public int compareTo(AccessibleMethod arg0) {
		return (arg0.priority - priority);
	}
	
	/**
	 * Equals is in the type of number.
	 */
	public boolean equals(Object arg0) {
		AccessibleMethod am = (AccessibleMethod) arg0;
		return am.number == number;
	}
}
