/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import san.networkInterfaces.IInterfacesChanged;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import san.ident.NetIdentifier;
import java.util.Collection;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class contains and manage all network interfaces.<br>
 * @author Michal Rejda
 * @since 4.12.2007
 * @version 2.0
 */
public class NetInterfaces {

	/**
	 * @uml.property name="netInterfaces"
	 * @uml.associationEnd readOnly="true" multiplicity="(1 1)"
	 *                     inverse="netInterfaces:san.networkInterfaces.NetInterface"
	 *                     qualifier="identifier:san.ident.NetIdentifier
	 *                     san.networkInterfaces.NetInterface"
	 */
	private Map<NetIdentifier, NetInterface> netInterfaces;
	/**
	 * @uml.property name="interfaceChangedListeners"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="netInterfaces1:san.networkInterfaces.IInterfacesChanged"
	 */
	private Collection<IInterfacesChanged> interfaceChangedListeners;

	public NetInterfaces() {
		this.netInterfaces = new HashMap<NetIdentifier, NetInterface>();
		this.interfaceChangedListeners = new LinkedList<IInterfacesChanged>();
	}

	/**
	 * Add new network interface.
	 * @param netInterface new network interface.
	 */
	public void addNetInterface(NetInterface netInterface) {
		netInterfaces.put(netInterface.getIdentifier(), netInterface);
		fireInterfaceAdded(netInterface);
	}

	/**
	 * Remove network interface.
	 * @param identifier network interface identifier
	 */
	public void remmoveNetInterface(NetIdentifier identifier) {
		NetInterface interfc = netInterfaces.remove(identifier);
		if (interfc != null)
			fireInterfaceRemoved(interfc);
	}

	/**
	 * Getter of the property <tt>netInterfaces</tt>
	 * @return Returns the netInterfaceMap.
	 * @uml.property name="netInterfaces"
	 */
	public Map<NetIdentifier, NetInterface> getNetInterfaces() {
		return netInterfaces;	
	}

	/**
	 * Add new interfaces changed listener.
	 */
	public void addInterfacesChangedListener(IInterfacesChanged listener) {
		this.interfaceChangedListeners.add(listener);
	}

	/**
	 * Remove interfaces changed listener.
	 */
	public void removeInterfaceChangedListener(IInterfacesChanged listener) {
		this.interfaceChangedListeners.remove(listener);
	}

	private void fireInterfaceRemoved(NetInterface interfc) {
		for (IInterfacesChanged listener : interfaceChangedListeners) {
			listener.interfaceRemoved(interfc);
		}
	}

	private void fireInterfaceAdded(NetInterface interfc) {
		for (IInterfacesChanged listener : interfaceChangedListeners) {
			listener.interfaceAdded(interfc);
		}
	}
}
