/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import java.util.Collection;
import java.util.LinkedList;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class receive capsules from another node.<br>
 * @author Jan Boháč, Michal Rejda
 * @since 30.10.2006
 * @version 1.0
 */
public abstract class NetListener {

	/**
	 * @uml.property name="pastReceiving"
	 * @uml.associationEnd aggregation="shared"
	 *                     inverse="netListener:san.networkInterfaces.IPastReceiving"
	 */
	IPastReceiving pastReceiving;
	/**
	 * @uml.property name="remoteInterface"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="netListener:san.networkInterfaces.RemoteInterface"
	 */
	RemoteInterface remoteInterface = null;
	/**
	 * @uml.property name="connectionLostListeners"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="netListener:san.networkInterfaces.IConnectionLostListener"
	 */
	private Collection<IConnectionLostListener> connectionLostListeners;
	/**
	 * Indicate if the Listener listen.
	 */
	boolean running;

	/**
	 * Construct net listener.
	 */
	NetListener(RemoteInterface remoteInterface) {
		this.remoteInterface = remoteInterface;
		this.connectionLostListeners = new LinkedList<IConnectionLostListener>();
		this.pastReceiving = null;
	}

	/**
	 * Start of listen.
	 */
	abstract void listen();

	/**
	 * Stop of listen.
	 */
	abstract void stop();

	/**
	 * Getter of the property <tt>pastReceiving</tt>
	 * @return Returns the pastReceiving.
	 * @uml.property name="pastReceiving"
	 */
	public IPastReceiving getPastReceiving() {
		return pastReceiving;
	}

	/**
	 * Setter of the property <tt>pastReceiving</tt>
	 * @param pastReceiving The pastReceiving to set.
	 * @uml.property name="pastReceiving"
	 */
	public void setPastReceiving(IPastReceiving pastReceiving) {
		this.pastReceiving = pastReceiving;
	}

	/**
	 * Add new connection lost listener.
	 */
	public void addConnectionLostListener(IConnectionLostListener conLostList) {
		this.connectionLostListeners.add(conLostList);
	}

	/**
	 * Remove connection lost listener.
	 */
	public void removeConnectionLostListener(IConnectionLostListener conLostList) {
		this.connectionLostListeners.remove(conLostList);
	}

	/**
	 * Fire connection lost listeners.
	 */
	void fireConnectionLostListeners() {
		for (IConnectionLostListener listener : connectionLostListeners) {
			listener.connectionLost(this, remoteInterface);
		}
	}

	/**
	 * Getter of the property <tt>remoteInterface</tt>
	 * @return Returns the remoteInterface.
	 * @uml.property name="remoteInterface"
	 */
	public RemoteInterface getRemoteInterface() {
		return remoteInterface;
	}

	/**
	 * Setter of the property <tt>remoteInterface</tt>
	 * @param remoteInterface The remoteInterface to set.
	 * @uml.property name="remoteInterface"
	 */
	public void setRemoteInterface(RemoteInterface remoteInterface) {
		this.remoteInterface = remoteInterface;
	}
}
