/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.LinkedList;
import java.util.List;

import org.apache.log4j.Logger;

import san.data.TransmitCapsule;
import san.localization.Localization;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class allow to incoming TCP connection for Capsules
 * transport.<br>
 * @author Jan Boháč, Michal Rejda
 * @since 30.10.2006
 * @version 1.0
 */
public class TcpListener extends NetListener {
	
	private static final transient Logger log = Logger.getLogger(TcpListener.class);
	
	// Controls of incoming requests.
	private List<TcpControl> controls;
	// Socket on it listen the TcpListener.
	private ServerSocket socket;
	// thread for listen
	private Thread thread;

	TcpListener(RemoteInterface remoteInterface) {
		super(remoteInterface);
		this.running = false;
		this.socket = ((TcpConnection) remoteInterface.getConnection());
		controls = new LinkedList<TcpControl>();
	}

	void listen() {
		thread = new Thread() {

			public void run() {
				running = true;
				try {
					TcpControl con = null;
					while (running) {
						LoggerFactory.createLogger().Log(
								Localization.getLocalization().getString(
										"M000023"), LogType.DEBUG);
						con = new TcpControl(socket.accept());
						controls.add(con);
						con.start();
					}
				}
				catch (SocketException se) {
					LoggerFactory.createLogger()
							.Log(
									Localization.getLocalization().getString(
											"M000024"), LogType.ERROR);
				}
				catch (IOException e) {
					Object[] values = new Object[]{socket.getLocalPort()};
					LoggerFactory.createLogger().Log(
							Localization.getLocalization().getString("M000025",
									values), LogType.ERROR);
				}
				finally {
					socket = null;
					running = false;
				}
			}
		};
		thread.start();
	}

	void stop() {
		running = false;
		try {
			socket.close();
		}
		catch (IOException e) {
			LoggerFactory.createLogger().Log(
					Localization.getLocalization().getString("M000026"),
					LogType.ERROR);
		}
		finally {
			if (socket.isClosed())
				socket = null;
		}
	}

	/**
	 * <b>Title:</b> Small Active Network <b>Description:</b> This class
	 * control the incomming request.
	 * @author Jan Boháč, Michal Rejda
	 * @since 31.10.2006
	 * @version 1.0
	 */
	private class TcpControl extends Thread {

		
		
		// the socket through it is the communication running
		private Socket socket;

		public TcpControl(Socket socket) {
			this.socket = socket;
		}

		public void run() {
			try {
				LoggerFactory.createLogger().Log(
						Localization.getLocalization().getString("M000027"),
						LogType.DEBUG);
				InputStream is = socket.getInputStream();
				ObjectInputStream ois = new ObjectInputStream(is);
				LoggerFactory.createLogger().Log(
						Localization.getLocalization().getString("M000028"),
						LogType.DEBUG);
				TransmitCapsule c;
				while(true){
					c = (TransmitCapsule) ois.readObject(); // read capsule
//					if (log.isDebugEnabled()) {
//						log.debug("read capsule " + c);
//					}

					if (pastReceiving != null) {
						pastReceiving.received(c);
					}
				}
			}
			catch (IOException e) {
				controls.remove(this); // remove from list
				log.error("Fall when reading socket.", e);
			}
			catch (ClassNotFoundException cnfe) {
				controls.remove(this); // remove from list
				log.error("The TransmitCapsule was not read from the socket.", cnfe);
				
			}
		}
	}
}
