/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;

import org.apache.log4j.Logger;

import san.data.TransmitCapsule;

/**
 * <b>Title:</b> Smart Active Node <br>
 * <b>Description:</b>This class send capsules by using TCP connection.<br>
 * @author Jan Boháč, Michal Rejda
 * @since 3.11.2006
 * @version 1.0
 */
public class TcpOut extends NetOut {
	
	private static final transient Logger log = Logger.getLogger(TcpOut.class);
	Socket socket = null; // socket to sending capsules
	OutputStream os = null;
	ObjectOutputStream oos = null;
	
	TcpOut(RemoteInterface remoteInterface) {
		super(remoteInterface);
		try{
			socket = new Socket(remoteInterface.getInetAddress(),
					remoteInterface.getPort());
			log.debug("Socket add:" + remoteInterface.getInetAddress() + ", port: " + remoteInterface.getPort());
			os = socket.getOutputStream();
			oos = new ObjectOutputStream(os);
		}catch(IOException e){
			fireConnectionLostListeners();
		}
	}

	public void send(TransmitCapsule c) {
//		if (log.isTraceEnabled()) {
//			log.trace("send(" + c + ")");
//		}
		try {
			oos.writeObject(c);
			oos.flush();
			os.flush();
		}
		catch (IOException e) {
			fireConnectionLostListeners();
		}
	}
}
