/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.routing;

import java.util.Collection;
import java.util.Map;
import java.util.SortedSet;

import san.ident.NetIdentifier;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This interface provide routing changes.<br>
 * @author Michal Rejda
 * @since 30.10.2007
 * @version 1.0
 */
public interface IRoutingTableControl {

	/**
	 * Return collection of neigbours or null if is empty.
	 * 
	 * public Collection<RoutingRecord> getDirectNeighbours();
	 */
	public Collection getDirectNeighbours();

	/**
	 * Return all routing records in routing table. Key is destination and value
	 * is sorted collection of ways into the destination.
	 * 
	 * public Map<NetIdentifier, SortedSet<RoutingRecord>> getRoutingRecords();
	 */
	public Map getRoutingRecords();

	/**
	 * Add routing record in routing records map.
	 */
	public void add(String description, NetIdentifier destination,
			NetIdentifier gateway, int metric);

	/**
	 * Update metric for specific destination end gateway.
	 */
	public void updateMetric(NetIdentifier destination, NetIdentifier gateway,
			int newMetric);

	/**
	 * Remove all routing records with specific destination.
	 */
	public void remove(NetIdentifier destination);

	/**
	 * Remove routing record with specific destination and gateway.
	 */
	public void remove(NetIdentifier destination, NetIdentifier gateway);
}
