/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.softstate;

import san.ident.CodePkgIdent;
import san.ident.Guid;

/**
 * <b>Title:</b> Smart Active Node<br> 
 * <b>Description:</b> Interface to use soft state.<br>
 * @author Michal Rejda
 * @since 8.2.2008
 * @version 1.0
 */
public interface ISoftState {

	/**
	 * Add object to soft state.
	 * @param value object to store into soft state
	 * @param time max time of existing object in the soft state
	 * @return random generated guid (key)
	 */
	public Guid add(Object value, int time);

	/**
	 * Add object to soft state.
	 * @param guid key associated with the object
	 * @param value object to store into soft state
	 * @param time max time of existing object in the soft state
	 */
	public void add(Guid guid, Object value, int time);

	/**
	 * Add object to soft state.
	 * @param codeIdent identification of the code that store object into soft state 
	 * @param guid key associated with the object
	 * @param value object to store into soft state
	 * @param time max time of existing object in the soft state
	 */
	public void add(CodePkgIdent codeIdent, Guid guid, Object value, int time);

	/**
	 * Returns associated value with the guid (key)
	 */
	public Object get(Guid guid);

	/**
	 * Returns associated value with the guid (key)
	 */
	public Object get(CodePkgIdent codeIdent, Guid guid);
}
