import java.util.ArrayList;
import java.util.List;



interface FiberThread {



    public void executeOneInstruction();



    public boolean isAlive();
}



class SimpleFiberTask implements FiberThread {



    private final int id;

    private int i;
    private double d;



    public SimpleFiberTask(int id) {
        this.id = id;
        i = 0;
        d = 0.0;
    }



    public void executeOneInstruction() {
        if (i == 0) {
            System.out.format("--- Task %d started%n", id);
        }
        d += Math.sin(Math.random() * 55.55) * Math.cos(Math.random() * -33.33) * 0.44;
        if (i % 1000000 == 0) {
            System.out.format("Task %d in iteration %d: %f%n", id, i, d);
        }
        i++;
        if (i >= 100000000) {
            System.out.format("--- Task %d finished%n", id);
        }
    }



    public boolean isAlive() {
        return (i < 100000000);
    }
}



class SimpleFiberScheduler extends Thread {



    private static final long TIME_QUANTUM = 5000L;

    private final List<FiberThread> threadList;

    private int lastIndex;



    public SimpleFiberScheduler(List<FiberThread> threadList) {
        this.threadList = threadList;
        lastIndex = 0;
    }



    private FiberThread selectThreadToSchedule() {
        if (threadList.isEmpty()) {
            return null;
        }
        if (lastIndex >= threadList.size()) {
            lastIndex = 0;
        }
        if (!threadList.get(lastIndex).isAlive()) {
            threadList.remove(lastIndex);
            return null;
        }
        return threadList.get(lastIndex++);
    }



    public void run() {
        System.out.println("--- Scheduler started");
        FiberThread selectedThread;
        long startTime;
        while (!threadList.isEmpty()) {
            selectedThread = selectThreadToSchedule();
            if (selectedThread != null && selectedThread.isAlive()) {
                startTime = System.currentTimeMillis();
                while (System.currentTimeMillis() < startTime + TIME_QUANTUM && selectedThread.isAlive()) {
                    selectedThread.executeOneInstruction();
                }
            }
        }
        System.out.println("--- Scheduler ended");
    }
}



public class SimpleFiberSchedulerTest {



    public static void main(String[] args) {
        List<FiberThread> threadList = new ArrayList<FiberThread>();
        threadList.add(new SimpleFiberTask(0));
        threadList.add(new SimpleFiberTask(1));
        threadList.add(new SimpleFiberTask(2));
        threadList.add(new SimpleFiberTask(3));
        SimpleFiberScheduler sfs = new SimpleFiberScheduler(threadList);
        sfs.start();
        try {
            sfs.join();
        } catch (InterruptedException e) {
        }
    }
}
