import java.util.ArrayList;
import java.util.List;



class SimpleTask extends Thread {



    private final int id;



    public SimpleTask(int id) {
        this.id = id;
        this.setPriority(Thread.MIN_PRIORITY);
    }



    public void run() {
        System.out.format("--- Task %d started%n", id);
        double d = 0.0;
        for (int i = 0; i < 100000000; i++) {
            d += Math.sin(Math.random() * 55.55) * Math.cos(Math.random() * -33.33) * 0.44;
            if (i % 1000000 == 0) {
                System.out.format("Task %d in iteration %d: %f%n", id, i, d);
            }
        }
        System.out.format("--- Task %d finished%n", id);
    }
}



class SimpleScheduler extends Thread {



    private static final long TIME_QUANTUM = 5000L;

    private final List<Thread> threadList;

    private int lastIndex;



    public SimpleScheduler(List<Thread> threadList) {
        this.threadList = threadList;
        lastIndex = 0;
    }



    private Thread selectThreadToSchedule() {
        if (threadList.isEmpty()) {
            return null;
        }
        if (lastIndex >= threadList.size()) {
            lastIndex = 0;
        }
        if (!threadList.get(lastIndex).isAlive()) {
            threadList.remove(lastIndex);
            return null;
        }
        return threadList.get(lastIndex++);
    }



    public void run() {
        System.out.println("--- Scheduler started");
        this.setPriority(Thread.MAX_PRIORITY);
        Thread selectedThread;
        while (!threadList.isEmpty()) {
            selectedThread = selectThreadToSchedule();
            if (selectedThread != null && selectedThread.isAlive()) {
                selectedThread.setPriority(Thread.NORM_PRIORITY);
                try {
                    Thread.sleep(TIME_QUANTUM);
                } catch (InterruptedException e) {
                }
                selectedThread.setPriority(Thread.MIN_PRIORITY);
            }
        }
        System.out.println("--- Scheduler ended");
    }
}



public class SimpleSchedulerTest {



    private static boolean stop = false;



    public static void main(String[] args) {
        for (int i = 1; i < Runtime.getRuntime().availableProcessors(); i++) {
            new Thread() {
                public void run() {
                    System.out.println("--- CPU limiter started");
                    this.setPriority(Thread.MAX_PRIORITY);
                    while (!stop);
                    System.out.println("--- CPU limiter ended");
                }
            }.start();
        }
        List<Thread> threadList = new ArrayList<Thread>();
        threadList.add(new SimpleTask(0));
        threadList.add(new SimpleTask(1));
        threadList.add(new SimpleTask(2));
        threadList.add(new SimpleTask(3));
        SimpleScheduler ss = new SimpleScheduler(threadList);
        for (Thread thread: threadList) {
            thread.start();
        }
        ss.start();
        try {
            ss.join();
        } catch (InterruptedException e) {
        }
        stop = true;
    }
}
