package fiber;



import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public class ConditionFeature extends IFeature {



    private static final int SIGNAL_ONE_COUNT = 5;
    private static final long SLEEP_TIME = 2000L;

    private static int conditionId;



    public ConditionFeature() {
        super("Creating and removing conditions with methods "
              + "applicationAPI.conditionInitialize/Destroy(int), "
              + "waiting on conditions with methods applicationAPI.conditionWait(int [, long]) "
              + "and signaling with methods applicationAPI.conditionSignalOne/All(int)");
    }



    @Override
    protected void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        if (getWorkerIndex() == 0) {
            for (int i = 0; i < SIGNAL_ONE_COUNT; i++) {
                Thread.sleep(SLEEP_TIME);
                println("Still waiting on condition: " + applicationAPI.conditionGetWaitingCount(conditionId));
                Thread.sleep(SLEEP_TIME);
                println("Signal one");
                applicationAPI.conditionSignalOne(conditionId);
            }
            Thread.sleep(SLEEP_TIME);
            println("Still waiting on condition: " + applicationAPI.conditionGetWaitingCount(conditionId));
            Thread.sleep(SLEEP_TIME);
            println("Signal all");
            applicationAPI.conditionSignalAll(conditionId);
            Thread.sleep(SLEEP_TIME);
            println("Still waiting on condition: " + applicationAPI.conditionGetWaitingCount(conditionId));
            Thread.sleep(SLEEP_TIME);
        } else {
            println("Waiting on condition...");
            applicationAPI.conditionWait(conditionId);
            println("Signal received");
        }
    }



    @Override
    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        conditionId = (int) (Math.random() * Integer.MAX_VALUE);
        applicationAPI.conditionInitialize(conditionId);
        println("Initialized condition " + conditionId);
    }



    @Override
    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        applicationAPI.conditionDestroy(conditionId);
        println("Removed condition " + conditionId);
    }
}
