package fiber;



import java.io.PrintWriter;
import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public abstract class IFeature {



    private final static int DEFAULT_WORKER_COUNT = 10;
    private final static int FARMER_INDEX = -1;

    private final String description;

    private int workerCount;
    private int workerIndex;
    private PrintWriter out;



    public IFeature(String description) {
        this.description = description;
    }



    public String getDescription() {
        return description;
    }



    protected int getWorkerCount() {
        return workerCount;
    }



    protected int getWorkerIndex() {
        return workerIndex;
    }



    protected void println(String message) {
        if (workerIndex == FARMER_INDEX) {
            out.print("Farmer: " + message + "\n");
        } else {
            out.print("Worker " + String.valueOf(workerIndex) + ": " + message + "\n");
        }
    }



    protected void println() {
        out.println();
    }



    protected void formatln(String formatString, Object... args) {
        if (workerIndex == FARMER_INDEX) {
            out.format("Farmer: " + formatString + "%n", args);
        } else {
            out.format("Worker " + String.valueOf(workerIndex) + ": " + formatString + "%n", args);
        }
    }



    protected abstract void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception;



    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
    }



    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
    }



    private void farmerMethod(String[] args, IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Starting workers");
        Guid[] workerGuidArray = new Guid[workerCount];
        for (int i = 0; i < workerCount; i++) {
            workerGuidArray[i] = applicationAPI.runApplication("fiber",
                                                               new String[] {args[0],
                                                                             String.valueOf(workerCount),
                                                                             String.valueOf(i)});
        }
        for (int i = 0; i < workerCount; i++) {
            applicationAPI.waitForApplication(workerGuidArray[i]);
            println("Joined worker " + String.valueOf(i));
        }
    }



    public void run(String[] args, IApplicationAPI applicationAPI, Guid guid) throws Exception {
        workerCount = DEFAULT_WORKER_COUNT;
        workerIndex = FARMER_INDEX;
        out = applicationAPI.getStdOut();
        if (args.length == 2 || args.length == 3) {
            workerCount = Integer.parseInt(args[1]);
        }
        if (args.length == 3) {
            workerIndex = Integer.parseInt(args[2]);
        }
        if (args.length == 1 || args.length == 2) {
            println("Starting");
            beforeFarmerMethod(applicationAPI, guid);
            farmerMethod(args, applicationAPI, guid);
            afterFarmerMethod(applicationAPI, guid);
            println("Ended");
            println();
        } else if (args.length == 3) {
            println("Starting");
            workerMethod(applicationAPI, guid);
            println("Ended");
        }
    }
}
