package fiber;



import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public class MonitorFeature extends IFeature {



    private static final int INCREMENT_COUNT = 10;
    private static final long SLEEP_TIME = 200L;

    private static Object lock;
    private static int sharedCounter;



    public MonitorFeature() {
        super("Accessing shared context in static variables with "
              + "block \"synchronized (Object) {}\" or methods applicationAPI.monitorEnter/Exit(Object), "
              + "shared variables and locks must be static and manually initialized");
    }



    @Override
    protected void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Incrementing shared variable by " + INCREMENT_COUNT);
        for (int i = 0; i < INCREMENT_COUNT; i++) {
            Thread.sleep(SLEEP_TIME);
            synchronized (lock) {
                int temporaryCounter = sharedCounter;
                temporaryCounter++;
                Thread.sleep(SLEEP_TIME);
                sharedCounter = temporaryCounter;
            }
        }
    }



    @Override
    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        lock = new Object();
        sharedCounter = 0;
        println("Shared counter value is " + sharedCounter);
    }



    @Override
    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        int expectedValue = INCREMENT_COUNT * getWorkerCount();
        println("Shared counter value is " + sharedCounter + " (expected " + expectedValue + ")");
    }
}
