package fiber;



import san.core.IApplicationAPI;
import san.ident.Guid;



/**
 *
 * @author Jiri Nohavec
 */
public class MonitorNoFeature extends IFeature {



    private static final int INCREMENT_COUNT = 10;
    private static final long SLEEP_TIME = 200L;

    private static int sharedCounter;



    public MonitorNoFeature() {
        super("Accessing shared context in static variables without synchronization, "
              + "shows how important is synchronization in multithreaded application");
    }



    @Override
    protected void workerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        println("Incrementing shared variable by " + INCREMENT_COUNT);
        for (int i = 0; i < INCREMENT_COUNT; i++) {
            Thread.sleep(SLEEP_TIME);
            int temporaryCounter = sharedCounter;
            temporaryCounter++;
            Thread.sleep(SLEEP_TIME);
            sharedCounter = temporaryCounter;
        }
    }



    @Override
    protected void beforeFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        sharedCounter = 0;
        println("Shared counter value is " + sharedCounter);
    }



    @Override
    protected void afterFarmerMethod(IApplicationAPI applicationAPI, Guid guid) throws Exception {
        int expectedValue = INCREMENT_COUNT * getWorkerCount();
        println("Shared counter value is " + sharedCounter + " (expected " + expectedValue + ")");
    }
}
