/*************************************************************************
 *                           Smart Active Node                           *
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package rendezvous2;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.PrintWriter;
import san.RendezVous.IRendezvousServer;
import san.application.IApplication;
import san.core.DestinationNotAvailableException;
import san.core.IApplicationAPI;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;

/**
 * This application get String in command line parameter on server node
 * by rendezvous and transmit it by capsule to the client node
 * @author Miroslav Hendrych
 */
public class Rendezvous2 implements IApplication, ReceiveDataListener, IRendezvousServer {
    public static final String ENTRY_NAME = "QueryEntryCall";
    public static final Guid ENTRY_GUID = Guid.parseGuid("12345678-90AB-CDEF-1234-567890ABCDEF");
    private IApplicationAPI applicationAPI;
    private PrintWriter out;
    private String answer = null;

    public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
        this.out = applicationAPI.getStdOut();
        this.applicationAPI = applicationAPI;

        if (args.length<2 || !args[0].equals("-c") && !args[0].equals("-s")) {
            error_params(out);
            return;
        }
        if (args[0].equals("-c")) {
            if (args.length<3) {
                error_params(out);
                return;
            }
            app_client(args[1], args[2]);
        }
        if (args[0].equals("-s")) {
            answer = args[1];
            for (int i=2; i<args.length; i++)
                answer = answer.concat(" "+args[i]);
            app_server();
        }
    }
    
    public void error_params(PrintWriter out) {
        out.println("Nezadany parametry prikazove radky:");
        out.println(" pro server: -s retezec, ktery ma byt zjisten");
        out.println(" pro klienta: -c 000 000 (adresa a sit serveru)");
    }

    public void app_client(String network, String node) {
        NetIdentifier serverAddr;
        try {
            Guid guidNetwork = Guid.parseGuid(network);
            Guid guidNode = Guid.parseGuid(node);

            serverAddr = new NetIdentifier(guidNode, guidNetwork);
        } catch (NumberFormatException e) {
            out.print(e);
            return;
        }

        applicationAPI.addOnSendDataListener(applicationAPI.getId());
        try {
            TransmitData data = new TransmitData(new byte[0]);

            CodePkgIdent capsulePkgIdent = applicationAPI.getExecutedCodeIdent();

            applicationAPI.injectCapsule(capsulePkgIdent, serverAddr, data);
            out.println("Capsule odeslana na "+serverAddr);

            while (this.answer == null) {
                Thread.sleep(1L);
            }
            out.println("Odpoved: "+answer);
        } catch (DestinationNotAvailableException ex) {
            out.println("DestinationNotAvailableException");
            ex.printStackTrace(out);
        } catch (InterruptedException ex) {
            out.println("InterruptedException");
            ex.printStackTrace(out);
        }
    }

    public void app_server() {
        if (!applicationAPI.registerServer(this, ENTRY_GUID)) {
            out.println("Nepodarilo se zaregistrovat server");
            return;
        }
        applicationAPI.registerEntryCall(ENTRY_GUID, ENTRY_NAME);
        try {
            applicationAPI.makeAccept(ENTRY_GUID);
        } catch (InterruptedException ex) { 
            ex.printStackTrace(out);
        }
        applicationAPI.unregisterServer(ENTRY_GUID);
    }


    public Object entry(String name, Object param) {
        return answer;
    }

    public void receiveData(Guid sender, TransmitData data, IApplicationAPI applicationAPI) {
        ByteArrayInputStream bais = new ByteArrayInputStream(data.getData());
        ObjectInputStream ois;
        try {
            ois = new ObjectInputStream(bais);
            answer = ois.readUTF();
        } catch (IOException ex) {
            answer = "Chyba";
        }
    }

}
