/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package traceroute;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Date;

import org.apache.log4j.Logger;

import san.application.IApplication;
import san.core.IApplicationAPI;
import san.data.ReceiveDataListener;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;

/**
 * This application displays all network interfaces on the way
 * from local node to entered destination. It sends its capsule 
 * and displays data from it when it returns back.
 * 
 * @author Petr Stepanek
 */
public class Traceroute implements IApplication, ReceiveDataListener {

	private static final transient Logger log = Logger.getLogger(Traceroute.class);
	
	private Object lock = new Object();	
	private TransmitData received = null;
	
	public void main(String[] args, IApplicationAPI applicationAPI, Guid guid) {
		
		applicationAPI.addOnSendDataListener(guid);
		PrintWriter out = applicationAPI.getStdOut();
		
		try{	
			
			Guid toAddr = null;
			Guid toNet = null;
			NetIdentifier pingDst = null;
			
			try{			
				toAddr = Guid.parseGuid(args[0]);
				toNet = Guid.parseGuid(args[1]);
				pingDst	 = new NetIdentifier(toAddr,toNet);
			}catch(NumberFormatException e) {				
				out.print(e);
				return;
			}			

			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bos);

			oos.writeBoolean(false); //it's on the way to destination
			oos.writeObject(""); //path is empty now
			oos.close();
			
			TransmitData data = new TransmitData(bos.toByteArray());
		  	CodePkgIdent capsulePkgIdent = applicationAPI.getExecutedCodeIdent(); 

			applicationAPI.injectCapsule(capsulePkgIdent,pingDst,data, true);
			
			synchronized (lock) {
				//wait for the return of the capsule
				lock.wait();
			}
			
			if(received != null) {
			
				ByteArrayInputStream bais = new ByteArrayInputStream(received.getData());
				ObjectInputStream ois = new ObjectInputStream(bais);
				
				ois.readBoolean();
				String path = (String) ois.readObject();

				if (log.isDebugEnabled()) {
					log.debug("TRACEROUTE " + args[0] + " " + args[1] + " END " + path);
				}
				
			} else {
				out.println("Chyba pri prijmu dat od capsule.");
			}
		} catch(Exception e) {
			out.println(e);
		}
	}
	
	public void receiveData(Guid sender, TransmitData data, IApplicationAPI applicationAPI) {
	
		received = data;

		synchronized (lock) {
			//wake up the application
			lock.notify();	
		}
	}
}
