/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package traceroute;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Date;

import san.application.ICapsule;
import san.core.ICapsuleAPI;
import san.data.TransmitData;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;

/**
 * Purpose of this capsule is to record network
 * interfaces through which it traveled.
 * 
 * @author Petr Stepanek
 */
public class TracerouteCapsule implements ICapsule {
	
	public void main(ICapsuleAPI capsuleAPI) {
		try{
			
			TransmitData data = capsuleAPI.getDataLoad();			
			ByteArrayInputStream bais = new ByteArrayInputStream(data.getData());
			ObjectInputStream ois = new ObjectInputStream(bais);
						
			boolean onWayBack = ois.readBoolean();
			
			if(onWayBack) {
				
				if(capsuleAPI.isThisServer(capsuleAPI.getDestination())) {
					// is back on starting node
					
					String path = (String) ois.readObject();
					path = path + capsuleAPI.getPrev().toString() + " -> " + capsuleAPI.getOver().toString();
					
					ByteArrayOutputStream bos = new ByteArrayOutputStream();
					ObjectOutputStream oos = new ObjectOutputStream(bos);
					
					oos.writeBoolean(onWayBack);					
					oos.writeObject(path);					
					oos.close();

					TransmitData dataBack = new TransmitData(bos.toByteArray());					
					Guid sourceApp = capsuleAPI.getSourceApplication();					
					capsuleAPI.sendData(sourceApp, dataBack);					
					
					capsuleAPI.finished(); //the way of this capsule is over
				}
			} else {
				// on the way to destination
				
				NetIdentifier pingDestination = capsuleAPI.getDestination();
				if(capsuleAPI.isThisServer(pingDestination)){
					//has reached the destination
					
					onWayBack = true;
					
					capsuleAPI.setDestination(capsuleAPI.getSource());
					capsuleAPI.setSource(pingDestination);
					
				} else{
					
				}
			}
			
			//always record the path
			String path = (String) ois.readObject();
			path = path + capsuleAPI.getPrev().toString() + " -> " + capsuleAPI.getOver().toString() + " : ";
			
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(bos);
			
			oos.writeBoolean(onWayBack);
			oos.writeObject(path);			
			oos.close();

			TransmitData dataBack = new TransmitData(bos.toByteArray());
			capsuleAPI.setDataLoad(dataBack);
			
		}catch(Exception e) {
			LoggerFactory.createLogger().Log(e, LogType.ERROR);
		}
	}
}
