/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import san.localization.Localization;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Utility.java<br>
 * @author Michal Rejda
 * @since 3.7.2007
 * @version 1.0
 */
public class Utility {
	
	/**
	 * Create from string the byte array.
	 */
	public static byte[] toByteArray (String s) {
		char[] tmp = s.toCharArray();
		byte[] ret = new byte[tmp.length];
		for (int i = 0; i < tmp.length; i++) {
			ret[i] = (byte) (tmp[i] - '0');
		}
		return ret;
	}
	
	/**
	 * Create string from array of bytes as string of hexadecimal values.
	 */
	public static String toHexString(byte[] array) {
		char[] val = new char[2*array.length];
        String hex = "0123456789ABCDEF";
        for (int i = 0; i < array.length; i++) {
            int b = array[i] & 0xff;
            val[2*i] = hex.charAt(b >>> 4);
            val[2*i + 1] = hex.charAt(b & 15);
        }
        return String.valueOf(val);
	}
	
	/**
	 * Create bytes array from string of hexadecimal values.
	 * @throws NumberFormatException if cannot convert string to byte array
	 */
	public static byte[] fromHexString(String s) throws NumberFormatException {
		int byteLength = s.length() / 2;
		byte[] bytes = new byte[byteLength];
		String hex;
		int j = 0;
		for (int i=0; i< bytes.length; i++)
		{
			bytes[i] = (byte) Integer.parseInt(s.substring(2*i, 2*i+2), 16);
		}
		return bytes;
	}
	
	/**
	 * Translate port to array of bytes.
	 */
	public static byte[] intTo2ByteArray(int number) {
		byte[] array = new byte[2];
		array[1] = (byte) (number & 0xFF);
		number = Integer.rotateRight(number, 8);
		array[0] = (byte) (number & 0xFF);
		return array;
	}
	
	/**
	 * Translate array of bytes to the integer.
	 */
	public static int array2BytesToInt(byte[] array) {
		int port = 0;
		port += (((int) array[0]) & 0xFF);
		port = Integer.rotateLeft(port, 8);
		port += (((int) array[1]) & 0xFF);
		return port;
	}
	
	/**
	 * Returns the contents of the file in a byte array.
	 */
    public static byte[] getBytesFromFile(File file) throws IOException {
        InputStream is = new FileInputStream(file);
    
        // Get the size of the file
        long length = file.length();
    
        // You cannot create an array using a long type.
        // It needs to be an int type.
        // Before converting to an int type, check
        // to ensure that file is not larger than Integer.MAX_VALUE.
        if (length > Integer.MAX_VALUE) {
            throw new IOException(Localization.getLocalization().getString("M000003"));
        }
    
        // Create the byte array to hold the data
        byte[] bytes = new byte[(int)length];
    
        // Read in the bytes
        int offset = 0;
        int numRead = 0;
        while (offset < bytes.length && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
            offset += numRead;
        }
    
        // Ensure all the bytes have been read in
        if (offset < bytes.length) {
        	Object[] value = new Object[]{
        		file.getName()
        	};
            throw new IOException(Localization.getLocalization().getString("M000004",value));
        }
    
        // Close the input stream and return bytes
        is.close();
        return bytes;
    }
}
