/*                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.codeDistribution.strategies;

import java.util.Set;

import org.apache.log4j.Logger;

import san.applicationRepository.IRepository;
import san.codeDistribution.CapsuleTrafficLogger;
import san.core.Kernel;
import san.data.CodePkgData;
import san.data.TransmitCapsule;
import san.ident.NetIdentifier;
import san.routing.IRouting;

/*
 * A strategy that looks for an app always on the neighbouring node
 */
public class AskNeighbourDistributionStrategy extends
		CodeDistributionStrategy {
	
	private static final transient Logger log = Logger.getLogger(AskNeighbourDistributionStrategy.class);
		
	public AskNeighbourDistributionStrategy(IRepository repository,
			IRouting router, Kernel kernel, CapsuleTrafficLogger capsuleTrafficLogger) {

		super(repository, router, kernel, capsuleTrafficLogger);
	}
	
	@Override
	public void sendRequestForApp(TransmitCapsule originalCapsule) {
		if (log.isTraceEnabled()) {
			log.trace("sendRequestForApp(" + originalCapsule + ")");
		}
		
		assert !originalCapsule.isAplication();

		TransmitCapsule requestCapsule = new TransmitCapsule();
		requestCapsule.setSrc(null);
		requestCapsule.setPrev(null);
		requestCapsule.setDest(getBestAppLocation(originalCapsule));
		requestCapsule.setOver(null);
		requestCapsule.setIsRequestForAplication(true);
		requestCapsule.setHash(originalCapsule.getHash());

		kernel.sendCapsule(requestCapsule);

	}

	@Override
	public boolean shouldCapsuleBeStoredInQueue(TransmitCapsule originalCapsule) {
		
		if (((originalCapsule.isRequestForApplication() || !originalCapsule.isDestinationExecute())
				&& !router.isThisNode(originalCapsule.getDest()))) {

			return false;
		} else if (originalCapsule.isRequestForApplication()) {

			return false;
		}

		return true;		
	}
	
	@Override
	public void dealWithCapsuleAfterQueue(TransmitCapsule originalCapsule) {
		
		if (log.isTraceEnabled()) {
			log.trace("dealWithCapsuleAfterQueue(" + originalCapsule + ")");
		}
		if(shouldCapsuleBeStoredInQueue(originalCapsule)){
			if(originalCapsule.isAplication() && !router.isThisNode(originalCapsule.getDest())){
				
				originalCapsule.setOver(null);
				originalCapsule.setPrev(null);
				
				kernel.sendCapsule(originalCapsule);
				
				if (log.isDebugEnabled()) {
					log.debug("preposilam aplikaci dal");
				}
			}
		}			
		else{
			if (originalCapsule.isRequestForApplication() && router.isThisNode(originalCapsule.getDest())) {
				sendApplicationFor(originalCapsule);
				
			} else if(!router.isThisNode(originalCapsule.getDest())){
				
				originalCapsule.setOver(null);
				originalCapsule.setPrev(null);
				
				kernel.sendCapsule(originalCapsule);
				
				if (log.isDebugEnabled()) {
					log.debug("preposilam kapsuli dal");
				}	
			}
		}
	}

	@Override
	NetIdentifier getBestAppLocation(TransmitCapsule capsuleDataArrivedIn) {
		return capsuleDataArrivedIn.getPrev();
	}

	@Override
	void sendApplicationFor(TransmitCapsule originalCapsule) {
		
		if (log.isTraceEnabled()) {
			log.trace("sendApplicationFor(" + originalCapsule + ")");
		}

		CodePkgData cpd = repository.getCodePkgData(originalCapsule.getHash());
		
		if (cpd != null) {

		
			TransmitCapsule applicationCapsule = new TransmitCapsule();
			applicationCapsule.setSrc(originalCapsule.getDest());
			applicationCapsule.setPrev(null);
			applicationCapsule.setDest(originalCapsule.getSrc());
			applicationCapsule.setOver(null);
			applicationCapsule.setAplication(true);
			applicationCapsule.setHash(originalCapsule.getHash());
			applicationCapsule.setData(cpd);	

			kernel.sendCapsule(applicationCapsule);
		
		} else {
			log.debug("App not found on this node");			

			Set<NetIdentifier> ni = capsuleTrafficLogger.getAppPlacementInfo(originalCapsule.getHash()).getNodesDataCameFrom();
			
			for (NetIdentifier n : ni) {
				
				if(n.compareTo(originalCapsule.getSrc()) != 0){
				
					TransmitCapsule applicationCapsule = new TransmitCapsule();
					applicationCapsule.setSrc(originalCapsule.getSrc());
					applicationCapsule.setPrev(null);
					applicationCapsule.setDest(n);
					applicationCapsule.setOver(null);
					applicationCapsule.setAplication(false);
					applicationCapsule.setIsRequestForAplication(true);
					applicationCapsule.setHash(originalCapsule.getHash());
	
					kernel.sendCapsule(applicationCapsule);
					return;
				}
			}
			
			log.error("No useful information about app " + originalCapsule.getHash() + " on this node");
		}
	}
}
