/*                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.codeDistribution.strategies;

import org.apache.log4j.Logger;

import san.applicationRepository.AplPkgException;
import san.applicationRepository.CodePackage;
import san.applicationRepository.IRepository;
import san.codeDistribution.CapsuleTrafficLogger;
import san.core.Kernel;
import san.data.CodePkgData;
import san.data.TransmitCapsule;
import san.ident.NetIdentifier;
import san.outgoingQueue.OutgoingQueue;
import san.routing.IRouting;

/*
 * Common parent for classes that define a code distribution strategy.
 */
public abstract class CodeDistributionStrategy {
	
	static final transient Logger log = Logger.getLogger(CodeDistributionStrategy.class);
	final IRepository repository;
	final IRouting router;
	final Kernel kernel;
	final CapsuleTrafficLogger capsuleTrafficLogger;

	public CodeDistributionStrategy(IRepository repository,
			IRouting router, Kernel kernel, CapsuleTrafficLogger capsuleTrafficLogger) {		

		this.repository = repository;
		this.router = router;
		this.kernel = kernel;
		this.capsuleTrafficLogger = capsuleTrafficLogger;
	}
	
	/*
	 * Returns identification of a node, with highest probability of a location of an application
	 */
	abstract NetIdentifier getBestAppLocation(TransmitCapsule capsuleDataArrivedIn);
	
	/*
	 * Sends capsule with an application as a response to capsule with request, which is a parameter
	 */
	abstract void sendApplicationFor(TransmitCapsule originalCapsule); 	
	
	/*
	 * Sends a capsule with a request for an application. Application is specified
	 * in the parameter of a method call. 
	 */
	public abstract void sendRequestForApp(TransmitCapsule originalCapsule);
	
	/*
	 * Decides, whether the capsule (parameter) should or shouldn't be process by this node  
	 */
	public abstract boolean shouldCapsuleBeStoredInQueue(TransmitCapsule originalCapsule);

	/*
	 * Can do anything with a capsule after decision is made in method 'shouldCapsuleBeStoredInQueue'
	 */
	public abstract void dealWithCapsuleAfterQueue(TransmitCapsule originalCapsule);
	
}
