/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.core;

import java.util.Collection;
import san.data.TransmitCapsule;
import san.ident.CodePkgIdent;
import san.ident.NetIdentifier;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This interface is necessary for code distribution among nodes.<br>
 * @author Michal Rejda
 * @since 23.6.2007
 * @version 1.0
 */
public interface ICodeDistribution {
	
	/**
	 * This method is called out when capsule with applications data is accepted.
	 * @param aplCapsule accepted capsule
	 */
	public void acceptedAplicationsCapsule(TransmitCapsule aplCapsule);
	
	/**
	 * Gets information of accessibility application code in this node.
	 * @param aplIdentification identification of the applications code
	 * @return true if is code accessible in this node or false if it isn't
	 */
	public boolean isAplication(CodePkgIdent aplIdentification);
	
	/**
	 * Inform of necessity of applications code.
	 * @param aplIdentification identification of the applications code
	 * @return true if the code is accessible in this node or false if it isn't
	 */
	public boolean needAplicationFor(CodePkgIdent aplIdentification);
	
	/**
	 * Inform of necessity of applications code.
	 * @param aplIdentification identification of the applications code
	 * @param nodeIdent identification of node of it came the applications code identifier
	 * @return true if the code is accessible in this node or false if it isn't
	 */
	public boolean needAplicationFor(CodePkgIdent aplIdentification, NetIdentifier nodeIdent);
	
	/**
	 * Distribute applications code into another nodes.
	 * @param aplIdentification identification of applications code that will be distributed
	 * @param nodes collection of nodes into witch will be the code distributed
	 */
	public void distributeCode(CodePkgIdent aplIdentification, Collection<NetIdentifier> nodes);
}
