/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.util.Collection;
import san.applicationRepository.CodePackage;

import san.core.Process.PriorityClass;
import san.core.fiber.api.IInfoAPI;
import san.ident.CodePkgIdent;
import san.ident.Guid;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> Scheduler interface<br>
 * @author Michal Rejda
 * @since 20.10.2007
 * @version 1.0
 */
public interface IScheduler {

	/**
	 * Indicates, that interpreting of program code was ended. The program was
	 * ended.
	 */
	public void intepretingEnded(Process code);

	/**
	 * Add application into queue for executing. Return application
	 * identification.
	 * @return application identification
	 */
	public Guid runApplication(CodePkgIdent appIdent, String[] attr,
			PriorityClass priority, Guid parent) throws ExecuteException;
    public Guid runApplication(CodePkgIdent appIdent,
                               String[] attr,
                               PriorityClass priority,
                               Guid parent,
                               CodePackage code) throws ExecuteException;

	/**
	 * Application can wait for end of collection of application.
	 * @param app waiting application id
	 * @param applId collection of application
	 */
	public void waitForApplication(Guid app, Collection<Guid> applId)
			throws InterruptedException;

	/**
	 * Return process with the guid.
	 */
	public Process getProcess(Guid ident);
    public Collection<Process> getProcess(CodePkgIdent processCode);
	
	/**
	 * slouzi k informovani Scheduleru, ze zrovna prisel kod na kterej ceka 
	 */
	public void codeExist(CodePkgIdent codeIdent);



    public Guid runSpecialProcess(ProcessApplication processApplication, CodePackage codePackage);
    public IInfoAPI getInfo();
}
