/*************************************************************************
 *                           Smart Active Node                           *
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.util.Collection;

import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.routing.IRoutingTableControl;
import san.routing.RoutingTable;
import san.routing.Router;

import san.networkInterfaces.NetInterfaces;

/**
 * <b>Title:</b> Smart Active Node<br> <b>Description:</b> This interface server to control the server.<br>
 * @author   Michal Rejda
 * @since   28.9.2007
 * @version   1.0
 * @uml.dependency   supplier="san.routing.IRoutingTableControl"
 */
public interface IServerControl {

	/**
	 * Store new application on to the server.
	 * @return return hash of this new application
	 */
	public String storeApplication(String localPath) throws AplStoreException;

	/**
	 * Run application basely on name. The name can by package name of package
	 * identification in hexadecimal format. Return application identification.
	 * @param name name or identification in hexadecimal format of the
	 *            application
	 * @param attr attributes with the application is started
	 * @return return application identification
	 */
	public Guid runApplication(String name, String[] attr)
			throws ExecuteException;

	/**
	 * Start the application. Return it's identification.
	 * @param appIdent Identification of the starting application.
	 * @param attr attributes with the application is started
	 * @return return application identification
	 */
	public Guid runApplication(CodePkgIdent appIdent, String[] attr)
			throws ExecuteException;

	/**
	 * Run application basely on name. The name can by package name of package
	 * identification in hexadecimal format. Return application identification.
	 * @param name name or identification in hexadecimal format of the
	 *            application
	 * @param attr attributes with the application is started
	 * @param parentAppl parent application id
	 * @return return application identification
	 */
	public Guid runApplication(String name, String[] attr, Guid parentAppl)
			throws ExecuteException;

	/**
	 * Start the application. Return it's identification.
	 * @param appIdent Identification of the starting application.
	 * @param attr attributes with the application is started
	 * @param parentAppl parent application id
	 * @return return application identification
	 */
	public Guid runApplication(CodePkgIdent appIdent, String[] attr,
			Guid parentAppl) throws ExecuteException;

	/**
	 * Application can wait for end of collection of application.
	 * @param app waiting application id
	 * @param applId collection of application
	 */
	public void waitForApplication(Guid app, Collection<Guid> applId)
			throws InterruptedException;

	/**
	 * For process create console.
	 * @param id process id
	 */
	public void createConsole(Guid id);

	/**
	 * Return object that provides to make changes in routing table.
	 */
	public IRoutingTableControl getRoutingModification();

	/**
	 * Stop running the server.
	 */
	public void shutDownServer();

  /**
   * Allow others to see the Guid of Init application
   * @return guid of init - the root application
   */
  public Guid getInitGuid();

  /**
   * @return RoutingTable - the routing table
   */
  public RoutingTable getRoutingTable();


  public Router getRouter();

  /**
   * @return NetInterfaces - records with network interfaces
   */
  public NetInterfaces getNetInterfaces();
} 