/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.core;

import java.util.logging.Level;
import java.util.logging.Logger;
import san.data.TransmitCapsule;
import san.data.TransmitData;
import san.ident.CodePkgIdent;
import san.ident.Guid;
import san.ident.NetIdentifier;
import san.ii.InterceptorInjector;
import san.ii.Packet;
import san.routing.IRoutingTableControl;
import san.routing.RoutingTable;
import san.softstate.ISoftState;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class is a capsule process.<br>
 * @author Michal Rejda
 * @since 21.10.2007
 * @version 1.0
 */
public class ProcessCapsule extends Process implements ICapsuleAPI {

	/**
	 * @uml.property name="capsule"
	 * @uml.associationEnd aggregation="shared"
	 *                     inverse="executableCapsule:san.data.TransmitCapsule"
	 */
	private TransmitCapsule capsule = null;
	/**
	 * If is true, the capsule is send when capsule process ended. If is false,
	 * the capsule is destroyed when capsule process ended. There is implicit
	 * true.
	 */
	private boolean sendCapsuleOnExit = true;

	public ProcessCapsule(Kernel kernel) {
		super(kernel, Type.CAPSULE, "main", "(Lsan/core/ICapsuleAPI;)V");
	}

	/**
	 * Getter of the property <tt>capsule</tt>
	 * @return Returns the capsule.
	 * @uml.property name="capsule"
	 */
	public TransmitCapsule getCapsule() {
		return capsule;
	}

	/**
	 * Setter of the property <tt>capsule</tt>
	 * @param capsule The capsule to set.
	 * @uml.property name="capsule"
	 */
	public void setCapsule(TransmitCapsule capsule) {
		this.capsule = capsule;
	}

	public void sendData(Guid toProcess, TransmitData data) {
		getKernel().sendDataToProcess(getId(), toProcess, data);
	}
	
	public void sendData(CodePkgIdent processCode, TransmitData data) {
		getKernel().sendDataToProcess(getId(), processCode, data);
	}

	public Guid getSourceApplication() {
		return capsule.getLocalApl();
	}

	public NetIdentifier getSource() {
		return capsule.getSrc();
	}

	public void setSource(NetIdentifier i) {
		capsule.setSrc(i);
	}
	
	public boolean isThisServer(NetIdentifier ident) {
		return getKernel().getRouter().isThisNode(ident);
	}

	public NetIdentifier getDestination() {
		return capsule.getDest();
	}

	public void setDestination(NetIdentifier dest) {
		capsule.setDest(dest);
	}

	public TransmitData getDataLoad() {
		return capsule.getData();
	}

	public void setDataLoad(TransmitData data) {
		capsule.setData(data);
	}

	public void finished() {
		sendCapsuleOnExit = false;
	}

	public void getServer() {
		throw new RuntimeException(
				"Not implemented: ProcessCapsule.getServer()");
	}

	public ISoftState getSoftState() {
		return kernel.getSoftState();
	}
	
	public IRoutingTableControl getRoutingModification() {
		return kernel.getRoutingModification();
	}
	
	/**
	 * @return the sendCapsuleOnExit
	 */
	public boolean isSendCapsuleOnExit() {
		return sendCapsuleOnExit;
	}
	
	public void injectCapsule(CodePkgIdent executedCode,
			NetIdentifier destination, TransmitData payload)
			throws DestinationNotAvailableException {
		getKernel().injectCapsule(executedCode, getId(), destination, payload);
	}
	
	public CodePkgIdent getExecutedCodeIdent() {
		return codePackage.getPkgIdentification();
	}

	@Override
	public void setExecuteContinuously(boolean b) {
		capsule.setDestinationExecute(b);
	}

	@Override
	public NetIdentifier getOver() {
		return capsule.getOver();
	}
	
	@Override
	public NetIdentifier getPrev() {
		return capsule.getPrev();
	}

  public void sendPayload(Packet p) {
    try {
      InterceptorInjector.getInterceptorInjector().sendPayload(p);
    } catch (Exception ex) {
      // Just do nothing - no data will be delivered
    }
  }

    public String[] getEntryCallsNamesByGuid(Guid server) {
        return kernel.getEntryCallsNamesByGuid(server);
    }

    public Object callEntryCallByName(Guid server, String entry, Object param) throws InterruptedException {
        Object obj = kernel.callEntryCallByName(server, entry, param);
        return obj;
    }

    public Object callEntryCallByID(Integer entryID, Object param) throws InterruptedException {
        return kernel.callEntryCallByID(entryID, param);
    }

    public Guid[] getServerGuidsByName(String entryName) {
        return kernel.getServerGuidsByName(entryName);
    }

    public Integer getEntryCallIdByGuidName(Guid server, String entryName) {
        return kernel.getEntryCallIdByGuidName(server, entryName);
    }

    public RoutingTable getRoutingTable(){
        return kernel.getRoutingTable();
    }

    public IServerControl getServerControl() {
        return getKernel();
    }
}
