package san.core.fiber;



import java.util.Collection;
import java.util.Iterator;



/**
 *
 * @param <E>
 * @author Jiri Nohavec
 */
public class MultiIterator<E> implements Iterable<E> {



    private static final int MAX_SIZE = 10;

    private final Iterator<E> iterator;
    private final Collection<E>[] collectionArray;
    private final Iterator<E>[] iteratorArray;

    private int size;
    private int currentIndex;



    @SuppressWarnings("unchecked")
    public MultiIterator() {
        collectionArray = (Collection<E>[]) new Collection[MAX_SIZE];
        iteratorArray = (Iterator<E>[]) new Iterator[MAX_SIZE];
        size = 0;
        currentIndex = 0;
        iterator = new Iterator<E>() {
            @Override
            public boolean hasNext() {
                while (currentIndex < size && !iteratorArray[currentIndex].hasNext()) {
                    currentIndex++;
                }
                return (currentIndex < size);
            }
            @Override
            public E next() {
                return iteratorArray[currentIndex].next();
            }
            @Override
            public void remove() {
                iteratorArray[currentIndex].remove();
            }
        };
    }



    public MultiIterator<E> add(Collection<E> collection) {
        if (size == MAX_SIZE) {
            throw new RuntimeException("Multi iterator max size reached");
        }
        collectionArray[size] = collection;
        size++;
        return this;
    }



    @Override
    public Iterator<E> iterator() {
        for (int i = 0; i < size; i++) {
            iteratorArray[i] = collectionArray[i].iterator();
        }
        currentIndex = 0;
        return iterator;
    }
}
