package san.core.fiber.monitor.deadlock;



import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Queue;



/**
 *
 * @author Jiri Nohavec
 */
public class RagGraph {



    private final List<RagNode> ragNodeList;
    private final List<IRagNodeCycle> ragNodeCycleList;
    private final Map<Object, RagNode> ragNodeMap;
    private final Queue<RagNode> ragNodeQueue;

    private int ragNodeListIndex;



    public RagGraph() {
        ragNodeList = new ArrayList<RagNode>();
        ragNodeCycleList = new ArrayList<IRagNodeCycle>();
        ragNodeMap = new HashMap<Object, RagNode>();
        ragNodeQueue = new LinkedList<RagNode>();
        ragNodeListIndex = 0;
    }



    private RagNode getNewRagNode() {
        RagNode ragNode;
        if (ragNodeListIndex < ragNodeList.size()) {
            ragNode = ragNodeList.get(ragNodeListIndex);
        } else {
            ragNode = new RagNode();
            ragNodeList.add(ragNode);
        }
        ragNodeListIndex++;
        return ragNode;
    }



    private void fillObjectCycleList(RagNode firstRagNode) {
        RagNode ragNode = firstRagNode;
        ragNodeCycleList.clear();
        do {
            ragNodeCycleList.add(ragNode);
            ragNode = ragNode.getPreviousRagNode();
        } while (ragNode != firstRagNode);
    }



    public void addEdge(Object object1, Object object2) {
        RagNode ragNode1 = ragNodeMap.get(object1);
        RagNode ragNode2 = ragNodeMap.get(object2);
        if (ragNode1 == null) {
            ragNode1 = getNewRagNode();
            ragNode1.setObject(object1);
            ragNodeMap.put(object1, ragNode1);
        }
        if (ragNode2 == null) {
            ragNode2 = getNewRagNode();
            ragNode2.setObject(object2);
            ragNodeMap.put(object2, ragNode2);
        }
        ragNode2.addToWaitingForThisList(ragNode1);
    }



    public boolean hasCycle() {
        int bfsFlag = RagNode.BFS_FLAG_FIRST;
        for (int i = 0; i < ragNodeListIndex; i++) {
            RagNode ragNode = ragNodeList.get(i);
            if (ragNode.getBfsFlag() == RagNode.BFS_FLAG_EMPTY) {
                ragNodeQueue.clear();
                ragNodeQueue.add(ragNode);
                while (!ragNodeQueue.isEmpty()) {
                    ragNode = ragNodeQueue.remove();
                    ragNode.setBfsFlag(bfsFlag);
                    if (!ragNode.isWaitingForThisListEmpty()) {
                        for (RagNode nextRagNode: ragNode.getWaitingForThisListIterable()) {
                            nextRagNode.setPreviousRagNode(ragNode);
                            if (nextRagNode.getBfsFlag() == RagNode.BFS_FLAG_EMPTY) {
                                ragNodeQueue.add(nextRagNode);
                            } else if (nextRagNode.getBfsFlag() == bfsFlag) {
                                fillObjectCycleList(nextRagNode);
                                return true;
                            }
                        }
                    }
                }
                bfsFlag++;
            }
        }
        return false;
    }



    public Iterable<IRagNodeCycle> getRagNodeCycleListIterable() {
        return ragNodeCycleList;
    }



    public void clear() {
        for (int i = 0; i < ragNodeListIndex; i++) {
            ragNodeList.get(i).clear();
        }
        ragNodeCycleList.clear();
        ragNodeMap.clear();
        ragNodeQueue.clear();
        ragNodeListIndex = 0;
    }
}
