package san.core.fiber.strategy;



import java.util.Queue;
import san.core.Process;



/**
 *
 * @author Jiri Nohavec
 */
public class MultiLevelQueueStrategy extends ISchedulerStrategy {



    private static final Process.PriorityClass[] PRIORITY_CLASS_ARRAY = {
        Process.PriorityClass.REAL_TIME,
        Process.PriorityClass.HIGH,
        Process.PriorityClass.NORMAL,
        Process.PriorityClass.LOW
    };



    public MultiLevelQueueStrategy(int cpuCount, Queue<Process> allProcessQueue) {
        super(cpuCount, allProcessQueue);
    }



    @Override
    protected void reschedule() {
        Process firstProcess;
        Process process = null;
        boolean firstIteration = true;
        int priorityClassIndex = 0;
        if (allProcessQueue.size() == 0) {
            return;
        }
        firstProcess = allProcessQueue.remove();
        while (selectedProcessQueue.size() < cpuCount) {
            if (!firstIteration) {
                process = allProcessQueue.remove();
                allProcessQueue.add(process);
                if (firstProcess == process) {
                    if (priorityClassIndex < PRIORITY_CLASS_ARRAY.length - 1) {
                        priorityClassIndex++;
                    } else {
                        break;
                    }
                }
            } else {
                process = firstProcess;
                allProcessQueue.add(process);
                firstIteration = false;
            }
            switch (process.getState()) {
                case RUNNING:
                case READY_TO_RUN:
                    if (process.getPriorityClass() == PRIORITY_CLASS_ARRAY[priorityClassIndex]) {
                        selectedProcessQueue.add(process);
                    }
                    break;
            }
        }
    }
}
