/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/

package san.ident;

import java.io.Serializable;

import san.Utility;
import san.applicationRepository.HashCalculation;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class identifies applications package.<br>
 * @author Michal Rejda
 * @since 21.2.2007
 * @version 1.0
 */
public class CodePkgIdent implements Comparable<CodePkgIdent>, Serializable {
	
	/**
	 * Count of Applications identifier bytes. It's set to 16 bytes = 128 bites.
	 */
	public static final int LENGHT = 16;
	
	private byte[] hash;
	
	public CodePkgIdent() {
		this.hash = null;
	}
	
	public CodePkgIdent(byte[] hash) {
		this.hash = hash;
	}
	
	/**
	 * Parse code package identifier from string to object.
	 */
	public static CodePkgIdent parse(String cpis){
		return new CodePkgIdent(Utility.fromHexString(cpis));
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof CodePkgIdent) {
			CodePkgIdent ident = (CodePkgIdent) obj;
			if(this.hash.length != ident.hash.length) {
				return false;
			}
			else {
				for (int i = 0; i < this.hash.length; i++) {
					if(this.hash[i] != ident.hash[i]) {
						return false;
					}
				}
				return true;
			}
		} else {
			return false;
		}
	}
	
	public int compareTo(CodePkgIdent o) {
		if(this.hash.length != o.hash.length) {
			return this.hash.length - o.hash.length;
		}
		else
			for (int i = 0; i < this.hash.length; i++) {
				if(this.hash[i] != o.hash[i]) {
					return (this.hash[i] - o.hash[i])*i;
				}
			}
		return 0;
	}
	
	public int hashCode() {
		int code = 0;
		for (int i = 0; i < hash.length; i++) {
			code = code  +  hash[i];
		}
		return code;
	}
	
	public String toString () {
		return HashCalculation.hashToString(hash);
	}
}
