package san.interpreter.sanint.classLoading;

import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.References;

public interface Class {
	/**Runs method of index
	 * @param index index of method in constant pool
	 * @param interpreter reference to interpreter
	 * @param interpreterReferences list of references
	 * @param stack current stack
	 * @param localVars current local variables
	 * @param staticMethod it is static method?
	 * @throws Exception
	 */
	void runMethod(
			int index, 
			References interpreterReferences, 
			InterpreterStack stack, 
			LocalVars localVars,
			boolean staticMethod,
			boolean virtualMethod, san.core.Process process)
	throws Exception;
	

	/**Gets value of static variable
	 * @param name name of variable
	 * @param stack current stack
	 * @param references list of references
	 * @throws Exception
	 */
	void getStaticVariable(String name, InterpreterStack stack, References references)
		throws Exception;
	
	/**Sets value of static variable
	 * @param name name of variable
	 * @param stack current stack
	 * @param references list of references
	 * @throws Exception
	 */
	void setStaticVariable(String name, InterpreterStack stack, References references)
		throws Exception;
	
	
	/**Finds method and returns its info 
	 * @param name name of method
	 * @param descriptor return and argument types
	 * @return MethodInfo of method
	 */
	MethodInfo getMethod(String name, String descriptor);	
	/**
	 * @return Type of class
	 */
	String getName();
	/**Checks if class can be casted to enother type 
	 * @param type type to be chcked
	 * @return
	 */
	boolean canBeCasted(String type);
}
