package san.interpreter.sanint.classLoading;

import java.net.URL;
//import java.util.ArrayList;

import san.applicationRepository.SanCodePkg;
import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.SanInterpreter;
import san.interpreter.sanint.interpreter.References;
import san.interpreter.sanint.fiber.stack.CallStackLevel;
import san.interpreter.sanint.fiber.stack.StaticVariableCallStackLevel;
import san.interpreter.sanint.fiber.stack.SystemInitCallStackLevel;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import com.sun.org.apache.bcel.internal.classfile.ConstantClass;
import com.sun.org.apache.bcel.internal.classfile.ConstantFieldref;
import com.sun.org.apache.bcel.internal.classfile.ConstantNameAndType;
import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import com.sun.org.apache.bcel.internal.classfile.ConstantUtf8;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.HashMap;

import san.security.SecurityMonitor;

public class ClassManager {
	public static String OBJECT_IMPLEMENTATION = "Object";
//	ArrayList<Class> loadedClasses = new ArrayList<Class>();
	HashMap<String, Class> loadedClasses = new HashMap<String, Class>();
	long lastInstanceId = 0;
        ClassLoader classLoader;
        san.core.Process executedProcess = null;
		
	SanCodePkg sanPackage = null;

    private CallStackLevel mainCallStackLevel;

	public ClassManager(SanCodePkg pkg, san.core.Process process)
	{
            this.executedProcess = process;
            sanPackage = pkg;
            classLoader = new ClassManagerClassLoader(sanPackage);
//            Init();
	}
	public Reference createReference(String type)
	throws Exception
	{
		Class cl = getClass(type);
		return createReference(cl);
	}
	public Reference createReference(Class type)
	{
		Reference ret = null;
		if (type instanceof HardClass)
			ret = new HardReference(this, (HardClass)type);
		else ret = new SoftReference((SoftClass)type);
		ret.setInstanceId(lastInstanceId++);
		return ret;
	}
	public Reference createArray(HardClass cl, InterpreterStack stack, int acode, int dimCount)
		throws Exception
	{
		int length = stack.readInt();
		return new SimpleArray(length, acode);
	}
	public Reference createReferenceArray(HardClass cl, InterpreterStack stack, int index, int dimCount)
		throws Exception
	{
		ConstantClass classConst = (ConstantClass)cl.cl.getConstantPool().getConstant(index);
		String typeName = classConst.getBytes(cl.cl.getConstantPool());
		
		int[] dims = new int[dimCount];
		for(int i = 0; i < dims.length; i++)
			dims[i] = stack.readInt();
//		Reference ret = createArray(dims, 0, java.lang.Class.forName(typeName.replace('/', '.')));
        Reference ret = createArray(dims, 0, java.lang.Class.forName(typeName.replace('/', '.'),
                                                                     true,
                                                                     classLoader));
		ret.setInstanceId(lastInstanceId++);
		if (ret.getInstanceId() == 1641)
			System.out.println();
		return ret;
	}
	private Reference createArray(int[] dimensions, int index, java.lang.Class type)
		throws Exception
	{
		if (index >= dimensions.length)
			return null;
		ReferenceArray ret = new ReferenceArray(dimensions[index], type, this);
		for(int i = 0; i < ret.getLength(); i++) 
			ret.setVariable(i, createArray(dimensions, index + 1, type));
		return ret;
	}
	
	
	public Class getClass(String type)
	throws Exception
	{
        type = type.replace('.', '/');
		/*for(int i = 0; i < loadedClasses.size(); i++)
		{
			if (loadedClasses.get(i).getName().equals(type))
				return loadedClasses.get(i);
		}*/

        Class loadclass = loadedClasses.get(type);
        if (loadclass != null)
            return loadclass;
        loadclass = loadClass(type);

        // TODO tohle z nejakeho duvodu nefunguje, chtelo by to prozkoumat...
        // Pada to treba u aplikace ping, zacykli se to
        //loadedClasses.put(loadclass.getName(), loadclass);

        return loadclass;//loadClass(type);
    }

	private Class loadClass(String type)
	throws Exception
	{
        CallStackLevel constructorCallStackLevel = executedProcess.getInterpretingContext().getNewCall();
        HardClass cl = null;
		try
		{
			cl = new HardClass(type, sanPackage, this);
		}
		catch (Exception e)
		{
            type = type.replace('.', '/');
            /* we cannot load the class directly, so try to load it through system class loader in classpath */

			URL url = ClassLoader.getSystemClassLoader().getResource(type + "." + SanCodePkg.BIN_EXTENSION);

            /* system class loader is unable to find this class. This could be ours active application which is not in classpath, but we know exactly, where it is... */
            if (url == null && sanPackage.getPkgName().equals(type.substring(0,type.indexOf('/')))) {        
                String path = sanPackage.getAplPkgZipFilename() + "!/" + SanCodePkg.BINARY_ENTRY + type.substring(type.indexOf('/')) + "." + SanCodePkg.BIN_EXTENSION;
             
                cl = new HardClass(path, this);
            } else { /* system class loader located the file and provides us a URL */
                //System.out.println(type + ".class");
                //System.out.println("  url:" + url);
                String str = new File(new URI(url.getPath())).getAbsolutePath(); /* This is the only way I know to convert URL to path in platform independent manner */
                cl = new HardClass(str, this);
            }
		}
		loadedClasses.put(cl.getName(), cl);
        //loadedClasses.add(cl);
		//spusteni <clinit>
		MethodInfo mi = cl.getMethodOnlyInThisClass(MethodInfo.nameClinit, "()V");

		if (mi != null)
		{
			References references = new References(SanInterpreter.REFERENCES_CAPACITY);
			InterpreterStack stack = new InterpreterStack(SanInterpreter.STACK_CAPACITY);
			LocalVars localVars = new LocalVars(SanInterpreter.LOCAL_VAR_CAPACITY);
//			InterpreterBytecode b = new InterpreterBytecode(this, SanInterpreter.MAX_REPEAT_COUNT);
//			b.Run(mi.getByteCode(), (HardClass)cl, references, stack, localVars, this.executedProcess);
            constructorCallStackLevel.set(mi.getByteCode(),
                                          cl,
                                          references,
                                          stack,
                                          localVars,
                                          0);
		}
		return cl;
	}


	public Object runMethod(String className, String name, String descriptor, Object[] args, Reference thisRef, boolean isStatic)
		throws Exception
	{
                Class cl = getClass(className);
		MethodInfo method = cl.getMethod(name, descriptor);
		if (method == null)
			throw new Exception("Method not found");
		if (!method.isStatic() && isStatic)
			throw new Exception("Not a static method");

		References references = new References(SanInterpreter.REFERENCES_CAPACITY);
		InterpreterStack stack = new InterpreterStack(SanInterpreter.STACK_CAPACITY);
		LocalVars localVars = new LocalVars(method.getMaxLocals() * 4);
//		InterpreterBytecode b = new InterpreterBytecode(this, SanInterpreter.MAX_REPEAT_COUNT);

		int localIndex = 0;
		
		if (!method.isStatic() && !isStatic)
		{
			if (thisRef == null)
				thisRef = createReference(className);
			localVars.AddLocalReference(localIndex++, references.putReference(thisRef));
		}
		if (args != null)
		{
			for(Object arg : args)
			{
				if (arg == null)
				{
					localVars.AddLocalReference(localIndex++, 
							references.putReference(new SoftReference(null)));
					continue;
				}
				java.lang.Class clazz = arg.getClass();
				if (clazz.isPrimitive())
				{
					throw new NotImplementedException();
					/*if (clazz.equals(int.class))
						stack.write(((Integer)arg).intValue());
					else if (clazz.equals(float.class))
						stack.write(((Float)arg).floatValue());
					else if (clazz.equals(double.class))
						stack.write(((Double)arg).doubleValue());
					else if (clazz.equals(long.class))
						stack.write(((Long)arg).longValue());
					else if (clazz.equals(boolean.class))
						if (((Boolean)arg).booleanValue())
							stack.write((byte) 1);
						else stack.write((byte) 0);
					else if (clazz.equals(byte.class))
						stack.write(((Byte)arg).byteValue());
					else if (clazz.equals(char.class))
						stack.write(((Character)arg).charValue());
					else if (clazz.equals(short.class))
						stack.write(((Short)arg).shortValue());*/
				}
				else if (clazz.isArray())
				{
					if (clazz.getComponentType().isPrimitive())
					{
						Array a = new SimpleArray(arg);
						localVars.AddLocalReference(localIndex++, references.putReference(a));
						//stack.writeRef(references.putReference(a));
					}
					else
					{
						Array a = new ReferenceArray(arg, this);
						localVars.AddLocalReference(localIndex++, references.putReference(a));
						//stack.writeRef(references.putReference(a));
					}
				}
				else
				{
					localVars.AddLocalReference(localIndex++, 
							references.putReference(new SoftReference(arg, this)));
				}
			}
		}

//		b.Run(method.getByteCode(), (HardClass)cl, references, stack, localVars, this.executedProcess);
//
//		String returnValueStr = descriptor.substring(descriptor.lastIndexOf(')') + 1);
//		switch (returnValueStr.charAt(0)) {
//            case 'I':
//                return stack.readInt();
//            case 'F':
//                return stack.readFloat();
//            case 'D':
//                return stack.readDouble();
//            case 'J':
//                return stack.readLong();
//            case 'Z':
//                return new Boolean(stack.readInt()!=0);
//            case 'B':
//                stack.readByte();
//            case 'C':
//                return (char)stack.readShort();
//            case 'S':
//                return stack.readShort();
//            case 'L':
//            case '[': // tohle mozna fungovat nemusi, ale jakekoliv pole by imho mela byt reference a mel by to resit ReferenceConverter
//                ReferenceConverter conv = new ReferenceConverter();
//                Object obj = conv.toJavaObject(references.getReference(stack.readRef()), null);
//                return obj;
//        }
//        return null;
        mainCallStackLevel.set(method.getByteCode(),
                               (HardClass) cl,
                               references,
                               stack,
                               localVars,
                               0);
        return null;
	}
	
	public void Init()
	{
        mainCallStackLevel = executedProcess.getInterpretingContext().getNewCall();
        SystemInitCallStackLevel systemInitCallStackLevel = executedProcess.getInterpretingContext().getNewSystemInitCall();
		//nahrada za 
		//cman.runStaticMethod("java.lang.System", "initializeSystemClass", "()V", null);		
		try
		{
			//runStaticMethod("java.lang.System", "initializeSystemClass", "()V", null);
			Class systemClass = getClass("java.lang.System");
			InterpreterStack stack = new InterpreterStack(64);
			References refs = new References(64);
//			stack.write(refs.putReference(new SoftReference(System.out, this)));
//			systemClass.setStaticVariable("out", stack, refs);
            systemInitCallStackLevel.set(refs,
                                         stack,
                                         systemClass,
                                         new SoftReference(System.out, this),
                                         new SoftReference(System.err, this),
                                         "out",
                                         "err");
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void getStaticVariable(HardClass invokingClass, int index, InterpreterStack stack,
								  References refs)
		throws Exception
	{
        StaticVariableCallStackLevel staticVariableCallStackLevel = executedProcess.getInterpretingContext().getNewStaticVariableCall();
		ConstantPool cp = invokingClass.getConstants();
		
		ConstantFieldref fieldInfo = (ConstantFieldref)cp.getConstant(index);
		ConstantNameAndType nameInfo = (ConstantNameAndType)cp.getConstant(fieldInfo.getNameAndTypeIndex());
		String name = ((ConstantUtf8)cp.getConstant(nameInfo.getNameIndex())).getBytes();
		ConstantClass classInfo = (ConstantClass)cp.getConstant(fieldInfo.getClassIndex());
		String className = ((ConstantUtf8)cp.getConstant(classInfo.getNameIndex())).getBytes();
		Class cl = getClass(className);
//		cl.getStaticVariable(name, stack, refs);
        staticVariableCallStackLevel.set(refs,
                                         stack,
                                         cl,
                                         name,
                                         true);
	}
	public void setStaticVariable(HardClass invokingClass, int index, InterpreterStack stack,
								  References refs)
		throws Exception
	{
        StaticVariableCallStackLevel staticVariableCallStackLevel = executedProcess.getInterpretingContext().getNewStaticVariableCall();
		ConstantPool cp = invokingClass.getConstants();
		
		ConstantFieldref fieldInfo = (ConstantFieldref)cp.getConstant(index);
		ConstantNameAndType nameInfo = (ConstantNameAndType)cp.getConstant(fieldInfo.getNameAndTypeIndex());
		String name = ((ConstantUtf8)cp.getConstant(nameInfo.getNameIndex())).getBytes();
		ConstantClass classInfo = (ConstantClass)cp.getConstant(fieldInfo.getClassIndex());
		String className = ((ConstantUtf8)cp.getConstant(classInfo.getNameIndex())).getBytes();
		Class cl = getClass(className);
//		cl.setStaticVariable(name, stack, refs);
        staticVariableCallStackLevel.set(refs,
                                         stack,
                                         cl,
                                         name,
                                         false);
	}

    public SanCodePkg getSanPackage() {
        return sanPackage;
    }

    public ClassLoader getClassLoader() {
        return classLoader;
    }
}

/**
 * Vlastni class loader se stara o nacteni trid z balicku aplikace,
 * ta totiz neni ve standardnim classpath
 * @author Miroslav Hendrych
 */
class ClassManagerClassLoader extends ClassLoader {
	SanCodePkg sanPackage = null;
    HashMap<String, java.lang.Class> loadedClass;

    public ClassManagerClassLoader(SanCodePkg sanPackage) {
        super(ClassManagerClassLoader.class.getClassLoader());
        loadedClass = new HashMap<String, java.lang.Class>();
        this.sanPackage = sanPackage;
    }

    @Override
    public java.lang.Class<?> loadClass(String name) throws ClassNotFoundException {
        return findClass(name);
    }

    @Override
    public java.lang.Class findClass(String className) throws ClassNotFoundException {
        java.lang.Class result = loadedClass.get(className);
        if (result != null)
            return result;

        try {
            return findSystemClass(className);
        } catch (Exception ex) {}

        String localClassName = className.replace(sanPackage.getPkgName()+".", "");
        InputStream is;
        try {
            is = sanPackage.getClass(localClassName);
        } catch (Exception ex) {
            throw new ClassNotFoundException(className);
        }
        try {
            byte[] data = new byte[is.available()];
            is.read(data, 0, data.length);
            result = defineClass(className, data, 0, data.length, null);
            loadedClass.put(className, result);
            return result;
        } catch (IOException ex) {
            
        }
        return null;
    }
}