package san.interpreter.sanint.classLoading;

import java.lang.reflect.Method;
import java.util.ArrayList;

import java.util.HashMap;
import org.apache.log4j.Logger;

import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.LocalVars;
import san.interpreter.sanint.interpreter.References;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

public class NativeMethodSupport {
	
	private static final transient Logger log = Logger.getLogger(NativeMethodSupport.class);
	public static final int MAX_LOCALS = 512;
    HashMap<Long, Object> convertedObject = new HashMap<Long, Object>();
	
	public void runNativeMethod(MethodInfo method, InterpreterStack stack, 
								References references, LocalVars localVars, int returnStackPointer,
								ClassManager manager)
		throws Exception
	{
		if (log.isTraceEnabled()) {
			/*System.out.println("Pokus o spusteni nativni funkce " + 
					method.getClassInfo().getName() + 
					"::" + method.getName() + 
					" - " + method.getDescriptor()); */
			log.trace("Pokus o spusteni nativni funkce " + 
					method.getClassInfo().getName() + 
					"::" + method.getName() + 
					" - " + method.getDescriptor());

		}
		
		Object[] args = convertParameters(localVars, method.getDescriptor(), references, method.isStatic());
		java.lang.Class[] types = getParameterTypes(method.getDescriptor(), args.length);
		if (types.length == 0) 
		{
			args = null;
			types = null;
		}
		java.lang.Class cl = java.lang.Class.forName(method.getClassInfo().getName().replace('/', '.'));
		Method meth = null;
		meth = cl.getDeclaredMethod(method.getName(), types);
		meth.setAccessible(true);
		
		Object thisObject = null;
		if (!method.isStatic())
		{
			ReferenceConverter conv = new ReferenceConverter();
			thisObject = conv.toJavaObject(references.getReference(localVars.GetLocalReference(0)), convertedObject);
		}
		Object ret = meth.invoke(thisObject, args);
		
		if (args != null)
			updateParameters(args, types, localVars, references, method.isStatic(), convertedObject);
		
		stack.setStackPointer(returnStackPointer);
		String descriptor = method.getDescriptor();
		String retType = descriptor.substring(descriptor.lastIndexOf(')'));
		switch(retType.charAt(1))
		{
		case 'I': //integer
			stack.write(((Integer)ret).intValue());
			break;
		case 'F': //float
			stack.write(((Float)ret).floatValue());
			break;
		case 'C': //char
			stack.write(((Character)ret).charValue());
			break;
		case 'S': //short
			stack.write(((Short)ret).shortValue());
			break;
		case 'Z': //boolean			
			if (((Boolean)ret).booleanValue())
				stack.write((int)1);
			else stack.write((int)0);
			break;
		case 'B': //byte
			stack.write(((Byte)ret).byteValue());
			break;
		case 'D': //double
			stack.write(((Double)ret).doubleValue());
			break;
		case 'J': //long
			stack.write(((Long)ret).longValue());
			break;
		case '[': //array
			throw new NotImplementedException();
		case 'L': //reference
			ReferenceConverter conv = new ReferenceConverter();
			Reference ref = conv.toHardReference(ret, manager);
			int newIndex = references.putReference(ref);
			stack.write(newIndex);
			break;
		}
	}
	
	public Object[] convertParameters(LocalVars localVars, String descriptor, References refs, boolean staticMethod)
		throws Exception
	{
		//TODO: konverze this...
		int index = 1;
		int localIndex = 1;
		if (staticMethod)
			localIndex = 0;
		ArrayList<Object> ret = new ArrayList<Object>();
		while(descriptor.charAt(index) != ')')
		{
			switch(descriptor.charAt(index))
			{
			case 'I': //integer
				ret.add(new Integer(localVars.GetLocalInt(localIndex)));
				break;
			case 'F': //float
				ret.add(new Float(localVars.GetLocalFloat(localIndex)));
				break;
			case 'C': //char
				ret.add(new Character((char)localVars.GetLocalInt(localIndex)));
				break;
			case 'S': //short
				ret.add(new Short((short)localVars.GetLocalInt(localIndex)));
				break;
			case 'Z': //boolean			
				int i = localVars.GetLocalInt(localIndex);
				if (i == 0)
					ret.add(Boolean.FALSE);
				else ret.add(Boolean.TRUE);
				break;
			case 'B': //byte
				ret.add(new Byte((byte)localVars.GetLocalInt(localIndex)));
				break;
			case 'D': //double
				ret.add( new Double(localVars.GetLocalDouble(localIndex)));
				localIndex++;
				break;
			case 'J': //long
				ret.add(new Long(localVars.GetLocalLong(localIndex)));
				localIndex++;
				break;
			case '[': //array
				throw new NotImplementedException();
			case 'L': //reference
				ReferenceConverter conv = new ReferenceConverter();
				Object obj = 
    				conv.toJavaObject(refs.getReference(localVars.GetLocalReference(localIndex)), convertedObject);
				ret.add(obj);
				for(; descriptor.charAt(index) != ';'; index++);
				break;
			}
			index++;
			localIndex++;
		}
		return ret.toArray();
	}
	public java.lang.Class[] getParameterTypes(String descriptor, int length)
		throws Exception
	{
		java.lang.Class[] ret = new java.lang.Class[length];
		int retIndex = 0;
		int index = 1;
		while(descriptor.charAt(index) != ')')
		{
			switch(descriptor.charAt(index))
			{
			case 'I': //integer
				ret[retIndex++] = int.class;
				break;
			case 'F': //float
				ret[retIndex++] = float.class;
				break;
			case 'C': //char
				ret[retIndex++] = char.class;
				break;
			case 'S': //short
				ret[retIndex++] = short.class;
				break;
			case 'Z': //boolean			
				ret[retIndex++] = boolean.class;
				break;
			case 'B': //byte
				ret[retIndex++] = byte.class;
				break;
			case 'D': //double
				ret[retIndex++] = double.class;
				break;
			case 'J': //long
				ret[retIndex++] = long.class;
				break;
			case '[': //array
				throw new NotImplementedException();
			case 'L': //reference
				int classNameEnd = index;
				for(; descriptor.charAt(classNameEnd) != ';'; classNameEnd++);
				String className = descriptor.substring(index + 1, classNameEnd);
				ret[retIndex++] = java.lang.Class.forName(className.replace('/', '.'));
				index = classNameEnd;
				
				break;
			}
			index++;
		}
		return ret;
	}
	
	private void updateParameters(Object[] args, java.lang.Class[] types, LocalVars localVars, References refs, boolean isStatic, HashMap<Long, Object> convertedObject)
		throws Exception
	{
		int offset = isStatic ? 0 : 1;
		for(int i = 0; i < args.length; i++)
		{
			if (!types[i].isPrimitive())
			{
				Reference ref = refs.getReference(localVars.GetLocalReference(i + offset));
				Object obj = args[i];
				if (ref != null && !ref.isNull())
					ref.fromObject(convertedObject, obj);
			}
		}
	}
	

}
