package san.interpreter.sanint.fiber.override;



import java.util.ArrayList;
import java.util.List;
import san.core.Process;
import san.interpreter.sanint.fiber.InterpretingException;
import san.interpreter.sanint.interpreter.InterpreterStack;
import san.interpreter.sanint.interpreter.References;



/**
 *
 * @author Jiri Nohavec
 */
public abstract class IMethodCall {



    private final boolean returnAfterExecution;
    private final List<String> classNameList;
    private final List<String> methodNameList;
    private final List<String> methodDescriptorList;



    public IMethodCall(boolean returnAfterExecution) {
        this.returnAfterExecution = returnAfterExecution;
        classNameList = new ArrayList<String>();
        methodNameList = new ArrayList<String>();
        methodDescriptorList = new ArrayList<String>();
    }



    public IMethodCall(boolean returnAfterExecution,
                       String methodFullName,
                       String methodDescriptor) {
        this(returnAfterExecution);
        addClass(methodFullName.substring(0, methodFullName.lastIndexOf(".")));
        addMethod(methodFullName.substring(methodFullName.lastIndexOf(".") + 1), methodDescriptor);
    }



    protected void addClass(String className) {
        classNameList.add(className.replace(".", "/"));
    }



    protected void addMethod(String methodName, String methodDescriptor) {
        methodNameList.add(methodName);
        methodDescriptorList.add(methodDescriptor);
    }



    public boolean isReturnAfterExecution() {
        return returnAfterExecution;
    }



    public boolean compareTo(String className,
                             String methodName,
                             String methodDescriptor) {
        boolean classMatch = false;
        for (int i = 0; i < classNameList.size(); i++) {
            if (className.equals(classNameList.get(i))) {
                classMatch = true;
                break;
            }
        }
        if (!classMatch) {
            return false;
        }
        for (int i = 0; i < methodNameList.size(); i++) {
            if (methodName.equals(methodNameList.get(i))
                    && methodDescriptor.equals(methodDescriptorList.get(i))) {
                return true;
            }
        }
        return false;
    }



    public abstract void execute(References references,
                                 InterpreterStack stack,
                                 Process process) throws InterpretingException;
}
