package san.interpreter.sanint.fiber.staticm;



import san.interpreter.sanint.classLoading.HardReference;
import san.interpreter.sanint.classLoading.Reference;
import san.interpreter.sanint.classLoading.ReferenceArray;
import san.interpreter.sanint.classLoading.ReferenceConverter;
import san.interpreter.sanint.classLoading.SimpleArray;
import san.interpreter.sanint.classLoading.SoftReference;



/**
 *
 * @author Jiri Nohavec
 */
public class StaticReferenceConverter {



    public static Reference convert(Reference reference,
                                    ReferenceConverter referenceConverter) throws Exception {
        if (reference == null || reference.isNull()) {
            return null;
        } else if (reference instanceof SoftReference) {
            return reference;
        } else if (reference instanceof HardReference) {
            return new SoftReference(referenceConverter.toJavaObject(reference, null),
                                     referenceConverter.getClassManager());
        } else if (reference instanceof SimpleArray) {
            /* always throws UnsuportedException from java.nio */
            return new SoftReference(((SimpleArray) reference).toArray(null),
                                     referenceConverter.getClassManager());
        } else if (reference instanceof ReferenceArray) {
            return new SoftReference(((ReferenceArray) reference).toArray(null),
                                     referenceConverter.getClassManager());
        } else {
            throw new RuntimeException("Unsuported reference type");
        }
    }



    public static boolean isSoftReferenceToObject(Reference reference) {
        return (reference != null
                && !reference.isNull()
                && reference instanceof SoftReference
                && ((SoftReference) reference).getReference().getClass().equals(Object.class));
    }



    public static boolean areBothSoftReferencesToObject(Reference reference1, Reference reference2) {
        return (isSoftReferenceToObject(reference1) && isSoftReferenceToObject(reference2));
    }
}
