package san.interpreter.sanint.interpreter;

import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Queue;

import org.apache.log4j.Logger;

import san.applicationRepository.BshCodePkg;
import san.applicationRepository.SanCodePkg;
import san.core.IApplicationAPI;
import san.core.ICapsuleAPI;
import san.core.Kernel;
import san.core.Process;
import san.core.ProcessApplication;
import san.core.ProcessCapsule;
import san.data.TransmitCapsule;
import san.interpreter.IInterpreter;
import san.interpreter.InterpretManager;
import san.interpreter.sanint.classLoading.ClassManager;
import san.interpreter.sanint.classLoading.HardClass;
import san.interpreter.sanint.classLoading.MethodInfo;
import san.interpreter.sanint.classLoading.Reference;
import san.interpreter.sanint.classLoading.SoftReference;
import san.interpreter.sanint.test.CapsuleImpl;
import san.localization.Localization;
import san.logging.LoggerFactory;
import san.logging.ILogger.LogType;

@Deprecated
public class SanInterpreter extends Thread implements IInterpreter {

  private static final transient Logger log = Logger.getLogger(SanInterpreter.class);
  public static final int STACK_CAPACITY = 1024;
  public static final int LOCAL_VAR_CAPACITY = 64;
  public static final int REFERENCES_CAPACITY = 64;
  public static final int MAX_REPEAT_COUNT = 1000;
  References references = new References(REFERENCES_CAPACITY);
  InterpreterStack stack = new InterpreterStack(STACK_CAPACITY);
  LocalVars localVars = new LocalVars(LOCAL_VAR_CAPACITY);
  private InterpretManager interpretManager;
  private Process executed = null;
  private Queue<TransmitCapsule> data;
  private String[] attributes;
  private boolean running = false;
  private Kernel kernel;

  public SanInterpreter(InterpretManager interpretManager, Kernel kernel) {

    this.interpretManager = interpretManager;
    this.kernel = kernel;
    this.data = new LinkedList<TransmitCapsule>();

  }

  /**
   * Invokes execution on SanInterpret
   * @param codePackage Package with data
   * @param mainClass The Name of the main class from current package
   */
  private void interpretExecution(SanCodePkg codePackage, String mainClass) {
    try {
      ClassManager cman = new ClassManager(codePackage, this.executed);
      Reference api = new SoftReference(executed, cman);
      Reference thisRef = cman.createReference(mainClass);
      if (executed instanceof IApplicationAPI) {
        this.interpretManager.addRunningApplication(executed.getId(), thisRef);
        cman.runMethod(mainClass, "main", "([Ljava/lang/String;Lsan/core/IApplicationAPI;Lsan/ident/Guid;)V",
                new Object[]{
                  attributes,
                  executed,
                  executed.getId(),}, thisRef, false);
      } else if (executed instanceof ICapsuleAPI) {
        cman.runMethod(mainClass, "main", "(Lsan/core/ICapsuleAPI;)V",
                new Object[]{
                  executed
                }, thisRef, false);
      }
      //System.out.println("OK");
      if (log.isDebugEnabled()) {
        log.debug("OK");
      }

    } catch (Exception e) {
      e.printStackTrace();
      log.error(e);
    }
  }

  public void run() {
    this.running = true;
    try {
      SanCodePkg codePackage = (SanCodePkg) executed.getCodePackage();

      if (executed instanceof ProcessApplication) {
        executed.setState(san.core.Process.State.RUNNING);
        ProcessApplication pa = (ProcessApplication) executed;
        String mainClass = codePackage.getManifestPropertyValue(SanCodePkg.APP_MAIN_CLASS_MFP);

        //interpreting program
        interpretExecution(codePackage, mainClass);

        executed.setState(san.core.Process.State.FINISHED);

      } else if (executed instanceof ProcessCapsule) {
        executed.setState(san.core.Process.State.RUNNING);
        ProcessCapsule pc = (ProcessCapsule) executed;
        String mainClass = codePackage.getManifestPropertyValue(SanCodePkg.CAPSULE_MAIN_CLASS_MFP);

        //interpreting program
        interpretExecution(codePackage, mainClass);

        executed.setState(san.core.Process.State.FINISHED);
        if (pc.isSendCapsuleOnExit()) {
          TransmitCapsule tc = pc.getCapsule();

          tc.setOver(null);
          tc.setPrev(null);

          if (log.isDebugEnabled()) {
            log.debug("run(" + tc + ")");
          }

          kernel.sendCapsule(tc);
        }

      }
    } catch (Exception e) {
      LoggerFactory.createLogger().Log(Localization.getLocalization().getString("M000031") + " " + e, LogType.ERROR);
      //e.printStackTrace();
      log.error(e);
    } finally {
      //application was ended
      this.running = false;
      //we say to application manager of end this application.
      interpretManager.interpretingEnded(this);
    }
  }

  /**
   * An Implementation of interface's method
   */
  public void runProcess(Process executable) {
    this.executed = executable;
    if (executable instanceof ProcessApplication) {
      attributes = ((ProcessApplication) executable).getArgs();
    } else if (executable instanceof ProcessCapsule) {
      data.add(((ProcessCapsule) executable).getCapsule());
    }
    this.start();
  }

  /**
   * Terminate code interpreting and return it with its context. Return null,
   * if nothing is interpreted.
   */
  public Process terminate() {

    if (this.running) {
      this.running = false;
      return this.executed;
    } else {
      return null;
    }
  }

  /**
   * Return program code with its context, but interpreting is not stopped.
   */
  public Process getProcess() {

//		if (this.running) {
    return this.executed;
//		}
//		else {
//			return null;
//		}
  }

  /**
   * Return true if interpreter interpret some code. Return false if
   * interpreter has nothing to do.
   */
  public boolean isInterpreting() {

    return this.running;
  }
}
