/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.logging;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * <b>Title:</b> Smart Active Node<br>
 * <b>Description:</b> This class provide logging into file.<br>
 * @author Michal Rejda
 * @since 27.9.2007
 * @version 1.0
 */
public class FileLogger implements ILogger {

	private String logFile;
	private LogType level;

	/**
	 * Construct File logger.
	 * @param logFile File, into the logger write the log.
	 * @param level log level
	 */
	FileLogger(String logFile, LogType level) {
		this.logFile = logFile;
		this.level = level;
	}

	/**
	 * @see san.logging.ILogger#Log(java.lang.String,
	 *      san.logging.ILogger.LogType)
	 */
	public synchronized void Log(Object message, LogType type) {
		// logging only the worst case
		if (level.compareTo(type) > 0)
			return;
		// creating log message
		GregorianCalendar gc = new GregorianCalendar();
		String logRecort = gc.get(Calendar.DAY_OF_MONTH) + "."
				+ (gc.get(Calendar.MONTH) + 1) + "." + gc.get(Calendar.YEAR)
				+ " " + gc.get(Calendar.HOUR_OF_DAY) + ":"
				+ gc.get(Calendar.MINUTE) + ":" + gc.get(Calendar.SECOND) + "."
				+ gc.get(Calendar.MILLISECOND) + " " + type + "" + "\n"
				+ message + "\n---\n\n" + "";
		try {
			// write message into file
			BufferedWriter out = new BufferedWriter(new FileWriter(
					this.logFile, true));
			out.write(logRecort);
			out.close();
		}
		catch (IOException e) {
			System.err.println("Error by using log file: " + this.logFile);
		}
	}

	public LogType getLogLevel() {
		return level;
	}

	public void setLogLevel(LogType level) {
		this.level = level;
	}
}
