/*************************************************************************
 *                           Smart Active Node                           * 
 *                 copyright (c) since 2007 KIV-FAV-ZCU                  *
 *************************************************************************
 * The license for this code is included in LICENSE.txt.  If you did not *
 * get this file with this distribution, request a copy from:            *
 * txkoutny@kiv.zcu.cz                                                   *
 *************************************************************************/
package san.networkInterfaces;

import san.data.TransmitCapsule;
import java.util.Collection;
import java.util.LinkedList;

/**
 * <b>Title:</b> Smart Active Node <br />
 * <b>Description:</b> Abstract network output.<br />
 * @author Jan Boháč, Michal Rejda
 * @since 3.11.2006
 * @version 1.1
 */
public abstract class NetOut {

	/**
	 * @uml.property name="connectionLostListeners"
	 * @uml.associationEnd multiplicity="(0 -1)" aggregation="shared"
	 *                     inverse="netOut:san.networkInterfaces.IConnectionLostListener"
	 */
	private Collection<IConnectionLostListener> connectionLostListeners;
	/**
	 * @uml.property name="remoteInterface"
	 * @uml.associationEnd multiplicity="(1 1)"
	 *                     inverse="netOut:san.networkInterfaces.RemoteInterface"
	 */
	RemoteInterface remoteInterface = null;

	NetOut(RemoteInterface remoteInterface) {
		this.remoteInterface = remoteInterface;
		this.connectionLostListeners = new LinkedList<IConnectionLostListener>();
	}

	/**
	 * Sends the transmitCapsule.
	 * @param bc transmitCapsule to send
	 */
	public abstract void send(TransmitCapsule bc);

	/**
	 * Add new connection lost listener.
	 */
	public void addConnectionLostListener(IConnectionLostListener conLostList) {
		this.connectionLostListeners.add(conLostList);
	}

	/**
	 * Remove connection lost listener.
	 */
	public void removeConnectionLostListener(IConnectionLostListener conLostList) {
		this.connectionLostListeners.remove(conLostList);
	}

	/**
	 * Fire connection lost listeners.
	 */
	void fireConnectionLostListeners() {
		for (IConnectionLostListener listener : connectionLostListeners) {
			listener.connectionLost(this, remoteInterface);
		}
	}

	/**
	 * Getter of the property <tt>remoteInterface</tt>
	 * @return Returns the remoteInterface.
	 * @uml.property name="remoteInterface"
	 */
	public RemoteInterface getRemoteInterface() {
		return remoteInterface;
	}

	/**
	 * Setter of the property <tt>remoteInterface</tt>
	 * @param remoteInterface The remoteInterface to set.
	 * @uml.property name="remoteInterface"
	 */
	public void setRemoteInterface(RemoteInterface remoteInterface) {
		this.remoteInterface = remoteInterface;
	}
}
