/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package san.security;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.util.*;

/**
 *
 * @author Vladimír Aubrecht
 */
class Authentication
{
    private class UserData
    {
        String username;
        String password;

        HashMap<String, RoleData> rolesTable = new HashMap<String, RoleData>();

        boolean logged = false;

        public UserData(String username, String password)
        {
            this.username = username;
            this.password = password;
        }

        public String getUsername()
        {
            return this.username;
        }

        public String getPassword()
        {
            return this.password;
        }

        public boolean isLogged()
        {
            return this.logged;
        }

        public void logUser()
        {
            this.logged = true;
        }

        public void addUserToRole(RoleData role)
        {
            if (!rolesTable.containsKey(role.getRoleName()))
                rolesTable.put(role.getRoleName(), role);
        }

        public HashMap<String, RoleData> getUserRoles()
        {
            return rolesTable;
        }

    }

    public class AccessData
    {
        String name;
        String type;
        String allow;

        public AccessData(String name, String type, String allow)
        {
            this.name = name;
            this.type = type;
            this.allow = allow;
        }

        public String getName()
        {
            return this.name;
        }

        public String getType()
        {
            return this.type;
        }

        public String getAllow()
        {
            return this.allow;
        }

    }

    public class ResourceProfile
    {
        String name;
        int priority;
        
        public ResourceProfile(String name, int priority)
        {
            this.name = name;
            this.priority = priority;
        }
        
        public String getName()
        {
            return this.name;
        }
        
        public int getPriority()
        {
            return this.priority;
        }
    }

    public class RoleData
    {
        String rolename;
        HashMap<String, UserData> users;
        HashMap<String, UserData> usersInRole = new HashMap<String, UserData>();
        HashMap<String, AccessData> permissions = new HashMap<String, AccessData>();
        ResourceProfile resourceProfile;
        
        public RoleData(String rolename, HashMap<String, UserData> users, ResourceProfile resourceProfile)
        {
            this.rolename = rolename;
            this.users = users;
            this.resourceProfile = resourceProfile;
        }

        public void addUser(String username)
        {
            if (this.users != null)
            {
                if (this.users.containsKey(username))
                {
                    usersInRole.put(username, users.get(username));
                }
            }
        }

        public boolean isUserInRole(String username)
        {
            if (this.users != null)
            {
                if (this.users.containsKey(username))
                {
                    return usersInRole.containsKey(username);
                }
            }

            return false;
        }

        public String getRoleName()
        {
            return this.rolename;
        }

        public void addAccess(AccessData accessData)
        {
                //java.util.logging.Logger.getLogger(Authentication.class.getName()).log(java.util.logging.Level.SEVERE, "Rolename " + this.getRoleName() + " add access to: " + accessData.getName()+accessData.type);
                permissions.put(accessData.getName()+accessData.type, accessData);
        }

        public boolean isMethodPermitted(String methodName)
        {
            return isPermitted(methodName+"method");
        }

        public boolean isProgramPermitted(String programName)
        {
            return isPermitted(programName+"program");
        }

        private boolean isPermitted(String name)
        {
            //java.util.logging.Logger.getLogger(Authentication.class.getName()).log(java.util.logging.Level.SEVERE, "Rolename " + this.getRoleName());
            
            Boolean result = isPermittedTest(name);

            if (result != null)
                return result;
            
            if (name.endsWith("program"))
                name = "*program";
            else if (name.endsWith("method"))
                name = "*method";

            result = isPermittedTest(name);
            if (result != null)
                return result;

            result = isPermittedTest("**");
            if (result != null)
                return result;

            //java.util.logging.Logger.getLogger(Authentication.class.getName()).log(java.util.logging.Level.SEVERE, "general permit for " + name);
            return true;
        }

        private Boolean isPermittedTest(String name)
        {
            if (permissions.containsKey(name))
            {
                AccessData data = permissions.get(name);

                if (data.getAllow().equals("permit"))
                {
                    //java.util.logging.Logger.getLogger(Authentication.class.getName()).log(java.util.logging.Level.SEVERE, "Permit for " + name);
                    return true;
                }
                else if (data.getAllow().equals("forbidden"))
                {
                    //java.util.logging.Logger.getLogger(Authentication.class.getName()).log(java.util.logging.Level.SEVERE, "Forbidden for " + name);
                    return false;
                }
            }

            return null;
        }

    }

    public class ResourcesData
    {
        int maxCapsules = 0;
        int maxActiveCodes = 0;

        HashMap<String, ResourceProfile> resourceProfiles = new HashMap<String, ResourceProfile>();

        public ResourcesData(int maxCapsules, int maxActiveCodes)
        {
            this.maxCapsules = maxCapsules;
            this.maxActiveCodes = maxActiveCodes;
        }

        public void addResourceProfile(ResourceProfile profile)
        {
            if (!this.resourceProfiles.containsKey(profile.getName()))
                resourceProfiles.put(profile.getName(), profile);
        }

        public ResourceProfile getResourceProfile(String name)
        {
            return resourceProfiles.get(name);
        }

        public int getMaxCapsules()
        {
            return this.maxCapsules;
        }

        public int getMaxActiveCodes()
        {
            return this.maxActiveCodes;
        }
    }

    private static Authentication instance = null;

    private static final String SecurityTableFilename = "conf/SecurityTable.xml";

    Document document = null;

    HashMap<String, UserData> usersTable;
    HashMap<String, RoleData> rolesTable;
    ResourcesData resourcesData;


    private Authentication()
    {
        this.usersTable = new HashMap<String, UserData>();
        this.rolesTable = new HashMap<String, RoleData>();

        loadFile();
        parseFile();
    }

    public static Authentication getInstance()
    {
        if (instance == null)
            instance = new Authentication();

        return instance;
    }

    private void loadFile()
    {
        if (SecurityMonitor.isSecurityDisabled())
            return;

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        try
        {
            DocumentBuilder db = dbf.newDocumentBuilder();
            try
            {
                document = db.parse(SecurityTableFilename);

            }
            catch (SAXException ex)
            {
                Logger.getLogger(SecurityMonitor.class.getName()).log(Level.SEVERE, null, ex);
            }
            catch (IOException ex)
            {
                Logger.getLogger(SecurityMonitor.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        catch (ParserConfigurationException ex)
        {
            Logger.getLogger(SecurityMonitor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void parseFile()
    {
        if (SecurityMonitor.isSecurityDisabled())
            return;

        createUserTable();
        loadResourceProfiles();
        createRoleTable();
    }

    private void createUserTable()
    {
        usersTable.clear();

        Element rootElement = document.getDocumentElement();

        NodeList users = ((Element)rootElement.getElementsByTagName("users").item(0)).getElementsByTagName("user");

        for (int i = 0; i < users.getLength(); i++)
        {
            Element user = (Element)users.item(i);

            String login = user.getAttribute("login");
            String password = user.getAttribute("password");
            usersTable.put(login, new UserData(login, password));
        }
    }

    private void createRoleTable()
    {
        rolesTable.clear();

        Element rootElement = document.getDocumentElement();

        NodeList roles = ((Element)rootElement.getElementsByTagName("roles").item(0)).getElementsByTagName("role");

        for (int i = 0; i <  roles.getLength(); i++)
        {
            Element role = (Element) roles.item(i);

            String roleName =  role.getAttribute("name");
            String resourceProfileName = role.getAttribute("resourceProfile");
            
            RoleData roleData = new RoleData(roleName, usersTable, resourcesData.getResourceProfile(resourceProfileName));

            loadPermissionsForRole(role, roleData);
            loadUsersForRole(role, roleData);

            rolesTable.put(roleName, roleData);
        }
    }

    private void loadResourceProfiles()
    {
        Element rootElement = document.getDocumentElement();

        Element resourceProfilesElement = (Element)rootElement.getElementsByTagName("resourceProfiles").item(0);
        NodeList profiles = resourceProfilesElement.getElementsByTagName("profile");

        int maxCapsules = Integer.parseInt(resourceProfilesElement.getAttribute("maxCapsules"));
        int maxActiveCodes = Integer.parseInt(resourceProfilesElement.getAttribute("maxActiveCodes"));

        resourcesData = new ResourcesData(maxCapsules, maxActiveCodes);

        for (int i = 0; i <  profiles.getLength(); i++)
        {
            Element profileElement = (Element) profiles.item(i);

            String name = profileElement.getAttribute("name");
            int priority = Integer.parseInt(profileElement.getAttribute("priority"));

            ResourceProfile profile = new ResourceProfile(name, priority);
            resourcesData.addResourceProfile(profile);
        }
    }

    private void loadPermissionsForRole(Element role, RoleData roleData)
    {
        NodeList permissions = role.getElementsByTagName("permissions");

        for (int i = 0; i < permissions.getLength(); i++)
        {
            Element permissionsNode = (Element)permissions.item(i);

            NodeList permissionElements = permissionsNode.getElementsByTagName("access");

            for (int t = 0; t < permissionElements.getLength(); t++)
            {
                Element permission = (Element)permissionElements.item(t);

                String permissionType = permission.getAttribute("type");
                String permissionName = permission.getAttribute("name");
                String permissionAllow = permission.getAttribute("allow");

                AccessData accessData = new AccessData(permissionName, permissionType, permissionAllow);
                roleData.addAccess(accessData);
            }
        }
    }

    private void loadUsersForRole(Element role, RoleData roleData)
    {
        NodeList users = role.getElementsByTagName("users");

        for (int i = 0; i < users.getLength(); i++)
        {
            Element usersNode = (Element)users.item(i);

            NodeList usersElements = usersNode.getElementsByTagName("user");

            for (int t = 0; t < usersElements.getLength(); t++)
            {
                Element user = (Element)usersElements.item(t);

                String username = user.getAttribute("login");

                if (isUserValid(username))
                {
                    roleData.addUser(username);
                    this.usersTable.get(username).addUserToRole(roleData);
                }
            }
        }
    }

    public boolean isInitSucefully()
    {
        return this.document != null;
    }

    public boolean Authenticate(String username, String password)
    {
        if (usersTable.containsKey(username))
        {
            UserData data = usersTable.get(username);

            if (data.getPassword().equals(password))
            {
                data.logUser();
                return data.isLogged();
            }

            return data.isLogged();
        }

        return false;
    }

    public boolean isUserValid(String username)
    {
        return this.usersTable.containsKey(username);
    }

    public HashMap<String, RoleData> getUserRoles(String username)
    {
        return this.usersTable.get(username).getUserRoles();
    }

}
