/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package san.security;

import java.util.*;



/**
 *
 * @author Vladimír Aubrecht
 */
public class SecurityMonitor {

        public static final String NoPermissionError = "You have no permission for this!";
        public static final String AnonymousUser = "anonymous";

        private static final boolean SecurityDisabled = true;
        private static SecurityMonitor instance = null;
        private static Authentication authenticationInstance;
        private static ResourceMonitor resourceMonitorInstance;

        public enum UseType {Method, Program}

        private SecurityMonitor()
        {
            authenticationInstance = Authentication.getInstance();
            resourceMonitorInstance = ResourceMonitor.getInstance();
        }

        public static SecurityMonitor getInstance()
        {
            if (instance == null)
                instance = new SecurityMonitor();

            return instance;
        }

        public static boolean isSecurityDisabled()
        {
            return SecurityDisabled;
        }

        public boolean log(String username, String password)
        {
            if (isSecurityDisabled())
                return true;

            return authenticationInstance.Authenticate(username, password);
        }

        public boolean isUserValid(String username)
        {
            if (isSecurityDisabled())
                return true;

            return authenticationInstance.isUserValid(username);
        }

        public boolean canUserUseCurrentMethod(String username)
        {

            StackTraceElement[] elements = Thread.currentThread().getStackTrace();

            String methodName = elements[2].getMethodName();

            //for (int t = 0; t < elements.length; t++)
            //java.util.logging.Logger.getLogger(SecurityMonitor.class.getName()).log(java.util.logging.Level.SEVERE, "CanUserUse: " + elements[t]);

            return canUserUse(username, methodName, UseType.Method);
        }

        public boolean canUserUse(String username, String name, UseType type)
        {
            if (isSecurityDisabled())
                return true;

            if (authenticationInstance.isUserValid(username))
            {
                Collection<Authentication.RoleData> roles = authenticationInstance.getUserRoles(username).values();

                boolean permitted = false;
                for (Authentication.RoleData role : roles)
                {
                    if (type == UseType.Method)
                        permitted |= role.isMethodPermitted(name);
                    else if (type == UseType.Program)
                        permitted |= role.isProgramPermitted(name);
                }

                //java.util.logging.Logger.getLogger(SecurityMonitor.class.getName()).log(java.util.logging.Level.SEVERE, "CanUserUse: " + Boolean.toString(permitted));
                return permitted;
            }
            else return false;
        }
}
